@@ -27,7 +27,7 @@ static size_t readbuflen=0;
 static size_t readbufmaxsize=(ASYNC_BUF_LEN*2)+32;
 
 static char *writebuf=NULL;
-static size_t writebuflen=0;
+size_t writebuflen=0;
 static size_t writebufmaxsize=(ASYNC_BUF_LEN*2)+32;
 
 int status_wfd=-1; // for the child to send information to the parent.
@@ -59,7 +59,7 @@ static int parse_readbuf(char *cmd, char **dest, size_t *rlen)
 		*cmd=cmdtmp;
 		if(!(*dest=(char *)malloc(s+1)))
 		{
-			logp("out of memory in parse_readbuf\n");
+			log_out_of_memory(__FUNCTION__);
 			truncate_buf(&readbuf, &readbuflen);
 			return -1;
 		}
@@ -88,7 +88,7 @@ static int async_alloc_buf(char **buf, size_t *buflen, size_t bufmaxsize)
 	{
 		if(!(*buf=(char *)malloc(bufmaxsize)))
 		{
-			logp("out of memory in async_alloc_buf\n");
+			log_out_of_memory(__FUNCTION__);
 			return -1;
 		}
 		truncate_buf(buf, buflen);
@@ -381,7 +381,7 @@ int async_rw(char *rcmd, char **rdst, size_t *rlen, char wcmd, const char *wsrc,
 
         if(doread || dowrite)
         {
-//printf("async_rw loop read %d write %d wbuflen: %d\n", doread, dowrite, writebuflen);
+//logp("async_rw loop read %d write %d wbuflen: %d\n", doread, dowrite, writebuflen);
                 mfd=-1;
 
                 if(doread) FD_ZERO(&fsr);
@@ -399,7 +399,8 @@ int async_rw(char *rcmd, char **rdst, size_t *rlen, char wcmd, const char *wsrc,
                 {
                         if(errno!=EAGAIN && errno!=EINTR)
                         {
-                                logp("select error: %s\n", strerror(errno));
+                                logp("select error in %s: %s\n", __func__,
+					strerror(errno));
                                 return -1;
                         }
                 }
@@ -504,7 +505,7 @@ int async_write_str(char wcmd, const char *wsrc)
 int async_read_expect(char cmd, const char *expect)
 {
 	int ret=0;
-	char rcmd;
+	char rcmd=0;
 	char *rdst=NULL;
 	size_t rlen=0;
 	if(async_read(&rcmd, &rdst, &rlen)) return -1;
@@ -589,6 +590,14 @@ void log_and_send(const char *msg)
 	if(fd>0) async_write_str(CMD_ERROR, msg);
 }
 
+void log_and_send_oom(const char *function)
+{
+	char m[256]="";
+	snprintf(m, sizeof(m), "out of memory in %s()\n", __FUNCTION__);
+	logp("%s", m);
+	if(fd>0) async_write_str(CMD_ERROR, m);
+}
+
 int async_get_fd(void)
 {
 	return fd;
@@ -654,6 +663,7 @@ int async_read_stat(FILE *fp, gzFile zp, struct sbuf *sb, struct cntr *cntr)
 		  || (cmd==CMD_GEN && !strcmp(buf, "backupphase2"))
 		  || (cmd==CMD_GEN && !strcmp(buf, "estimateend")))
 		{
+			if(buf) free(buf);
 			if(d) free(d);
 			return 1;
 		}
@@ -678,7 +688,11 @@ int logw(struct cntr *cntr, const char *fmt, ...)
 	va_start(ap, fmt);
 	vsnprintf(buf, sizeof(buf), fmt, ap);
 	if(doing_estimate) printf("\nWARNING: %s\n", buf);
-	else r=async_write_str(CMD_WARNING, buf);
+	else
+	{
+		r=async_write_str(CMD_WARNING, buf);
+		logp("WARNING: %s\n", buf);
+	}
 	va_end(ap);
 	do_filecounter(cntr, CMD_WARNING, 1);
 	return r;
