@@ -10,6 +10,8 @@
 extern int status_wfd; // for the child to send information to the parent.
 extern int status_rfd; // for the child to read information from the parent.
 
+extern size_t writebuflen;
+
 extern int async_init(int afd, SSL *assl, struct config *conf, int estimate);
 
 extern void async_free(void);
@@ -40,6 +42,7 @@ extern int async_write_str(char wcmd, const char *wsrc);
 extern int async_read_expect(char cmd, const char *expect);
 
 extern void log_and_send(const char *msg);
+extern void log_and_send_oom(const char *function);
 
 // Read from fp if given, is read from our fd.
 extern int async_read_stat(FILE *fp, gzFile zp, struct sbuf *sb, struct cntr *cntr);
