@@ -176,14 +176,14 @@ void decode_stat(const char *buf, struct stat *statp, int64_t *winattr, int *com
    }
 }
 
-static int set_file_times(const char *path, struct utimbuf *ut, struct stat *statp, struct cntr *cntr)
+static int set_file_times(const char *path, struct utimbuf *ut, struct cntr *cntr)
 {
 	int e;
-// The mingw64 utime() appears not to work on directories.
+// The mingw64 utime() appears not to work on read-only files.
 // Use the utime() from bacula instead.
 #ifdef HAVE_WIN32
-	if(S_ISDIR(statp->st_mode)) e=win32_utime(path, ut);
-	else e=utime(path, ut);
+	//e=utime(path, ut);
+	e=win32_utime(path, ut);
 #else
 	e=utime(path, ut);
 #endif
@@ -207,7 +207,7 @@ bool set_attributes(const char *path, char cmd, struct stat *statp, int64_t wina
 
 #ifdef HAVE_WIN32
 	win32_chmod(path, statp->st_mode, winattr);
-	set_file_times(path, &ut, statp, cntr);
+	set_file_times(path, &ut, cntr);
 	return true;
 #endif
 
@@ -242,7 +242,7 @@ bool set_attributes(const char *path, char cmd, struct stat *statp, int64_t wina
          ok = false;
       }
 
-      if(set_file_times(path, &ut, statp, cntr)) ok=false;
+      if(set_file_times(path, &ut, cntr)) ok=false;
 #ifdef HAVE_CHFLAGS
       /*
        * FreeBSD user flags
