@@ -32,17 +32,10 @@ static int check_passwd(const char *passwd, const char *plain_text)
 
 static int check_client_and_password(struct config *conf, const char *client, const char *password, struct config *cconf)
 {
-	char cpath[256]="";
-
-	// Some client settings can be globally set in the server config and
-	// overridden in the client specific config.
 	// Cannot load it until here, because we need to have the name of the
 	// client.
-	init_config(cconf);
-	snprintf(cpath, sizeof(cpath), "%s/%s", conf->clientconfdir, client);
-	if(set_client_global_config(conf, cconf, client)
-	  || load_config(cpath, cconf, FALSE))
-		return -1;
+	if(load_client_config(conf, cconf, client)) return -1;
+
 	if(!cconf->ssl_peer_cn)
 	{
 		logp("ssl_peer_cn unset");
@@ -51,28 +44,31 @@ static int check_client_and_password(struct config *conf, const char *client, co
 			logp("Falling back to using '%s'\n", client);
 			if(!(cconf->ssl_peer_cn=strdup(client)))
 			{
-				logp("out of memory\n");
+				log_out_of_memory(__FUNCTION__);
 				return -1;
 			}
 		}
 	}
 
-	if(!cconf->password && !cconf->passwd)
+	if(cconf->password_check)
 	{
-		logp("password rejected for client %s\n", client);
-		return -1;
-	}
-	// check against plain text
-	if(cconf->password && strcmp(cconf->password, password))
-	{
-		logp("password rejected for client %s\n", client);
-		return -1;
-	}
-	// check against encypted passwd
-	if(cconf->passwd && !check_passwd(cconf->passwd, password))
-	{
-		logp("password rejected for client %s\n", client);
-		return -1;
+		if(!cconf->password && !cconf->passwd)
+		{
+			logp("password rejected for client %s\n", client);
+			return -1;
+		}
+		// check against plain text
+		if(cconf->password && strcmp(cconf->password, password))
+		{
+			logp("password rejected for client %s\n", client);
+			return -1;
+		}
+		// check against encypted passwd
+		if(cconf->passwd && !check_passwd(cconf->passwd, password))
+		{
+			logp("password rejected for client %s\n", client);
+			return -1;
+		}
 	}
 
 	if(!cconf->keep)
@@ -173,9 +169,11 @@ int authorise_server(struct config *conf, char **client, char **cversion, struct
 		return -1;
 	}
 
-	version_warn(p1cntr, *client, *cversion);
+	if(cconf->version_warn)
+		version_warn(p1cntr, *client, *cversion);
 
-	logp("auth ok for client: %s\n", *client);
+	logp("auth ok for: %s%s\n", *client,
+		cconf->password_check?"":" (no password needed)");
 	if(password) free(password);
 
 	async_write_str(CMD_GEN, "ok");
