@@ -22,6 +22,7 @@ static int receive_file(const char *autoupgrade_dir, const char *file, struct cn
 
 int autoupgrade_client(struct config *conf, struct cntr *p1cntr)
 {
+	int a=0;
 	int ret=-1;
 	char *cp=NULL;
 	char *copy=NULL;
@@ -32,6 +33,7 @@ int autoupgrade_client(struct config *conf, struct cntr *p1cntr)
 	char script_name[32]="";
 	char package_name[32]="";
 	char write_str[256]="";
+	const char *args[2];
 
 	if(!conf->autoupgrade_dir)
 	{
@@ -45,7 +47,7 @@ int autoupgrade_client(struct config *conf, struct cntr *p1cntr)
 	}
 	if(!(copy=strdup(conf->autoupgrade_dir)))
 	{
-		logp("out of memory\n");
+		log_out_of_memory(__FUNCTION__);
 		goto end;
 	}
 	// strip trailing slash
@@ -83,7 +85,7 @@ int autoupgrade_client(struct config *conf, struct cntr *p1cntr)
 	}
 
 #ifdef HAVE_WIN32
-	win32_enable_backup_privileges(1 /* ignore_errors */);
+	win32_enable_backup_privileges();
 	snprintf(script_name, sizeof(script_name), "script.bat");
 	snprintf(package_name, sizeof(package_name), "package.exe");
 #else
@@ -110,11 +112,11 @@ int autoupgrade_client(struct config *conf, struct cntr *p1cntr)
 	chmod(script_path, 0755);
 
 	/* Run the script here. */
-	ret=run_script(script_path,
-		NULL, 0, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
-		NULL, NULL,
-		p1cntr,
-		0 /* do not wait */, 1 /* use logp */);
+	a=0;
+	args[a++]=script_path;
+	args[a++]=NULL;
+	ret=run_script(args, NULL, 0,
+		p1cntr, 0 /* do not wait */, 1 /* use logp */);
 	/* To get round Windows problems to do with installing over files
 	   that the current process is running from, I am forking the child,
 	   then immediately exiting the parent process. */
