@@ -10,7 +10,12 @@
 #include "backup_phase1_client.h"
 
 static char filesymbol=CMD_FILE;
+#ifdef HAVE_WIN32
+static char metasymbol=CMD_VSS;
+static char vss_trail_symbol=CMD_VSS_T;
+#else
 static char metasymbol=CMD_METADATA;
+#endif
 
 static int maybe_send_extrameta(const char *path, char cmd, const char *attribs, struct cntr *p1cntr)
 {
@@ -31,7 +36,10 @@ int send_file(FF_PKT *ff, bool top_level, struct config *conf, struct cntr *p1cn
 
    if(!file_is_included(conf->incexcdir, conf->iecount,
 	conf->incext, conf->incount,
-	conf->excext, conf->excount, ff->fname, top_level)) return 0;
+	conf->excext, conf->excount,
+	conf->increg, conf->ircount,
+	conf->excreg, conf->ercount,
+	ff->fname, top_level)) return 0;
 #ifdef HAVE_WIN32
 // Useful Windows attributes debug
 /*
@@ -90,6 +98,7 @@ if(ff->winattr & FILE_ATTRIBUTE_VIRTUAL) printf("virtual\n");
    //logp("%d: %s\n", ff->type, ff->fname);
 
    switch (ff->type) {
+#ifndef HAVE_WIN32
    case FT_LNKSAVED:
         //printf("Lnka: %s -> %s\n", ff->fname, ff->link);
    	encode_stat(attribs, &ff->statp, ff->winattr, conf->compression);
@@ -101,23 +110,47 @@ if(ff->winattr & FILE_ATTRIBUTE_VIRTUAL) printf("virtual\n");
 	// At least FreeBSD 8.2 can have different xattrs on hard links.
 	if(maybe_send_extrameta(ff->fname, CMD_HARD_LINK, attribs, p1cntr))
 		return -1;
+#endif
       break;
+   case FT_RAW:
    case FT_FIFO:
    case FT_REGE:
    case FT_REG:
-      encode_stat(attribs, &ff->statp, ff->winattr, conf->compression);
+      encode_stat(attribs, &ff->statp, ff->winattr,
+		in_exclude_comp(conf->excom, conf->excmcount,
+			ff->fname, conf->compression));
+#ifdef HAVE_WIN32
+      if(conf->split_vss && !conf->strip_vss
+	&& maybe_send_extrameta(ff->fname, filesymbol, attribs, p1cntr))
+		return -1;
+#endif
       if(async_write_str(CMD_STAT, attribs)
 	|| async_write_str(filesymbol, ff->fname))
 		return -1;
       do_filecounter(p1cntr, filesymbol, 1);
       if(ff->type==FT_REG)
 	do_filecounter_bytes(p1cntr, (unsigned long long)ff->statp.st_size);
+#ifdef HAVE_WIN32
+      // Possible trailing VSS meta data
+      if(conf->split_vss && !conf->strip_vss)
+      {
+	if(async_write_str(CMD_STAT, attribs)
+	 || async_write_str(vss_trail_symbol, ff->fname))
+		return -1;
+        do_filecounter(p1cntr, vss_trail_symbol, 1);
+      }
+#else
       if(maybe_send_extrameta(ff->fname, filesymbol, attribs, p1cntr))
 		return -1;
+#endif
       break;
+#ifndef HAVE_WIN32
    case FT_LNK:
 	//printf("link: %s -> %s\n", ff->fname, ff->link);
    	encode_stat(attribs, &ff->statp, ff->winattr, conf->compression);
+        if(conf->split_vss && !conf->strip_vss
+	  && maybe_send_extrameta(ff->fname, CMD_SOFT_LINK, attribs, p1cntr))
+		return -1;
 	if(async_write_str(CMD_STAT, attribs)
 	  || async_write_str(CMD_SOFT_LINK, ff->fname)
 	  || async_write_str(CMD_SOFT_LINK, ff->link))
@@ -125,6 +158,7 @@ if(ff->winattr & FILE_ATTRIBUTE_VIRTUAL) printf("virtual\n");
 	do_filecounter(p1cntr, CMD_SOFT_LINK, 1);
         if(maybe_send_extrameta(ff->fname, CMD_SOFT_LINK, attribs, p1cntr))
 		return -1;
+#endif
       break;
    case FT_DIREND:
       return 0;
@@ -146,12 +180,25 @@ if(ff->winattr & FILE_ATTRIBUTE_VIRTUAL) printf("virtual\n");
 	 {
 		encode_stat(attribs,
 			&ff->statp, ff->winattr, conf->compression);
-	      	if(async_write_str(CMD_STAT, attribs)) return -1;
-#if defined(WIN32_VSS)
-		if(async_write_str(filesymbol, ff->fname)) return -1;
-		do_filecounter(p1cntr, filesymbol, 1);
+#ifdef HAVE_WIN32
+		if(conf->split_vss || conf->strip_vss)
+		{
+			if(!conf->strip_vss
+			  && maybe_send_extrameta(ff->fname,
+				CMD_DIRECTORY, attribs, p1cntr)) return -1;
+	      		if(async_write_str(CMD_STAT, attribs)) return -1;
+			if(async_write_str(CMD_DIRECTORY, ff->fname)) return -1;
+			do_filecounter(p1cntr, CMD_DIRECTORY, 1);
+		}
+		else
+		{
+	      		if(async_write_str(CMD_STAT, attribs)) return -1;
+			if(async_write_str(filesymbol, ff->fname)) return -1;
+			do_filecounter(p1cntr, filesymbol, 1);
+		}
 #else
-		if(async_write_str(CMD_DIRECTORY, ff->fname)) return -1;
+	      	if(async_write_str(CMD_STAT, attribs)
+		  || async_write_str(CMD_DIRECTORY, ff->fname)) return -1;
 		do_filecounter(p1cntr, CMD_DIRECTORY, 1);
         	if(maybe_send_extrameta(ff->fname, CMD_DIRECTORY,
 			attribs, p1cntr)) return -1;
@@ -159,6 +206,7 @@ if(ff->winattr & FILE_ATTRIBUTE_VIRTUAL) printf("virtual\n");
 	 }
 	}
       break;
+#ifndef HAVE_WIN32
    case FT_SPEC: // special file - fifo, socket, device node...
       encode_stat(attribs, &ff->statp, ff->winattr, conf->compression);
       if(async_write_str(CMD_STAT, attribs)
@@ -167,6 +215,7 @@ if(ff->winattr & FILE_ATTRIBUTE_VIRTUAL) printf("virtual\n");
       do_filecounter(p1cntr, CMD_SPECIAL, 1);
       if(maybe_send_extrameta(ff->fname, CMD_SPECIAL, attribs, p1cntr))
 		return -1;
+#endif
       break;
    case FT_NOACCESS:
       logw(p1cntr, _("Err: Could not access %s: %s"), ff->fname, strerror(errno));
@@ -186,9 +235,6 @@ if(ff->winattr & FILE_ATTRIBUTE_VIRTUAL) printf("virtual\n");
    case FT_NOOPEN:
       logw(p1cntr, _("Err: Could not open directory %s: %s"), ff->fname, strerror(errno));
       break;
-   case FT_RAW:
-      logw(p1cntr, _("Err: Raw partition: %s"), ff->fname);
-      break;
    default:
       logw(p1cntr, _("Err: Unknown file ff->type %d: %s"), ff->type, ff->fname);
       break;
@@ -209,7 +255,12 @@ int backup_phase1_client(struct config *conf, int estimate, struct cntr *p1cntr,
 	if(conf->encryption_password)
 	{
 		filesymbol=CMD_ENC_FILE;
+#ifdef HAVE_WIN32
+		metasymbol=CMD_ENC_VSS;
+		vss_trail_symbol=CMD_ENC_VSS_T;
+#else
 		metasymbol=CMD_ENC_METADATA;
+#endif
 	}
 
 	ff=init_find_files();
