@@ -35,7 +35,7 @@ static int load_signature(rs_signature_t **sumset, struct cntr *cntr)
 	return r;
 }
 
-static int load_signature_and_send_delta(BFILE *bfd, FILE *in, unsigned long long *bytes, unsigned long long *sentbytes, struct cntr *cntr)
+static int load_signature_and_send_delta(BFILE *bfd, FILE *in, unsigned long long *bytes, unsigned long long *sentbytes, struct cntr *cntr, size_t datalen)
 {
 	rs_job_t *job;
 	rs_result r;
@@ -57,8 +57,10 @@ static int load_signature_and_send_delta(BFILE *bfd, FILE *in, unsigned long lon
 		return RS_IO_ERROR;
 	}
 
-	if(!(infb=rs_filebuf_new(bfd, in, NULL, -1, ASYNC_BUF_LEN, cntr))
-	  || !(outfb=rs_filebuf_new(NULL, NULL, NULL, async_get_fd(), ASYNC_BUF_LEN, cntr)))
+	if(!(infb=rs_filebuf_new(bfd,
+		in, NULL, -1, ASYNC_BUF_LEN, datalen, cntr))
+	  || !(outfb=rs_filebuf_new(NULL, NULL,
+		NULL, async_get_fd(), ASYNC_BUF_LEN, -1, cntr)))
 	{
 		logp("could not rs_filebuf_new for delta\n");
 		if(infb) rs_filebuf_free(infb);
@@ -122,14 +124,17 @@ static int load_signature_and_send_delta(BFILE *bfd, FILE *in, unsigned long lon
 	return r;
 }
 
-static int send_whole_file_w(char cmd, const char *fname, const char *datapth, int quick_read, unsigned long long *bytes, const char *encpassword, struct cntr *cntr, int compression, BFILE *bfd, FILE *fp, const char *extrameta, size_t elen)
+static int send_whole_file_w(char cmd, const char *fname, const char *datapth, int quick_read, unsigned long long *bytes, const char *encpassword, struct cntr *cntr, int compression, BFILE *bfd, FILE *fp, const char *extrameta, size_t elen, size_t datalen)
 {
 	if((compression || encpassword) && cmd!=CMD_EFS_FILE)
-		return send_whole_file_gz(fname, datapth, quick_read, bytes, 
-		  encpassword, cntr, compression, bfd, fp, extrameta, elen);
+		return send_whole_file_gz(fname, datapth, quick_read,
+		  bytes, 
+		  encpassword, cntr, compression, bfd, fp, extrameta, elen,
+		  datalen);
 	else
 		return send_whole_file(cmd, fname, datapth, quick_read, bytes, 
-		  cntr, bfd, fp, extrameta, elen);
+		  cntr, bfd, fp, extrameta, elen,
+		  datalen);
 }
 
 static int forget_file(struct sbuf *sb, char cmd, struct cntr *cntr)
@@ -153,7 +158,7 @@ static int forget_file(struct sbuf *sb, char cmd, struct cntr *cntr)
 	return 0;
 }
 
-static int do_backup_phase2_client(struct config *conf, int resume, struct cntr *cntr)
+static int do_backup_phase2_client(struct config *conf, int resume, struct cntr *p1cntr, struct cntr *cntr)
 {
 	int ret=0;
 	int quit=0;
@@ -161,6 +166,16 @@ static int do_backup_phase2_client(struct config *conf, int resume, struct cntr
 	char *buf=NULL;
 	size_t len=0;
 	char attribs[MAXSTRING];
+	// For efficiency, open Windows files for the VSS data, and do not
+	// close them until another time around the loop, when the actual
+	// data is read.
+	BFILE bfd;
+	// Windows VSS headers tell us how much file
+	// data to expect.
+	size_t datalen=0;
+#ifdef HAVE_WIN32
+	binit(&bfd, NULL, 0);
+#endif
 
 	struct sbuf sb;
 
@@ -173,6 +188,13 @@ static int do_backup_phase2_client(struct config *conf, int resume, struct cntr
 		  || async_read_expect(CMD_GEN, "ok"))
 			return -1;
 	}
+	else if(conf->send_client_counters)
+	{
+		// On resume, the server might update the client with the
+ 		// counters.
+		if(recv_counters(p1cntr, cntr))
+			return -1;	
+	}
 
 	while(!quit)
 	{
@@ -204,6 +226,10 @@ static int do_backup_phase2_client(struct config *conf, int resume, struct cntr
 			  || cmd==CMD_ENC_FILE
 			  || cmd==CMD_METADATA
 			  || cmd==CMD_ENC_METADATA
+			  || cmd==CMD_VSS
+			  || cmd==CMD_ENC_VSS
+			  || cmd==CMD_VSS_T
+			  || cmd==CMD_ENC_VSS_T
 			  || cmd==CMD_EFS_FILE)
 			{
 				int forget=0;
@@ -212,8 +238,8 @@ static int do_backup_phase2_client(struct config *conf, int resume, struct cntr
 				char *extrameta=NULL;
 				size_t elen=0;
 				unsigned long long bytes=0;
-				BFILE bfd;
 				FILE *fp=NULL;
+				int compression=conf->compression;
 
 				sb.path=buf;
 				buf=NULL;
@@ -235,29 +261,41 @@ static int do_backup_phase2_client(struct config *conf, int resume, struct cntr
 				}
 
 				if(conf->min_file_size
-				  && statbuf.st_size<(boffset_t)conf->min_file_size)
+				  && statbuf.st_size<
+					(boffset_t)conf->min_file_size
+				  && (cmd==CMD_FILE
+				  || cmd==CMD_ENC_FILE
+				  || cmd==CMD_EFS_FILE))
 				{
-					logw(cntr, "File size decreased below min_file_size after initial scan: %s", sb.path);
+					logw(cntr, "File size decreased below min_file_size after initial scan: %c:%s", cmd, sb.path);
 					forget++;
 				}
 				else if(conf->max_file_size
-				  && statbuf.st_size>(boffset_t)conf->max_file_size)
+				  && statbuf.st_size>
+					(boffset_t)conf->max_file_size
+				  && (cmd==CMD_FILE
+				  || cmd==CMD_ENC_FILE
+				  || cmd==CMD_EFS_FILE))
 				{
-					logw(cntr, "File size increased above max_file_size after initial scan: %s", sb.path);
+					logw(cntr, "File size increased above max_file_size after initial scan: %c:%s", cmd, sb.path);
 					forget++;
 				}
 
 				if(!forget)
 				{
+					compression=in_exclude_comp(conf->excom,
+					  conf->excmcount, sb.path,
+					  conf->compression);
 					encode_stat(attribs,
-					  &statbuf, winattr, conf->compression);
+					  &statbuf, winattr, compression);
 					if(open_file_for_send(
 #ifdef HAVE_WIN32
 						&bfd, NULL,
 #else
 						NULL, &fp,
 #endif
-						sb.path, winattr, cntr))
+						sb.path, &statbuf, winattr,
+						&datalen, cntr))
 							forget++;
 				}
 
@@ -273,18 +311,42 @@ static int do_backup_phase2_client(struct config *conf, int resume, struct cntr
 				}
 
 				if(cmd==CMD_METADATA
-				  || cmd==CMD_ENC_METADATA)
+				  || cmd==CMD_ENC_METADATA
+				  || cmd==CMD_VSS
+				  || cmd==CMD_ENC_VSS
+#ifdef HAVE_WIN32
+				  || conf->strip_vss
+#endif
+				  )
 				{
-					if(get_extrameta(sb.path,
+					if(get_extrameta(
+#ifdef HAVE_WIN32
+						&bfd,
+#else
+						NULL,
+#endif
+						sb.path,
 						&statbuf, &extrameta, &elen,
-						cntr))
+						winattr, cntr,
+						&datalen))
 					{
 						logw(cntr, "Meta data error for %s", sb.path);
 						free_sbuf(&sb);
 						close_file_for_send(&bfd, &fp);
 						continue;
 					}
-					if(!extrameta)
+					if(extrameta)
+					{
+#ifdef HAVE_WIN32
+						if(conf->strip_vss)
+						{
+							free(extrameta);
+							extrameta=NULL;
+							elen=0;
+						}
+#endif
+					}
+					else
 					{
 						logw(cntr, "No meta data after all: %s", sb.path);
 						free_sbuf(&sb);
@@ -302,7 +364,8 @@ static int do_backup_phase2_client(struct config *conf, int resume, struct cntr
 					  || async_write_str(CMD_FILE, sb.path)
 					  || load_signature_and_send_delta(
 						&bfd, fp,
-						&bytes, &sentbytes, cntr))
+						&bytes, &sentbytes, cntr,
+						datalen))
 					{
 						logp("error in sig/delta for %s (%s)\n", sb.path, sb.datapth);
 						ret=-1;
@@ -320,14 +383,16 @@ static int do_backup_phase2_client(struct config *conf, int resume, struct cntr
 					//logp("need to send whole file: %s\n",
 					//	sb.path);
 					// send the whole file.
-					if(async_write_str(CMD_STAT, attribs)
-					  || async_write_str(cmd, sb.path)
+
+					if((async_write_str(CMD_STAT, attribs)
+					  || async_write_str(cmd, sb.path))
 					  || send_whole_file_w(cmd, sb.path,
 						NULL, 0, &bytes,
 						conf->encryption_password,
-						cntr, conf->compression,
+						cntr, compression,
 						&bfd, fp,
-						extrameta, elen))
+						extrameta, elen,
+						datalen))
 					{
 						ret=-1;
 						quit++;
@@ -339,7 +404,19 @@ static int do_backup_phase2_client(struct config *conf, int resume, struct cntr
 						do_filecounter_sentbytes(cntr, bytes);
 					}
 				}
-				close_file_for_send(&bfd, &fp);
+#ifdef HAVE_WIN32
+				// If using Windows do not close bfd - it needs
+				// to stay open to read VSS/file data/VSS.
+				// It will get closed either when given a
+				// different file path, or when this function
+				// exits.
+				
+				//if(cmd!=CMD_VSS
+				// && cmd!=CMD_ENC_VSS)
+				//	close_file_for_send(&bfd, NULL);
+#else
+				close_file_for_send(NULL, &fp);
+#endif
 				free_sbuf(&sb);
 				if(extrameta) free(extrameta);
 			}
@@ -366,6 +443,10 @@ static int do_backup_phase2_client(struct config *conf, int resume, struct cntr
 			}
 		}
 	}
+#ifdef HAVE_WIN32
+	// It is possible for a bfd to still be open.
+	close_file_for_send(&bfd, NULL);
+#endif
 	return ret;
 }
 
@@ -375,10 +456,10 @@ int backup_phase2_client(struct config *conf, struct cntr *p1cntr, int resume, s
 
 	logp("Phase 2 begin (send file data)\n");
 
-	ret=do_backup_phase2_client(conf, resume, cntr);
+	ret=do_backup_phase2_client(conf, resume, p1cntr, cntr);
 
 	print_endcounter(cntr);
-	print_filecounters(p1cntr, cntr, ACTION_BACKUP, 1);
+	print_filecounters(p1cntr, cntr, ACTION_BACKUP);
 
 	if(ret) logp("Error in phase 2\n");
 	logp("Phase 2 end (send file data)\n");
