@@ -41,7 +41,11 @@ static int make_rev_sig(const char *dst, const char *sig, const char *endfile, i
 		RS_DEFAULT_STRONG_LEN, NULL, cntr);
 	gzclose_fp(&dstzp);
 	close_fp(&dstfp);
-	close_fp(&sigp);
+	if(close_fp(&sigp))
+	{
+		logp("error closing %s in make_rev_sig\n", sig);
+		return -1;
+	}
 //logp("end of make rev sig\n");
 	return result;
 }
@@ -89,7 +93,11 @@ static int make_rev_delta(const char *src, const char *sig, const char *del, int
 			return -1;
 		}
 		result=rs_delta_gzfile(sumset, srcfp, srczp, NULL, delzp, NULL, cntr);
-		gzclose_fp(&delzp);
+		if(gzclose_fp(&delzp))
+		{
+			logp("error closing %s in make_rev_delta\n", del);
+			result=RS_IO_ERROR;
+		}
 	}
 	else
 	{
@@ -102,7 +110,14 @@ static int make_rev_delta(const char *src, const char *sig, const char *del, int
 			return -1;
 		}
 		result=rs_delta_gzfile(sumset, srcfp, srczp, delfp, NULL, NULL, cntr);
-		close_fp(&delfp);
+		if(close_fp(&delfp))
+		{
+			logp("error closing %s in make_rev_delta\n", del);
+			gzclose_fp(&srczp);
+			close_fp(&srcfp);
+			rs_free_sumset(sumset);
+			return -1;
+		}
 	}
 
 	rs_free_sumset(sumset);
@@ -119,7 +134,7 @@ static int gen_rev_delta(const char *sigpath, const char *deltadir, const char *
 	char *delpath=NULL;
 	if(!(delpath=prepend_s(deltadir, path, strlen(path))))
 	{
-		logp("out of memory\n");
+		log_out_of_memory(__FUNCTION__);
 		ret=-1;
 	}
 	//logp("Generating reverse delta...\n");
@@ -151,18 +166,45 @@ static int gen_rev_delta(const char *sigpath, const char *deltadir, const char *
 	return ret;
 }
 
-static int inflate_or_link_oldfile(const char *oldpath, const char *infpath, int compression, struct config *cconf)
+static int inflate_or_link_oldfile(const char *oldpath, const char *infpath, const char *datadirtmp, const char *datapth, int compression, struct strlist **partials, int pcount, int *is_partial, struct config *cconf)
 {
+	int p=0;
 	int ret=0;
+	char *partial=NULL;
 	struct stat statp;
+	const char *opath=oldpath;
 
-	if(lstat(oldpath, &statp))
+	// First, check whether this is in our list of partial files.
+	// If it is, use the path to the partial file. The delta will apply
+	// to it.
+	for(p=0; p<pcount; p++)
 	{
-		logp("could not lstat %s\n", oldpath);
+		if(!strcmp(partials[p]->path, datapth))
+		{
+			char *tmp=NULL;
+			// It is a partial transfer, it is in "data.tmp/p".
+			// Build the full path to it.
+			if(!(tmp=prepend_s(datadirtmp, "p", strlen("p"))))
+				return -1;
+			partial=prepend_s(tmp, datapth, strlen(datapth));
+			free(tmp);
+			if(!partial) return -1;
+			logp("Dealing with partial resumed file: %s\n",
+				partial);
+			opath=partial;
+			*is_partial=1;
+			break;
+		}
+	}
+
+	if(lstat(opath, &statp))
+	{
+		logp("could not lstat %s\n", opath);
+		if(partial) free(partial);
 		return -1;
 	}
 
-	if(dpth_is_compressed(compression, oldpath))
+	if(dpth_is_compressed(compression, opath))
 	{
 		FILE *source=NULL;
 		FILE *dest=NULL;
@@ -172,6 +214,7 @@ static int inflate_or_link_oldfile(const char *oldpath, const char *infpath, int
 		if(!(dest=open_file(infpath, "wb")))
 		{
 			close_fp(&dest);
+			if(partial) free(partial);
 			return -1;
 		}
 
@@ -180,31 +223,48 @@ static int inflate_or_link_oldfile(const char *oldpath, const char *infpath, int
 			// Empty file - cannot inflate.
 			// just close the destination and we have duplicated a
 			// zero length file.
-			logp("asked to inflate zero length file: %s\n", oldpath);
-			close_fp(&dest);
+			logp("asked to inflate zero length file: %s\n", opath);
+			if(close_fp(&dest))
+			{
+				if(partial) free(partial);
+				logp("error closing %s in %s\n",
+					infpath, __FUNCTION__);
+				return -1;
+			}
 			return 0;
 		}
-		if(!(source=open_file(oldpath, "rb")))
+		if(!(source=open_file(opath, "rb")))
 		{
+			if(partial) free(partial);
 			close_fp(&dest);
 			return -1;
 		}
 		if((ret=zlib_inflate(source, dest))!=Z_OK)
 		logp("zlib_inflate returned: %d\n", ret);
 		close_fp(&source);
-		close_fp(&dest);
+		if(close_fp(&dest))
+		{
+			if(partial) free(partial);
+			logp("error closing %s in %s\n",
+				infpath, __FUNCTION__);
+			return -1;
+		}
 	}
 	else
 	{
 		// If it was not a compressed file, just hard link it.
-		if(do_link(oldpath, infpath, &statp, cconf))
-			ret=-1;
+		// It is possible that infpath already exists, if the server
+		// was interrupted on a previous run just after this point.
+		if(do_link(opath, infpath, &statp, cconf,
+			TRUE /* allow overwrite of infpath */))
+				ret=-1;
 	
 	}
+	if(partial) free(partial);
 	return ret;
 }
 
-static int jiggle(const char *datapth, const char *currentdata, const char *datadirtmp, const char *datadir, const char *deltabdir, const char *deltafdir, const char *sigpath, const char *endfile, int hardlinked, int compression, struct cntr *cntr, struct config *cconf)
+static int jiggle(const char *datapth, const char *currentdata, const char *datadirtmp, const char *datadir, const char *deltabdir, const char *deltafdir, const char *sigpath, const char *endfile, const char *deletionsfile, FILE **delfp, struct sbuf *sb, int hardlinked, int compression, struct strlist **partials, int pcount, struct cntr *cntr, struct config *cconf)
 {
 	int ret=0;
 	struct stat statp;
@@ -218,7 +278,7 @@ static int jiggle(const char *datapth, const char *currentdata, const char *data
 	  || !(finpath=prepend_s(datadir, datapth, strlen(datapth)))
 	  || !(deltafpath=prepend_s(deltafdir, datapth, strlen(datapth))))
 	{
-		logp("out of memory\n");
+		log_out_of_memory(__FUNCTION__);
 		ret=-1;	
 		goto cleanup;
 	}
@@ -254,6 +314,8 @@ static int jiggle(const char *datapth, const char *currentdata, const char *data
 	}
 	else if(!lstat(deltafpath, &statp) && S_ISREG(statp.st_mode))
 	{
+		int lrs;
+		int is_partial=0;
 		char *infpath=NULL;
 
 		// Got a forward patch to do.
@@ -264,39 +326,64 @@ static int jiggle(const char *datapth, const char *currentdata, const char *data
 	  	if(!(infpath=prepend_s(deltafdir,
 			"inflate", strlen("inflate"))))
 		{
-			logp("out of memory\n");
+			log_out_of_memory(__FUNCTION__);
 			ret=-1;
 			goto cleanup;
 		}
 
 		//logp("Fixing up: %s\n", datapth);
-		if(inflate_or_link_oldfile(oldpath, infpath, compression, cconf))
+		if(inflate_or_link_oldfile(oldpath, infpath, datadirtmp,
+			datapth, compression, partials, pcount, &is_partial,
+			cconf))
 		{
 			logp("error when inflating old file: %s\n", oldpath);
 			ret=-1;
+			free(infpath);
 			goto cleanup;
 		}
 
-		if(do_patch(infpath, deltafpath, newpath, cconf->compression,
-			compression /* from the manifest */, cntr, cconf))
+		if((lrs=do_patch(infpath, deltafpath, newpath,
+			cconf->compression,
+			compression /* from the manifest */, cntr, cconf)))
 		{
-			logp("error when patching\n");
-			ret=-1;
+			logp("WARNING: librsync error when patching %s: %d\n",
+				oldpath, lrs);
+			do_filecounter(cntr, CMD_WARNING, 1);
+			// Try to carry on with the rest of the backup
+			// regardless.
+			//ret=-1;
 			// Remove anything that got written.
 			unlink(newpath);
+			unlink(infpath);
+			free(infpath);
+
+			// First, note that we want to remove this entry from
+			// the manifest.
+			if(!*delfp && !(*delfp=open_file(deletionsfile, "ab")))
+			{
+				// Could not mark this file as deleted. Fatal.
+				ret=-1;
+				goto cleanup;
+			}
+			if(sbuf_to_manifest(sb, *delfp, NULL))
+				ret=-1;
+			if(fflush(*delfp))
+			{
+				logp("error fflushing deletions file in jiggle: %s\n", strerror(errno));
+				ret=-1;
+			}
+
 			goto cleanup;
 		}
 
 		// Get rid of the inflated old file.
-		// This will also remove it if there was an
-		// error.
 		unlink(infpath);
 		free(infpath);
 
 		// Need to generate a reverse diff,
 		// unless we are keeping a hardlinked
 		// archive.
-		if(!hardlinked)
+		if(!hardlinked && !is_partial)
 		{
 			if(gen_rev_delta(sigpath, deltabdir,
 				oldpath, newpath, datapth, endfile,
@@ -362,7 +449,8 @@ static int jiggle(const char *datapth, const char *currentdata, const char *data
 		// Use the old unchanged file.
 		// Hard link it first.
 		//logp("Hard linking to old file: %s\n", datapth);
-		if(do_link(oldpath, finpath, &statp, cconf))
+		if(do_link(oldpath, finpath, &statp, cconf,
+		  FALSE /* do not overwrite finpath (should never need to) */))
 		{
 			ret=-1;
 			goto cleanup;
@@ -428,9 +516,204 @@ static int do_hardlinked_archive(struct config *cconf, unsigned long bno)
 	return !ret;
 }
 
+static int maybe_delete_files_from_manifest(const char *manifest, const char *deletionsfile, struct config *cconf, struct cntr *cntr)
+{
+	int ars=0;
+	int ret=0;
+	int pcmp=0;
+	FILE *dfp=NULL;
+	struct sbuf db;
+	struct sbuf mb;
+	gzFile nmzp=NULL;
+	gzFile omzp=NULL;
+	char *manifesttmp=NULL;
+	struct stat statp;
+
+	if(lstat(deletionsfile, &statp))
+	{
+		// No deletions, no problem.
+		return 0;
+	}
+	logp("Performing deletions on manifest\n");
+
+	if(!(manifesttmp=get_tmp_filename(manifest)))
+	{
+		ret=-1;
+		goto end;
+	}
+
+        if(!(dfp=open_file(deletionsfile, "rb"))
+	  || !(omzp=gzopen_file(manifest, "rb"))
+	  || !(nmzp=gzopen_file(manifesttmp, comp_level(cconf))))
+	{
+		ret=-1;
+		goto end;
+	}
+
+	init_sbuf(&db);
+	init_sbuf(&mb);
+
+	while(omzp || dfp)
+	{
+		if(dfp && !db.path && (ars=sbuf_fill(dfp, NULL, &db, cntr)))
+		{
+			if(ars<0) { ret=-1; break; }
+			// ars==1 means it ended ok.
+			close_fp(&dfp);
+		}
+		if(omzp && !mb.path && (ars=sbuf_fill(NULL, omzp, &mb, cntr)))
+		{
+			if(ars<0) { ret=-1; break; }
+			// ars==1 means it ended ok.
+			gzclose_fp(&omzp);
+		}
+
+		if(mb.path && !db.path)
+		{
+			if(sbuf_to_manifest(&mb, NULL, nmzp)) { ret=-1; break; }
+			free_sbuf(&mb);
+		}
+		else if(!mb.path && db.path)
+		{
+			free_sbuf(&db);
+		}
+		else if(!mb.path && !db.path) 
+		{
+			continue;
+		}
+		else if(!(pcmp=sbuf_pathcmp(&mb, &db)))
+		{
+			// They were the same - do not write.
+			free_sbuf(&mb);
+			free_sbuf(&db);
+		}
+		else if(pcmp<0)
+		{
+			// Behind in manifest. Write.
+			if(sbuf_to_manifest(&mb, NULL, nmzp)) { ret=-1; break; }
+			free_sbuf(&mb);
+		}
+		else
+		{
+			// Behind in deletions file. Do not write.
+			free_sbuf(&db);
+		}
+	}
+
+end:
+	if(gzclose_fp(&nmzp))
+	{
+		logp("error closing %s in maybe_delete_files_from_manifest\n",
+			manifesttmp);
+		ret=-1;
+	}
+	
+	close_fp(&dfp);
+	gzclose_fp(&omzp);
+	free_sbuf(&db);
+	free_sbuf(&mb);
+	if(!ret)
+	{
+		unlink(deletionsfile);
+		if(do_rename(manifesttmp, manifest))
+		{
+			free(manifesttmp);
+			return -1;
+		}
+	}
+	if(manifesttmp)
+	{
+		unlink(manifesttmp);
+		free(manifesttmp);
+	}
+	return ret;
+}
+
+static int get_partials_list(const char *path, const char *datapth, struct strlist ***partials, int *pcount)
+{
+	int n=-1;
+	int ret=0;
+	struct dirent **dir;
+
+	if((n=scandir(path, &dir, 0, 0))<0)
+	{
+		logp("scandir %s: %s\n", path, strerror(errno));
+		return -1;
+	}
+	while(n--)
+	{
+		char *fullpath=NULL;
+
+		if(dir[n]->d_ino==0
+		  || !strcmp(dir[n]->d_name, ".")
+		  || !strcmp(dir[n]->d_name, ".."))
+		{ free(dir[n]); continue; }
+
+		if(!(fullpath=prepend_s(path,
+			dir[n]->d_name, strlen(dir[n]->d_name))))
+				break;
+
+		if(is_dir(fullpath, dir[n]))
+		{
+			char *newdatapth=NULL;
+			if(!(newdatapth=prepend_s(datapth,
+				dir[n]->d_name, strlen(dir[n]->d_name))))
+					break;
+			if(get_partials_list(fullpath, newdatapth,
+				partials, pcount))
+			{
+				free(fullpath);
+				free(newdatapth);
+				break;
+			}
+			free(newdatapth);
+		}
+		else
+		{
+			char *tmp=NULL;
+			// Add to list.
+			if(!(tmp=prepend_s(datapth,
+				dir[n]->d_name, strlen(dir[n]->d_name))))
+					break;
+			logp("Found partial resumed file: %s\n", tmp);
+			if(strlist_add(partials, pcount, tmp, 0))
+			{
+				free(tmp);
+				break;
+			}
+			free(tmp);
+		}
+		
+		free(fullpath);
+		free(dir[n]);
+	}
+        if(n>0)
+        {
+                ret=-1;
+                for(; n>0; n--) free(dir[n]);
+        }
+        free(dir);
+
+	return ret;
+}
+
+static int get_partials_list_w(const char *datadirtmp, struct strlist ***partials, int *pcount)
+{
+	int ret=0;
+	char *path=NULL;
+	char *datapth=NULL;
+	// Build the full path to it.
+	if(!(path=prepend_s(datadirtmp, "p", strlen("p")))) return -1;
+	if(is_dir_lstat(path))
+		ret=get_partials_list(path, datapth, partials, pcount);
+	free(path);
+
+	return ret;
+}
+
 /* Need to make all the stuff that this does atomic so that existing backups
    never get broken, even if somebody turns the power off on the server. */ 
-static int atomic_data_jiggle(const char *finishing, const char *working, const char *manifest, const char *current, const char *currentdata, const char *datadir, const char *datadirtmp, struct config *cconf, const char *client, int hardlinked, unsigned long bno, struct cntr *p1cntr, struct cntr *cntr)
+static int atomic_data_jiggle(const char *finishing, const char *working, const char *manifest, const char *current, const char *currentdata, const char *datadir, const char *datadirtmp, const char *deletionsfile, struct config *cconf, const char *client, int hardlinked, unsigned long bno, struct cntr *p1cntr, struct cntr *cntr)
 {
 	int ret=0;
 	int ars=0;
@@ -444,9 +727,18 @@ static int atomic_data_jiggle(const char *finishing, const char *working, const
 	gzFile zp=NULL;
 	struct sbuf sb;
 
+	FILE *delfp=NULL;
+
+	int pcount=0;
+	struct strlist **partials=NULL;
+
 	logp("Doing the atomic data jiggle...\n");
 
-	if(!(tmpman=get_tmp_filename(manifest))) return -1;
+	if(!(tmpman=get_tmp_filename(manifest)))
+	{
+		ret=-1;
+		goto end;
+	}
 	if(lstat(manifest, &statp))
 	{
 		// Manifest does not exist - maybe the server was killed before
@@ -455,7 +747,11 @@ static int atomic_data_jiggle(const char *finishing, const char *working, const
 		do_rename(tmpman, manifest);
 	}
 	free(tmpman);
-	if(!(zp=gzopen_file(manifest, "rb"))) return -1;
+	if(!(zp=gzopen_file(manifest, "rb")))
+	{
+		ret=-1;
+		goto end;
+	}
 
 	if(!(deltabdir=prepend_s(current,
 		"deltas.reverse", strlen("deltas.reverse")))
@@ -464,12 +760,19 @@ static int atomic_data_jiggle(const char *finishing, const char *working, const
 	  || !(sigpath=prepend_s(current,
 		"sig.tmp", strlen("sig.tmp"))))
 	{
-		logp("out of memory\n");
-		gzclose_fp(&zp);
-		return -1;
+		log_out_of_memory(__FUNCTION__);
+		ret=-1;
+		goto end;
+	}
+
+	if(get_partials_list_w(datadirtmp, &partials, &pcount))
+	{
+		ret=-1;
+		goto end;
 	}
 
 	mkdir(datadir, 0777);
+
 	init_sbuf(&sb);
 	while(!(ars=sbuf_fill(NULL, zp, &sb, cntr)))
 	{
@@ -480,8 +783,9 @@ static int atomic_data_jiggle(const char *finishing, const char *working, const
 
 			if((ret=jiggle(sb.datapth, currentdata, datadirtmp,
 				datadir, deltabdir, deltafdir,
-				sigpath, sb.endfile,
-				hardlinked, sb.compression, cntr, cconf)))
+				sigpath, sb.endfile, deletionsfile, &delfp,
+				&sb, hardlinked, sb.compression,
+				partials, pcount, cntr, cconf)))
 					break;
 		}
 		free_sbuf(&sb);
@@ -491,15 +795,26 @@ static int atomic_data_jiggle(const char *finishing, const char *working, const
 		if(ars>0) ret=0;
 		else ret=-1;
 	}
+
+	if(close_fp(&delfp))
+	{
+		logp("error closing %s in atomic_data_jiggle\n", deletionsfile);
+		ret=-1;
+	}
 	gzclose_fp(&zp);
 
-	if(ret)
-	{
-		// Remove the temporary data directory, we have now removed
-		// everything useful from it.
-		sync(); // try to help CIFS
-		recursive_delete(deltafdir, NULL, FALSE /* do not del files */);
-	}
+	if(maybe_delete_files_from_manifest(manifest, deletionsfile,
+		cconf, cntr)) ret=-1;
+
+	// Remove the temporary data directory, we have probably removed
+	// useful files from it.
+	sync(); // try to help CIFS
+	recursive_delete(deltafdir, NULL, FALSE /* do not del files */);
+
+end:
+	strlists_free(partials, pcount);
+	if(zp) gzclose_fp(&zp);
+	if(delfp) close_fp(&delfp);
 	if(deltabdir) free(deltabdir);
 	if(deltafdir) free(deltafdir);
 	if(sigpath) free(sigpath);
@@ -512,12 +827,12 @@ int backup_phase4_server(const char *basedir, const char *working, const char *c
 	int ret=0;
 	struct stat statp;
 	char *manifest=NULL;
+	char *deletionsfile=NULL;
 	char *datadir=NULL;
 	char *datadirtmp=NULL;
 	char *currentdup=NULL;
 	char *currentduptmp=NULL;
 	char *currentdupdata=NULL;
-	char *forward=NULL;
 	char *timestamp=NULL;
 	char *fullrealcurrent=NULL;
 	char *deleteme=NULL;
@@ -527,6 +842,13 @@ int backup_phase4_server(const char *basedir, const char *working, const char *c
 	char realcurrent[256]="";
 	FILE *logfp=NULL;
 
+	unsigned long bno=0;
+	FILE *fwd=NULL;
+	int hardlinked=0;
+	char tstmp[64]="";
+	int newdup=0;
+	int previous_backup=0;
+
 	if((len=readlink(current, realcurrent, sizeof(realcurrent)-1))<0)
 		len=0;
 	realcurrent[len]='\0';
@@ -534,10 +856,10 @@ int backup_phase4_server(const char *basedir, const char *working, const char *c
 	if(!(datadir=prepend_s(finishing, "data", strlen("data")))
 	  || !(datadirtmp=prepend_s(finishing, "data.tmp", strlen("data.tmp")))
 	  || !(manifest=prepend_s(finishing, "manifest.gz", strlen("manifest.gz")))
+	  || !(deletionsfile=prepend_s(finishing, "deletions", strlen("deletions")))
 	  || !(currentdup=prepend_s(finishing, "currentdup", strlen("currentdup")))
 	  || !(currentduptmp=prepend_s(finishing, "currentdup.tmp", strlen("currentdup.tmp")))
 	  || !(currentdupdata=prepend_s(currentdup, "data", strlen("data")))
-	  || !(forward=prepend_s(currentdup, "forward", strlen("forward")))
 	  || !(timestamp=prepend_s(finishing, "timestamp", strlen("timestamp")))
 	  || !(fullrealcurrent=prepend_s(basedir, realcurrent, strlen(realcurrent)))
 	  || !(deleteme=prepend_s(basedir, "deleteme", strlen("deleteme")))
@@ -548,7 +870,7 @@ int backup_phase4_server(const char *basedir, const char *working, const char *c
 		goto endfunc;
 	}
 
-	if(!(logfp=open_file(logpath, "ab")) || set_logfp(logfp, cconf))
+	if(set_logfp(logpath, cconf))
 	{
 		ret=-1;
 		goto endfunc;
@@ -560,11 +882,7 @@ int backup_phase4_server(const char *basedir, const char *working, const char *c
 
 	if(!lstat(current, &statp)) // Had a previous backup
 	{
-		unsigned long bno=0;
-		FILE *fwd=NULL;
-		int hardlinked=0;
-		char tstmp[64]="";
-		int newdup=0;
+		previous_backup++;
 
 		if(lstat(currentdup, &statp))
 		{
@@ -601,18 +919,6 @@ int backup_phase4_server(const char *basedir, const char *working, const char *c
 		// Get the backup number.
 		bno=strtoul(tstmp, NULL, 10);
 
-		// Put forward reference in, indicating the timestamp of
-		// the working directory (which will soon become the current
-		// directory).
-		if(!(fwd=open_file(forward, "wb")))
-		{
-			log_and_send("could not create forward file");
-			ret=-1;
-			goto endfunc;
-		}
-		fprintf(fwd, "%s\n", tstmp);
-		close_fp(&fwd);
-
 		if(newdup)
 		{
 			// When we have just created currentdup, determine
@@ -653,7 +959,12 @@ int backup_phase4_server(const char *basedir, const char *working, const char *c
 			// be useful one day when wondering when the next
 			// backup, now deleted, was made.
 			fprintf(hfp, "%s\n", tstmp);
-			close_fp(&hfp);
+			if(close_fp(&hfp))
+			{
+				logp("error closing hardlinked indication\n");
+				ret=-1;
+				goto endfunc;
+			}
 			logp(" doing hardlinked archive\n");
 			logp(" will not generate reverse deltas\n");
 		}
@@ -663,50 +974,52 @@ int backup_phase4_server(const char *basedir, const char *working, const char *c
 			logp(" will generate reverse deltas\n");
 			unlink(hlinkedpath);
 		}
-
-		if(atomic_data_jiggle(finishing,
-			working, manifest, currentdup,
-			currentdupdata,
-			datadir, datadirtmp, cconf, client,
-			hardlinked, bno, p1cntr, cntr))
-		{
-			logp("could not finish up backup.\n");
-			ret=-1;
-			goto endfunc;
-		}
-
-		write_status(client, STATUS_SHUFFLING,
-			"deleting temporary files", p1cntr, cntr);
-
-		// Remove the temporary data directory, we have now removed
-		// everything useful from it.
-		recursive_delete(datadirtmp, NULL, TRUE /* del files */);
-
-		// Clean up the currentdata directory - this is now the 'old'
-		// currentdata directory. Any files that were deleted from
-		// the client will be left in there, so call recursive_delete
-		// with the option that makes it not delete files.
-		// This will have the effect of getting rid of unnecessary
-		// directories.
-		sync(); // try to help CIFS
-		recursive_delete(currentdupdata, NULL, FALSE /* do not del files */);
-
-		// Rename the old current to something that we know to
-		// delete.
-		if(do_rename(fullrealcurrent, deleteme))
-		{
-			ret=-1;
-			goto endfunc;
-		}
 	}
 	else
 	{
-		// No previous backup, just put datadirtmp in the right place.
-		if(do_rename(datadirtmp, datadir))
-		{
-			ret=-1;
-			goto endfunc;
-		}
+		// No previous backup. Set hardlinked=1 so that the jiggle
+		// does not attempt to produce reverse deltas when it fixes
+		// up a partial file.
+		//hardlinked=1;
+	}
+
+	if(atomic_data_jiggle(finishing,
+		working, manifest, currentdup,
+		currentdupdata,
+		datadir, datadirtmp, deletionsfile, cconf, client,
+		hardlinked, bno, p1cntr, cntr))
+	{
+		logp("could not finish up backup.\n");
+		ret=-1;
+		goto endfunc;
+	}
+
+	write_status(client, STATUS_SHUFFLING,
+		"deleting temporary files", p1cntr, cntr);
+
+	// Remove the temporary data directory, we have now removed
+	// everything useful from it.
+	recursive_delete(datadirtmp, NULL, TRUE /* del files */);
+
+	// Clean up the currentdata directory - this is now the 'old'
+	// currentdata directory. Any files that were deleted from
+	// the client will be left in there, so call recursive_delete
+	// with the option that makes it not delete files.
+	// This will have the effect of getting rid of unnecessary
+	// directories.
+	sync(); // try to help CIFS
+	recursive_delete(currentdupdata, NULL, FALSE /* do not del files */);
+
+	// If deleteme already exists, delete it.
+	if(!lstat(deleteme, &statp))
+		recursive_delete(deleteme, NULL, TRUE /* delete all */);
+
+	// Rename the old current to something that we know to
+	// delete.
+	if(previous_backup && do_rename(fullrealcurrent, deleteme))
+	{
+		ret=-1;
+		goto endfunc;
 	}
 
 	if(!lstat(deleteme, &statp))
@@ -722,7 +1035,7 @@ int backup_phase4_server(const char *basedir, const char *working, const char *c
 	// Rename the finishing symlink so that it becomes the current symlink
 	do_rename(finishing, current);
 
-	print_filecounters(p1cntr, cntr, ACTION_BACKUP, 0);
+	print_filecounters(p1cntr, cntr, ACTION_BACKUP);
 	logp("Backup completed.\n");
 	logp("End phase4 (shuffle files)\n");
 	set_logfp(NULL, cconf); // will close logfp.
@@ -733,10 +1046,10 @@ endfunc:
 	if(datadir) free(datadir);
 	if(datadirtmp) free(datadirtmp);
 	if(manifest) free(manifest);
+	if(deletionsfile) free(deletionsfile);
 	if(currentdup) free(currentdup);
 	if(currentduptmp) free(currentduptmp);
 	if(currentdupdata) free(currentdupdata);
-	if(forward) free(forward);
 	if(timestamp) free(timestamp);
 	if(fullrealcurrent) free(fullrealcurrent);
 	if(deleteme) free(deleteme);
