@@ -26,6 +26,7 @@
 #define DEF_MAX_LINKS		10000
 
 static int makelinks=0;
+static int deletedups=0;
 static char *prog=NULL;
 
 static unsigned long long savedbytes=0;
@@ -35,6 +36,8 @@ static int ccount=0;
 static struct strlist **locklist=NULL;
 static int lockcount=0;
 
+static int verbose=0;
+
 typedef struct file file_t;
 
 struct file
@@ -70,7 +73,7 @@ static int add_file(struct mystruct *s, struct file *f)
 	struct file *newfile;
 	if(!(newfile=(struct file *)malloc(sizeof(struct file))))
 	{
-		logp("out of memory\n");
+		log_out_of_memory(__FUNCTION__);
 		return -1;
 	}
 	memcpy(newfile, f, sizeof(struct file));
@@ -85,7 +88,7 @@ static int add_key(off_t st_size, struct file *f)
 
 	if(!(s=(struct mystruct *)malloc(sizeof(struct mystruct))))
 	{
-		logp("out of memory\n");
+		log_out_of_memory(__FUNCTION__);
 		return -1;
 	}
 	s->st_size = st_size;
@@ -104,7 +107,7 @@ static char *prepend(const char *oldpath, const char *newpath, const char *sep)
 	len+=2;
 	if(!(path=(char *)malloc(len)))
 	{
-		logp("out of memory\n");
+		log_out_of_memory(__FUNCTION__);
 		return NULL;
 	}
 	snprintf(path, len, "%s%s%s", oldpath, *oldpath?sep:"", newpath);
@@ -261,7 +264,7 @@ static int do_hardlink(struct file *o, struct file *n, const char *ext)
 	char *tmppath=NULL;
 	if(!(tmppath=prepend(o->path, ext, "")))
 	{
-		logp("out of memory\n");
+		log_out_of_memory(__FUNCTION__);
 		return -1;
 	}
 	if(link(n->path, tmppath))
@@ -372,6 +375,8 @@ static int check_files(struct mystruct *find, struct file *newfile, struct stat
 		found++;
 		count++;
 
+		if(verbose) printf("%s\n", newfile->path);
+
 		// Now hardlink it.
 		if(makelinks)
 		{
@@ -393,6 +398,21 @@ static int check_files(struct mystruct *find, struct file *newfile, struct stat
 				count--;
 			}
 		}
+		else if(deletedups)
+		{
+			if(unlink(newfile->path))
+			{
+				logp("Could not delete %s: %s\n",
+					newfile->path, strerror(errno));
+			}
+			else
+			{
+				// Only count bytes as saved if we removed the
+				// last link.
+				if(newfile->nlink==1)
+					savedbytes+=info->st_size;
+			}
+		}
 		else
 		{
 			// To be able to tell how many bytes
@@ -422,7 +442,7 @@ static int get_link(const char *basedir, const char *lnk, char real[], size_t r)
 	char *tmp=NULL;
 	if(!(tmp=prepend(basedir, lnk, "/")))
 	{
-		logp("out of memory");
+		log_out_of_memory(__FUNCTION__);
 		return -1;
 	}
 	if((len=readlink(tmp, real, r-1))<0) len=0;
@@ -572,33 +592,16 @@ static int process_dir(const char *oldpath, const char *newpath, const char *ext
 	return 0;
 }
 
-static int looks_like_vim_tmpfile(const char *filename)
-{
-	const char *cp=NULL;
-	// vim tmpfiles look like ".filename.swp".
-	if(filename[0]=='.'
-	  && (cp=strrchr(filename, '.'))
-	  && !strcmp(cp, ".swp"))
-		return 1;
-	return 0;
-}
-
-static int in_group(const char *clientconfdir, const char *client, strlist_t **grouplist, int gcount, struct config *conf)
+static int in_group(const char *client, strlist_t **grouplist, int gcount, struct config *conf)
 {
 	int i=0;
-	char *ccfile=NULL;
 	struct config cconf;
 
-	if(!(ccfile=prepend(clientconfdir, client, "/"))) return -1;
-	init_config(&cconf);
-	if(set_client_global_config(conf, &cconf, client)
-	  || load_config(ccfile, &cconf, 0))
+	if(load_client_config(conf, &cconf, client))
 	{
 		logp("could not load config for client %s\n", client);
-		free(ccfile);
 		return 0;
 	}
-	free(ccfile);
 
 	if(!cconf.dedup_group) return 0;
 
@@ -660,16 +663,18 @@ static int iterate_over_clients(struct config *conf, strlist_t **grouplist, int
 	{
 		char *lockfile=NULL;
 		char *lockfilebase=NULL;
-		if(!strcmp(dirinfo->d_name, ".")
-		  || !strcmp(dirinfo->d_name, "..")
-		  || looks_like_vim_tmpfile(dirinfo->d_name)
+		char *client_lockdir=NULL;
+
+		if(dirinfo->d_ino==0
+		// looks_like...() also avoids '.' and '..'.
+		  || looks_like_tmp_or_hidden_file(dirinfo->d_name)
 		  || !is_regular_file(conf->clientconfdir, dirinfo->d_name))
 			continue;
 
 		if(gcount)
 		{
 			int ig=0;
-			if((ig=in_group(conf->clientconfdir,
+			if((ig=in_group(
 				dirinfo->d_name, grouplist, gcount, conf))<0)
 			{
 				ret=-1;
@@ -678,7 +683,10 @@ static int iterate_over_clients(struct config *conf, strlist_t **grouplist, int
 			if(!ig) continue;
 		}
 
-		if(!(lockfilebase=prepend(conf->client_lockdir,
+		if(!(client_lockdir=conf->client_lockdir))
+			client_lockdir=conf->directory;
+
+		if(!(lockfilebase=prepend(client_lockdir,
 			dirinfo->d_name, "/"))
 		 || !(lockfile=prepend(lockfilebase,
 			BEDUP_LOCKFILE_NAME, "/")))
@@ -742,6 +750,8 @@ static int usage(void)
 	printf("                           group, use the 'dedup_group' option in the client\n");
 	printf("                           configuration file on the server.\n");
 	printf("  -h|-?                    Print this text and exit.\n");
+	printf("  -d                       Delete any duplicate files found.\n");
+	printf("                           (non-burp mode only)\n");
 	printf("  -l                       Hard link any duplicate files found.\n");
 	printf("  -m <number>              Maximum number of hard links to a single file.\n");
 	printf("                           (non-burp mode only - in burp mode, use the\n");
@@ -750,7 +760,8 @@ static int usage(void)
 	printf("                           of links possible is 32000, but space is needed\n");
 	printf("                           for the normal operation of burp.\n");
 	printf("  -n <list of directories> Non-burp mode. Deduplicate any (set of) directories.\n");
-	printf("  -v                       Print version and exit.\n");
+	printf("  -v                       Print duplicate paths.\n");
+	printf("  -V                       Print version and exit.\n");
 	printf("\n");
 	printf("By default, %s will read %s and deduplicate client storage\n", prog, get_config_path());
 	printf("directories using special knowledge of the structure.\n");
@@ -807,7 +818,7 @@ int main(int argc, char *argv[])
 	configfile=get_config_path();
 	snprintf(ext, sizeof(ext), ".bedup.%d", getpid());
 
-	while((option=getopt(argc, argv, "c:g:hlmnv?"))!=-1)
+	while((option=getopt(argc, argv, "c:dg:hlm:nvV?"))!=-1)
 	{
 		switch(option)
 		{
@@ -815,6 +826,9 @@ int main(int argc, char *argv[])
 				configfile=optarg;
 				givenconfigfile=1;
 				break;
+			case 'd':
+				deletedups=1;
+				break;
 			case 'g':
 				groups=optarg;
 				break;
@@ -827,9 +841,12 @@ int main(int argc, char *argv[])
 			case 'n':
 				nonburp=1;
 				break;
-			case 'v':
+			case 'V':
 				printf("%s-%s\n", prog, VERSION);
 				return 0;
+			case 'v':
+				verbose=1;
+				break;
 			case 'h':
 			case '?':
 				return usage();
@@ -851,6 +868,16 @@ int main(int argc, char *argv[])
 		logp("-m option is specified via the configuration file in burp mode (max_hardlinks=)\n");
 		return 1;
 	}
+	if(deletedups && makelinks)
+	{
+		logp("-d and -l options are mutually exclusive\n");
+		return 1;
+	}
+	if(deletedups && !nonburp)
+	{
+		logp("-d option requires -n option\n");
+		return 1;
+	}
 
 	if(optind>=argc)
 	{
@@ -908,7 +935,7 @@ int main(int argc, char *argv[])
 					if(strlist_add(&grouplist, &gcount,
 						tok, 1))
 					{
-						logp("out of memory\n");
+						log_out_of_memory(__FUNCTION__);
 						return -1;
 					}
 				} while((tok=strtok(NULL, ",\n")));
@@ -970,7 +997,7 @@ int main(int argc, char *argv[])
 	logp("%llu duplicate %s found\n",
 		count, count==1?"file":"files");
 	logp("%llu bytes %s%s\n",
-		savedbytes, makelinks?"saved":"saveable",
+		savedbytes, (makelinks || deletedups)?"saved":"saveable",
 			bytes_to_human(savedbytes));
 	return ret;
 }
