@@ -38,13 +38,20 @@
 char *unix_name_to_win32(char *name);
 extern "C" HANDLE get_osfhandle(int fd);
 
-void binit(BFILE *bfd, int64_t winattr)
+void binit(BFILE *bfd, struct stat *statp, int64_t winattr)
 {
    memset(bfd, 0, sizeof(BFILE));
    bfd->mode = BF_CLOSED;
    bfd->use_backup_api = have_win32_api();
+   if(statp)
+   {
+   	bfd->statp.st_atime = statp->st_atime;
+   	bfd->statp.st_mtime = statp->st_mtime;
+   	bfd->statp.st_mode = statp->st_mode;
+   }
    bfd->winattr = winattr;
    bfd->pvContext = NULL;
+   bfd->path = NULL;
 }
 
 /*
@@ -252,6 +259,11 @@ int bopen(BFILE *bfd, const char *fname, int flags, mode_t mode, int isdir)
       bfd->berrno = b_errno_win32;
       errno = b_errno_win32;
       bfd->mode = BF_CLOSED;
+   } else {
+      if(!(bfd->path = strdup(fname))) {
+		log_out_of_memory(__FUNCTION__);
+		return -1;
+      }
    }
    bfd->errmsg = NULL;
    bfd->lpContext = NULL;
@@ -282,6 +294,10 @@ int bclose(BFILE *bfd)
       free(bfd->errmsg);
       bfd->errmsg = NULL;
    }
+   if (bfd->path) {
+      free(bfd->path);
+      bfd->path=NULL;
+   }
    if (bfd->mode == BF_CLOSED) {
       return 0;
    }
