@@ -11,6 +11,7 @@
 #include "backup_phase1_client.h"
 #include "backup_phase2_client.h"
 #include "restore_client.h"
+#include "delete_client.h"
 #include "list_client.h"
 #include "ssl.h"
 #include "berrno.h"
@@ -21,11 +22,14 @@
 #include "ca_client.h"
 
 #include <sys/types.h>
+#ifndef HAVE_WIN32
+#include <sys/utsname.h>
+#endif
 
 // Return 0 for OK, -1 for error, 1 for timer conditions not met.
 static int maybe_check_timer(const char *phase1str, struct config *conf, int *resume)
 {
-	char rcmd;
+	char rcmd=0;
 	char *rdst=NULL;
 	size_t rlen=0;
 	int complen=0;
@@ -69,36 +73,66 @@ static int maybe_check_timer(const char *phase1str, struct config *conf, int *re
 }
 
 // Return 0 for OK, -1 for error, 1 for timer conditions not met.
-static int do_backup_client(struct config *conf, int resume, int estimate, struct cntr *p1cntr, struct cntr *cntr)
+static int do_backup_client(struct config *conf, int resume, enum action act, struct cntr *p1cntr, struct cntr *cntr)
 {
 	int ret=0;
 
-	if(estimate)
+	if(act==ACTION_ESTIMATE)
 		logp("do estimate client\n");
 	else
 		logp("do backup client\n");
 
 #if defined(HAVE_WIN32)
-	win32_enable_backup_privileges(1 /* ignore_errors */);
-#endif
+	win32_enable_backup_privileges();
 #if defined(WIN32_VSS)
 	if((ret=win32_start_vss(conf))) return ret;
 #endif
+	if(act==ACTION_BACKUP_TIMED)
+	{
+		// Run timed backups with lower priority.
+		// I found that this has to be done after the snapshot, or the
+		// snapshot never finishes. At least, I waited 30 minutes with
+		// nothing happening.
+#if defined(B_VSS_XP) || defined(B_VSS_W2K3)
+		if(SetThreadPriority(GetCurrentThread(),
+				THREAD_PRIORITY_LOWEST))
+			logp("Set thread_priority_lowest\n");
+		else
+			logp("Failed to set thread_priority_lowest\n");
+#else
+		if(SetThreadPriority(GetCurrentThread(),
+				THREAD_MODE_BACKGROUND_BEGIN))
+			logp("Set thread_mode_background_begin\n");
+		else
+			logp("Failed to set thread_mode_background_begin\n");
+#endif
+	}
+#endif
 
 	// Scan the file system and send the results to the server.
 	// Skip phase1 if the server wanted to resume.
 	if(!ret && !resume) ret=backup_phase1_client(conf,
-		estimate, p1cntr, cntr);
+		act==ACTION_ESTIMATE, p1cntr, cntr);
 
 	// Now, the server will be telling us what data we need to send.
-	if(!estimate && !ret)
+	if(act!=ACTION_ESTIMATE && !ret)
 		ret=backup_phase2_client(conf, p1cntr, resume, cntr);
 
-	if(estimate)
-		print_filecounters(p1cntr, cntr, ACTION_ESTIMATE, 1);
+	if(act==ACTION_ESTIMATE)
+		print_filecounters(p1cntr, cntr, ACTION_ESTIMATE);
 
+#if defined(HAVE_WIN32)
+	if(act==ACTION_BACKUP_TIMED)
+	{
+		if(SetThreadPriority(GetCurrentThread(),
+				THREAD_MODE_BACKGROUND_END))
+			logp("Set thread_mode_background_end\n");
+		else
+			logp("Failed to set thread_mode_background_end\n");
+	}
 #if defined(WIN32_VSS)
 	win32_stop_vss();
+#endif
 #endif
 
 	return ret;
@@ -148,7 +182,7 @@ static int s_server_session_id_context=1;
    Returns 2 if there were restore/verify warnings.
    Returns 3 if timer conditions were not met.
 */
-static int do_client(struct config *conf, enum action act)
+static int do_client(struct config *conf, enum action act, int vss_restore, int json)
 {
 	int ret=0;
 	int rfd=-1;
@@ -163,8 +197,8 @@ static int do_client(struct config *conf, enum action act)
 	char *server_version=NULL;
 	const char *phase1str="backupphase1";
 
-	reset_filecounter(&p1cntr);
-	reset_filecounter(&cntr);
+	reset_filecounter(&p1cntr, time(NULL));
+	reset_filecounter(&cntr, time(NULL));
 
 	setup_signals_client();
 //	settimers(0, 100);
@@ -217,7 +251,7 @@ static int do_client(struct config *conf, enum action act)
 
 	if(act!=ACTION_ESTIMATE)
 	{
-		char cmd;
+		char cmd=0;
 		size_t len=0;
 		char *feat=NULL;
 		int ca_ret=0;
@@ -294,23 +328,58 @@ static int do_client(struct config *conf, enum action act)
 		// :srestore: means that the server wants to do a restore.
 		if(server_supports(feat, ":srestore:"))
 		{
-			logp("Server is initiating a restore\n");
-			if(incexc) { free(incexc); incexc=NULL; }
-			if((ret=incexc_recv_client_restore(&incexc,
-				conf, &p1cntr)))
-					goto end;
-			if(incexc)
+			if(conf->server_can_restore)
 			{
-				if((ret=parse_incexcs_buf(conf, incexc)))
-					goto end;
-				act=ACTION_RESTORE;
-				log_restore_settings(conf, 1);
+				logp("Server is initiating a restore\n");
+				if(incexc) { free(incexc); incexc=NULL; }
+				if((ret=incexc_recv_client_restore(&incexc,
+					conf, &p1cntr)))
+						goto end;
+				if(incexc)
+				{
+					if((ret=parse_incexcs_buf(conf,
+						incexc))) goto end;
+					act=ACTION_RESTORE;
+					log_restore_settings(conf, 1);
+				}
 			}
+			else
+			{
+				logp("Server wants to initiate a restore\n");
+				logp("Client configuration says no\n");
+				if(async_write_str(CMD_GEN, "srestore not ok"))
+				{
+					ret=-1;
+					goto end;
+				}
+			}
+		}
+
+		if(conf->orig_client)
+		{
+			char str[512]="";
+			snprintf(str, sizeof(str),
+				"orig_client=%s", conf->orig_client);
+			if(!server_supports(feat, ":orig_client:"))
+			{
+				logp("Server does not support switching client.\n");
+				ret=-1;
+				goto end;
+			}
+			if((ret=async_write_str(CMD_GEN, str))
+			  || (ret=async_read_expect(CMD_GEN, "orig_client ok")))
+			{
+				logp("Problem requesting %s\n", str);
+				ret=-1;
+				goto end;
+			}
+			logp("Switched to client %s\n", conf->orig_client);
 		}
 
 		// :sincexc: is for the server giving the client the
 		// incexc config.
-		if(act==ACTION_BACKUP || act==ACTION_BACKUP_TIMED)
+		if(act==ACTION_BACKUP
+		  || act==ACTION_BACKUP_TIMED)
 		{
 			if(!incexc && server_supports(feat, ":sincexc:"))
 			{
@@ -323,6 +392,13 @@ static int do_client(struct config *conf, enum action act)
 			}
 		}
 
+		if(server_supports(feat, ":counters:"))
+		{
+			if(async_write_str(CMD_GEN, "countersok"))
+				goto end;
+			conf->send_client_counters=1;
+		}
+
 		// :incexc: is for the client sending the server the
 		// incexc config so that it better knows what to do on
 		// resume.
@@ -330,6 +406,30 @@ static int do_client(struct config *conf, enum action act)
 		  && (ret=incexc_send_client(conf, &p1cntr)))
 			goto end;
 
+		if(server_supports(feat, ":uname:"))
+		{
+			const char *clientos=NULL;
+#ifdef HAVE_WIN32
+#ifdef _WIN64
+			clientos="Windows 64bit";
+#else
+			clientos="Windows 32bit";
+#endif
+#else
+			struct utsname utsname;
+			if(!uname(&utsname))
+				clientos=(const char *)utsname.sysname;
+#endif
+			if(clientos)
+			{
+				char msg[128]="";
+				snprintf(msg, sizeof(msg),
+					"uname=%s", clientos);
+				if(async_write_str(CMD_GEN, msg))
+					goto end;
+			}
+		}
+
 		if((ret=async_write_str(CMD_GEN, "extra_comms_end"))
 		  || (ret=async_read_expect(CMD_GEN, "extra_comms_end ok")))
 		{
@@ -352,7 +452,11 @@ static int do_client(struct config *conf, enum action act)
 			{
 				logp("Server is overriding the configuration\n");
 				logp("with the following settings:\n");
-				printf("%s\n", incexc);
+				if(log_incexcs_buf(incexc))
+				{
+					ret=-1;
+					goto end;
+				}
 			}
 			if(!conf->sdcount)
 			{
@@ -364,39 +468,45 @@ static int do_client(struct config *conf, enum action act)
 			if(!(ret=maybe_check_timer(phase1str,
 				conf, &resume)))
 			{
-				if(conf->backup_script_pre
-				 && run_script(
-				  conf->backup_script_pre,
-				  conf->backup_script_pre_arg,
-				  conf->bprecount,
-				  "pre",
-				  "reserved2",
-				  "reserved3",
-				  "reserved4",
-				  "reserved5",
-				  NULL, NULL, NULL, NULL, NULL,
-				  &p1cntr, 1, 1)) ret=-1;
+				if(conf->backup_script_pre)
+				{
+					int a=0;
+					const char *args[12];
+					args[a++]=conf->backup_script_pre;
+					args[a++]="pre";
+					args[a++]="reserved2";
+					args[a++]="reserved3";
+					args[a++]="reserved4";
+					args[a++]="reserved5";
+					args[a++]=NULL;
+					if(run_script(args,
+						conf->backup_script_pre_arg,
+						conf->bprecount,
+						&p1cntr, 1, 1)) ret=-1;
+				}
 
 				if(!ret && do_backup_client(conf,
-					resume, 0, &p1cntr, &cntr))
+					resume, act, &p1cntr, &cntr))
 						ret=-1;
 
 				if((conf->backup_script_post_run_on_fail
 				  || !ret) && conf->backup_script_post)
 				{
-				  if(run_script(
-					conf->backup_script_post,
-					conf->backup_script_post_arg,
-					conf->bpostcount,
-					"post",
-				// Tell post script whether the restore
-				// failed.
-					ret?"1":"0",
-					"reserved3",
-					"reserved4",
-					"reserved5",
-					NULL, NULL, NULL, NULL, NULL,
-					&cntr, 1, 1)) ret=-1;
+					int a=0;
+					const char *args[12];
+					args[a++]=conf->backup_script_post;
+					args[a++]="post";
+					// Tell post script whether the restore
+					// failed.
+					args[a++]=ret?"1":"0";
+					args[a++]="reserved3";
+					args[a++]="reserved4";
+					args[a++]="reserved5";
+					args[a++]=NULL;
+					if(run_script(args,
+						conf->backup_script_post_arg,
+						conf->bpostcount,
+						&cntr, 1, 1)) ret=-1;
 				}
 			}
 
@@ -419,36 +529,40 @@ static int do_client(struct config *conf, enum action act)
 		case ACTION_RESTORE:
 		case ACTION_VERIFY:
 		{
-			if(conf->restore_script_pre
-			   && run_script(
-				conf->restore_script_pre,
-				conf->restore_script_pre_arg,
-				conf->rprecount,
-				"pre",
-				"reserved2",
-				"reserved3",
-				"reserved4",
-				"reserved5",
-				NULL, NULL, NULL, NULL, NULL,
-				&cntr, 1, 1)) ret=-1;
+			if(conf->restore_script_pre)
+			{
+				int a=0;
+				const char *args[12];
+				args[a++]=conf->restore_script_pre;
+				args[a++]="pre";
+				args[a++]="reserved2";
+				args[a++]="reserved3";
+				args[a++]="reserved4";
+				args[a++]="reserved5";
+				args[a++]=NULL;
+				if(run_script(args,
+					conf->restore_script_pre_arg,
+					conf->rprecount, &cntr, 1, 1)) ret=-1;
+			}
 			if(!ret && do_restore_client(conf,
-				act, &p1cntr, &cntr)) ret=-1;
+				act, vss_restore, &p1cntr, &cntr)) ret=-1;
 			if((conf->restore_script_post_run_on_fail
 			  || !ret) && conf->restore_script_post)
 			{
-			   if(run_script(
-				conf->restore_script_post,
-				conf->restore_script_post_arg,
-				conf->rpostcount,
-				"post",
+				int a=0;
+				const char *args[12];
+				args[a++]=conf->restore_script_pre;
+				args[a++]="post";
 				// Tell post script whether the restore
 				// failed.
-				ret?"1":"0",
-				"reserved3",
-				"reserved4",
-				"reserved5",
-				NULL, NULL, NULL, NULL, NULL,
-				&cntr, 1, 1)) ret=-1;
+				args[a++]=ret?"1":"0";
+				args[a++]="reserved3";
+				args[a++]="reserved4";
+				args[a++]="reserved5";
+				args[a++]=NULL;
+				if(run_script(args,
+					conf->restore_script_post_arg,
+					conf->rpostcount, &cntr, 1, 1)) ret=-1;
 			}
 
 			// Return non-zero if there were warnings,
@@ -459,13 +573,16 @@ static int do_client(struct config *conf, enum action act)
 			break;
 		}
 		case ACTION_ESTIMATE:
-			if(!ret) ret=do_backup_client(conf, 0, 1,
+			if(!ret) ret=do_backup_client(conf, 0, act,
 					&p1cntr, &cntr);
 			break;
+		case ACTION_DELETE:
+			if(!ret) ret=do_delete_client(conf);
+			break;
 		case ACTION_LIST:
 		case ACTION_LONG_LIST:
 		default:
-			ret=do_list_client(conf, act);
+			ret=do_list_client(conf, act, json);
 			break;
 	}
 
@@ -481,14 +598,26 @@ end:
 	return ret;
 }
 
-int client(struct config *conf, enum action act)
+int client(struct config *conf, enum action act, int vss_restore, int json)
 {
 	int ret=0;
-	if((ret=do_client(conf, act))==1)
+	
+#ifdef HAVE_WIN32
+	// prevent sleep when idle
+	SetThreadExecutionState(ES_CONTINUOUS | ES_SYSTEM_REQUIRED);
+#endif
+	
+	if((ret=do_client(conf, act, vss_restore, json))==1)
 	{
 		logp("Re-opening connection to server\n");
 		sleep(5);
-		ret=do_client(conf, act);
+		ret=do_client(conf, act, vss_restore, json);
 	}
+	
+#ifdef HAVE_WIN32
+	// allow sleep when idle
+	SetThreadExecutionState(ES_CONTINUOUS);
+#endif
+	
 	return ret;
 }
