@@ -3,8 +3,10 @@
 #include "counter.h"
 #include "conf.h"
 #include "msg.h"
+#include "lock.h"
 #include "strlist.h"
 #include "prepend.h"
+#include "regexp.h"
 
 /* Init only stuff related to includes/excludes.
    This is so that the server can override them all on the client. */
@@ -16,9 +18,15 @@ static void init_incexcs(struct config *conf)
 	conf->nobackup=NULL; conf->nbcount=0;
 	conf->incext=NULL; conf->incount=0; // include extensions
 	conf->excext=NULL; conf->excount=0; // exclude extensions
+	conf->increg=NULL; conf->ircount=0; // include (regular expression)
+	conf->excreg=NULL; conf->ercount=0; // include (regular expression)
 	conf->excfs=NULL; conf->exfscount=0; // exclude filesystems
+	conf->excom=NULL; conf->excmcount=0; // exclude from compression
 	conf->fifos=NULL; conf->ffcount=0;
 	conf->blockdevs=NULL; conf->bdcount=0;
+	conf->split_vss=0;
+	conf->strip_vss=0;
+	conf->vss_drives=NULL;
 	/* stuff to do with restore */
 	conf->overwrite=0;
 	conf->strip=0;
@@ -37,12 +45,16 @@ static void free_incexcs(struct config *conf)
 	strlists_free(conf->nobackup, conf->nbcount);
 	strlists_free(conf->incext, conf->incount); // include extensions
 	strlists_free(conf->excext, conf->excount); // exclude extensions
+	strlists_free(conf->increg, conf->ircount); // include (regular expression)
+	strlists_free(conf->excreg, conf->ercount); // exclude (regular expression)
 	strlists_free(conf->excfs, conf->exfscount); // exclude filesystems
+	strlists_free(conf->excom, conf->excmcount); // exclude from compression
 	strlists_free(conf->fifos, conf->ffcount);
 	strlists_free(conf->blockdevs, conf->bdcount);
 	if(conf->backup) free(conf->backup);
 	if(conf->restoreprefix) free(conf->restoreprefix);
 	if(conf->regex) free(conf->regex);
+	if(conf->vss_drives) free(conf->vss_drives);
 	init_incexcs(conf);
 }
 
@@ -54,21 +66,23 @@ void init_config(struct config *conf)
 	conf->status_port=NULL;
 	conf->hardlinked_archive=0;
 	conf->working_dir_recovery_method=NULL;
-	conf->forking=0;
-	conf->daemon=0;
+	conf->forking=1;
+	conf->daemon=1;
 	conf->directory_tree=1;
 	conf->clientconfdir=NULL;
 	conf->cname=NULL;
 	conf->directory=NULL;
 	conf->timestamp_format=NULL;
+	conf->password_check=1;
 	conf->ca_conf=NULL;
 	conf->ca_name=NULL;
 	conf->ca_server_name=NULL;
 	conf->ca_burp_ca=NULL;
 	conf->ca_csr_dir=NULL;
 	conf->lockfile=NULL;
-	conf->logfile=NULL;
-	conf->syslog=0;
+	conf->log_to_syslog=0;
+	conf->log_to_stdout=1;
+	conf->progress_counter=0;
 	conf->password=NULL;
 	conf->passwd=NULL;
 	conf->server=NULL;
@@ -82,9 +96,10 @@ void init_config(struct config *conf)
 	conf->autoupgrade_dir=NULL;
 	conf->autoupgrade_os=NULL;
 	conf->ssl_cert_ca=NULL;
-        conf->ssl_cert=NULL;
-        conf->ssl_key=NULL;
-        conf->ssl_key_password=NULL;
+	conf->ssl_cert=NULL;
+	conf->ssl_key=NULL;
+	conf->ssl_key_password=NULL;
+	conf->ssl_ciphers=NULL;
 	conf->ssl_dhfile=NULL;
 	conf->ssl_peer_cn=NULL;
 	conf->encryption_password=NULL;
@@ -94,6 +109,8 @@ void init_config(struct config *conf)
 	conf->max_storage_subdirs=30000;
 	conf->librsync=1;
 	conf->compression=9;
+	conf->version_warn=1;
+	conf->resume_partial=0;
 	conf->client_lockdir=NULL;
 	conf->umask=0022;
 	conf->user=NULL;
@@ -101,6 +118,7 @@ void init_config(struct config *conf)
 	conf->keep=NULL;
 	conf->kpcount=0;
 	conf->max_hardlinks=10000;
+	conf->client_is_windows=0;
 
 	conf->timer_script=NULL;
 	conf->timer_arg=NULL;
@@ -137,11 +155,13 @@ void init_config(struct config *conf)
 	conf->server_script_pre=NULL;
 	conf->server_script_pre_arg=NULL;
 	conf->sprecount=0;
+	conf->server_script_pre_notify=0;
 
 	conf->server_script_post=NULL;
 	conf->server_script_post_arg=NULL;
 	conf->spostcount=0;
 	conf->server_script_post_run_on_fail=0;
+	conf->server_script_post_notify=0;
 
 	conf->backup_script=NULL;
 	conf->backup_script_arg=NULL;
@@ -153,22 +173,35 @@ void init_config(struct config *conf)
 	conf->server_script=NULL;
 	conf->server_script_arg=NULL;
 	conf->sscount=0;
+	conf->server_script_notify=0;
 
 	conf->dedup_group=NULL;
 	conf->browsefile=NULL;
 	conf->browsedir=NULL;
 
+	conf->client_can_delete=1;
 	conf->client_can_force_backup=1;
 	conf->client_can_list=1;
 	conf->client_can_restore=1;
 	conf->client_can_verify=1;
 
+	conf->rclients=NULL;
+	conf->rccount=0;
+
+	conf->server_can_restore=1;
+
+	conf->send_client_counters=0;
+	conf->restore_client=NULL;
+	conf->restore_path=NULL;
+	conf->orig_client=NULL;
+
 	init_incexcs(conf);
 }
 
 void free_config(struct config *conf)
 {
 	if(!conf) return;
+	if(conf->port) free(conf->port);
 	if(conf->configfile) free(conf->configfile);
 	if(conf->clientconfdir) free(conf->clientconfdir);
 	if(conf->cname) free(conf->cname);
@@ -180,7 +213,6 @@ void free_config(struct config *conf)
 	if(conf->ca_burp_ca) free(conf->ca_burp_ca);
 	if(conf->ca_csr_dir) free(conf->ca_csr_dir);
 	if(conf->lockfile) free(conf->lockfile);
-	if(conf->logfile) free(conf->logfile);
 	if(conf->password) free(conf->password);
 	if(conf->passwd) free(conf->passwd);
 	if(conf->server) free(conf->server);
@@ -190,6 +222,7 @@ void free_config(struct config *conf)
         if(conf->ssl_cert) free(conf->ssl_cert);
         if(conf->ssl_key) free(conf->ssl_key);
         if(conf->ssl_key_password) free(conf->ssl_key_password);
+        if(conf->ssl_ciphers) free(conf->ssl_ciphers);
         if(conf->ssl_dhfile) free(conf->ssl_dhfile);
         if(conf->ssl_peer_cn) free(conf->ssl_peer_cn);
         if(conf->user) free(conf->user);
@@ -208,6 +241,8 @@ void free_config(struct config *conf)
 	if(conf->notify_failure_script) free(conf->notify_failure_script);
 	strlists_free(conf->notify_failure_arg, conf->nfcount);
 
+	strlists_free(conf->rclients, conf->rccount);
+
 	if(conf->backup_script_pre) free(conf->backup_script_pre);
 	strlists_free(conf->backup_script_pre_arg, conf->bprecount);
 	if(conf->backup_script_post) free(conf->backup_script_post);
@@ -235,6 +270,9 @@ void free_config(struct config *conf)
 	if(conf->dedup_group) free(conf->dedup_group);
 	if(conf->browsefile) free(conf->browsefile);
 	if(conf->browsedir) free(conf->browsedir);
+	if(conf->restore_client) free(conf->restore_client);
+	if(conf->restore_path) free(conf->restore_path);
+	if(conf->orig_client) free(conf->orig_client);
 
 	free_incexcs(conf);
 
@@ -441,7 +479,7 @@ static int pre_post_override(char **override, char **pre, char **post)
 	if(!(*pre=strdup(*override))
 	  || !(*post=strdup(*override)))
 	{
-		logp("out of memory\n");
+		log_out_of_memory(__FUNCTION__);
 		return -1;
 	}
 	free(*override);
@@ -474,6 +512,18 @@ static void do_strlist_sort(struct strlist **list, int count, struct strlist ***
 	*dest=list;
 }
 
+static void do_build_regex(struct strlist **list, int count, struct strlist ***dest)
+{
+	int i;
+	for(i=0; i<count; i++)
+	{
+		*dest=list;
+		compile_regex(&(list[i]->re), list[i]->path);
+        }
+
+}
+
+
 #ifdef HAVE_LINUX_OS
 struct fstype
 {
@@ -558,7 +608,10 @@ struct llists
 	struct strlist **bdlist; // blockdevs to read
 	struct strlist **inlist; // include extensions
 	struct strlist **exlist; // exclude extensions
+	struct strlist **irlist; // include (regular expression)
+	struct strlist **erlist; // exclude (regular expression)
 	struct strlist **exfslist; // exclude filesystems
+	struct strlist **excomlist; // exclude from compression
 	struct strlist **talist;
 	struct strlist **nslist;
 	struct strlist **nflist;
@@ -571,6 +624,7 @@ struct llists
 	struct strlist **bslist;
 	struct strlist **rslist;
 	struct strlist **sslist;
+	struct strlist **rclist;
 	int got_kp_args;
 	int got_timer_args;
 	int got_ns_args;
@@ -578,18 +632,27 @@ struct llists
 	int got_spre_args;
 	int got_spost_args;
 	int got_ss_args;
+	int got_rc_args;
 };
 
 static int load_config_ints(struct config *conf, const char *field, const char *value)
 {
 	get_conf_val_int(field, value, "syslog",
-		&(conf->syslog));
+		&(conf->log_to_syslog));
+	get_conf_val_int(field, value, "stdout",
+		&(conf->log_to_stdout));
+	get_conf_val_int(field, value, "progress_counter",
+		&(conf->progress_counter));
 	get_conf_val_int(field, value, "hardlinked_archive",
 		&(conf->hardlinked_archive));
 	get_conf_val_int(field, value, "max_hardlinks",
 		&(conf->max_hardlinks));
 	get_conf_val_int(field, value, "librsync",
 		&(conf->librsync));
+	get_conf_val_int(field, value, "version_warn",
+		&(conf->version_warn));
+	get_conf_val_int(field, value, "resume_partial",
+		&(conf->resume_partial));
 	get_conf_val_int(field, value, "cross_all_filesystems",
 		&(conf->cross_all_filesystems));
 	get_conf_val_int(field, value, "read_all_fifos",
@@ -600,6 +663,12 @@ static int load_config_ints(struct config *conf, const char *field, const char *
 		&(conf->backup_script_post_run_on_fail));
 	get_conf_val_int(field, value, "server_script_post_run_on_fail",
 		&(conf->server_script_post_run_on_fail));
+	get_conf_val_int(field, value, "server_script_pre_notify",
+		&(conf->server_script_pre_notify));
+	get_conf_val_int(field, value, "server_script_post_notify",
+		&(conf->server_script_post_notify));
+	get_conf_val_int(field, value, "server_script_notify",
+		&(conf->server_script_notify));
 	get_conf_val_int(field, value, "notify_success_warnings_only",
 		&(conf->notify_success_warnings_only));
 	get_conf_val_int(field, value, "notify_success_changes_only",
@@ -614,6 +683,10 @@ static int load_config_ints(struct config *conf, const char *field, const char *
 		&(conf->max_storage_subdirs));
 	get_conf_val_int(field, value, "overwrite",
 		&(conf->overwrite));
+	get_conf_val_int(field, value, "split_vss",
+		&(conf->split_vss));
+	get_conf_val_int(field, value, "strip_vss",
+		&(conf->strip_vss));
 	get_conf_val_int(field, value, "strip",
 		&(conf->strip));
 	get_conf_val_int(field, value, "fork",
@@ -622,6 +695,8 @@ static int load_config_ints(struct config *conf, const char *field, const char *
 		&(conf->daemon));
 	get_conf_val_int(field, value, "directory_tree",
 		&(conf->directory_tree));
+	get_conf_val_int(field, value, "client_can_delete",
+		&(conf->client_can_delete));
 	get_conf_val_int(field, value, "client_can_force_backup",
 		&(conf->client_can_force_backup));
 	get_conf_val_int(field, value, "client_can_list",
@@ -630,6 +705,10 @@ static int load_config_ints(struct config *conf, const char *field, const char *
 		&(conf->client_can_restore));
 	get_conf_val_int(field, value, "client_can_verify",
 		&(conf->client_can_verify));
+	get_conf_val_int(field, value, "server_can_restore",
+		&(conf->server_can_restore));
+	get_conf_val_int(field, value, "password_check",
+		&(conf->password_check));
 
 	return 0;
 }
@@ -655,6 +734,8 @@ static int load_config_strings(struct config *conf, const char *field, const cha
 		return -1;
 	if(get_conf_val(field, value, "ssl_peer_cn", &(conf->ssl_peer_cn)))
 		return -1;
+	if(get_conf_val(field, value, "ssl_ciphers", &(conf->ssl_ciphers)))
+		return -1;
 	if(get_conf_val(field, value, "clientconfdir", &(conf->clientconfdir)))
 		return -1;
 	if(get_conf_val(field, value, "cname", &(conf->cname)))
@@ -679,6 +760,8 @@ static int load_config_strings(struct config *conf, const char *field, const cha
 		return -1;
 	if(get_conf_val(field, value, "regex", &(conf->regex)))
 		return -1;
+	if(get_conf_val(field, value, "vss_drives", &(conf->vss_drives)))
+		return -1;
 	if(get_conf_val(field, value, "browsedir", &(conf->browsedir)))
 		return -1;
 	if(get_conf_val(field, value, "browsefile", &(conf->browsefile)))
@@ -694,7 +777,6 @@ static int load_config_strings(struct config *conf, const char *field, const cha
 	// "pidfile" is a synonym for "lockfile".
 	if(get_conf_val(field, value, "pidfile", &(conf->lockfile)))
 		return -1;
-	if(get_conf_val(field, value, "logfile", &(conf->logfile))) return -1;
 	if(get_conf_val(field, value, "password", &(conf->password))) return -1;
 	if(get_conf_val(field, value, "passwd", &(conf->passwd))) return -1;
 	if(get_conf_val(field, value, "server", &(conf->server))) return -1;
@@ -723,8 +805,14 @@ static int load_config_strings(struct config *conf, const char *field, const cha
 		NULL, &(conf->incount), &(l->inlist), 0)) return -1;
 	if(get_conf_val_args(field, value, "exclude_ext", NULL,
 		NULL, &(conf->excount), &(l->exlist), 0)) return -1;
+	if(get_conf_val_args(field, value, "include_regex", NULL,
+		NULL, &(conf->ircount), &(l->irlist), 0)) return -1;
+	if(get_conf_val_args(field, value, "exclude_regex", NULL,
+		NULL, &(conf->ercount), &(l->erlist), 0)) return -1;
 	if(get_conf_val_args(field, value, "exclude_fs", NULL,
 		NULL, &(conf->exfscount), &(l->exfslist), 0)) return -1;
+	if(get_conf_val_args(field, value, "exclude_comp", NULL,
+		NULL, &(conf->excmcount), &(l->excomlist), 0)) return -1;
 	if(get_conf_val(field, value, "timer_script", &(conf->timer_script)))
 		return -1;
 	if(get_conf_val_args(field, value, "timer_arg", &(conf->timer_arg),
@@ -789,9 +877,16 @@ static int load_config_strings(struct config *conf, const char *field, const cha
 		&(conf->server_script_arg), &(l->got_ss_args),
 		&(conf->sscount), &(l->sslist), 0)) return -1;
 
+	if(get_conf_val_args(field, value, "restore_client",
+		&(conf->rclients), &(l->got_rc_args),
+		&(conf->rccount), &(l->rclist), 0)) return -1;
+
 	if(get_conf_val(field, value, "dedup_group", &(conf->dedup_group)))
 		return -1;
 
+	if(get_conf_val(field, value, "orig_client", &(conf->orig_client)))
+		return -1;
+
 	return 0;
 }
 
@@ -799,8 +894,16 @@ static int load_config_field_and_value(struct config *conf, const char *field, c
 {
 	if(!strcmp(field, "mode"))
 	{
-		if(!strcmp(value, "server")) conf->mode=MODE_SERVER;
-		else if(!strcmp(value, "client")) conf->mode=MODE_CLIENT;
+		if(!strcmp(value, "server"))
+		{
+			conf->mode=MODE_SERVER;
+			conf->progress_counter=0; // default to off for server
+		}
+		else if(!strcmp(value, "client"))
+		{
+			conf->mode=MODE_CLIENT;
+			conf->progress_counter=1; // default to on for client
+		}
 		else return -1;
 	}
 	else if(!strcmp(field, "compression"))
@@ -869,7 +972,7 @@ static int parse_config_line(struct config *conf, struct llists *l, const char *
 
 		if(!(extrafile=strdup(buf+2)))
 		{
-			logp("out of memory\n");
+			log_out_of_memory(__FUNCTION__);
 			return -1;
 		}
 
@@ -894,7 +997,7 @@ static int parse_config_line(struct config *conf, struct llists *l, const char *
 			char *tmp=NULL;
 			if(!(copy=strdup(config_path)))
 			{
-				logp("out of memory\n");
+				log_out_of_memory(__FUNCTION__);
 				free(extrafile);
 				return -1;
 			}
@@ -902,7 +1005,7 @@ static int parse_config_line(struct config *conf, struct llists *l, const char *
 			if(!(tmp=prepend_s(copy,
 				extrafile, strlen(extrafile))))
 			{
-				logp("out of memory\n");
+				log_out_of_memory(__FUNCTION__);
 				free(extrafile);
 				free(copy);
 			}
@@ -1044,7 +1147,7 @@ static int client_conf_checks(struct config *conf, const char *path, int *r)
 			logp("falling back to '%s'\n", conf->server);
 			if(!(conf->ssl_peer_cn=strdup(conf->server)))
 			{
-				logp("out of memory\n");
+				log_out_of_memory(__FUNCTION__);
 				return -1;
 			}
 		}
@@ -1083,6 +1186,21 @@ static int client_conf_checks(struct config *conf, const char *path, int *r)
 	return 0;
 }
 
+// Set the flag of the first item in a list that looks at extensions to the
+// maximum number of characters that need to be checked, plus one. This is for
+// a bit of added efficiency.
+static void set_max_ext(struct strlist **list, int count)
+{
+	int i=0;
+	int max=0;
+	for(i=0; i<count; i++)
+	{
+		int s=strlen(list[i]->path);
+		if(s>max) max=s;
+	}
+	if(count) list[0]->flag=max+1;
+}
+
 static int finalise_config(const char *config_path, struct config *conf, struct llists *l, bool loadall)
 {
 	int i=0;
@@ -1116,8 +1234,19 @@ static int finalise_config(const char *config_path, struct config *conf, struct
 	do_strlist_sort(l->inlist, conf->incount, &(conf->incext));
 	// exclude extensions
 	do_strlist_sort(l->exlist, conf->excount, &(conf->excext));
+	// include (regular expression)
+	do_build_regex(l->irlist, conf->ircount, &(conf->increg));
+	// exclude (regular expression)
+	do_build_regex(l->erlist, conf->ercount, &(conf->excreg));
 	// exclude filesystems
 	do_strlist_sort(l->exfslist, conf->exfscount, &(conf->excfs));
+	// exclude from compression
+	do_strlist_sort(l->excomlist, conf->excmcount, &(conf->excom));
+
+	set_max_ext(conf->incext, conf->incount);
+	set_max_ext(conf->excext, conf->excount);
+	set_max_ext(conf->excom, conf->excmcount);
+
 	do_strlist_sort(l->fflist, conf->ffcount, &(conf->fifos));
 	do_strlist_sort(l->bdlist, conf->bdcount, &(conf->blockdevs));
 	do_strlist_sort(l->fslist, conf->fscount, &(conf->fschgdir));
@@ -1190,11 +1319,17 @@ static int finalise_config(const char *config_path, struct config *conf, struct
 		&(conf->restore_script_pre), &(conf->restore_script_post));
 	pre_post_override(&(conf->server_script),
 		&(conf->server_script_pre), &(conf->server_script_post));
+	if(conf->server_script_notify)
+	{
+		conf->server_script_pre_notify=conf->server_script_notify;
+		conf->server_script_post_notify=conf->server_script_notify;
+	}
 
 	if(!l->got_timer_args) conf->timer_arg=l->talist;
 	if(!l->got_ns_args) conf->notify_success_arg=l->nslist;
 	if(!l->got_nf_args) conf->notify_failure_arg=l->nflist;
 	if(!l->got_ss_args) conf->server_script_arg=l->sslist;
+	if(!l->got_rc_args) conf->rclients=l->rclist;
 
 	setup_script_arg_override(l->bslist, conf->bscount,
 		&(l->bprelist), &(l->bpostlist),
@@ -1238,14 +1373,6 @@ static int finalise_config(const char *config_path, struct config *conf, struct
 			break;
 	}
 
-	// If client_lockdir not set, use conf->directory.
-	if(!conf->client_lockdir && conf->directory
-	  && !(conf->client_lockdir=strdup(conf->directory)))
-	{
-		logp("out of memory\n");
-		return -1;
-	}
-
 	return r;
 }
 
@@ -1284,6 +1411,7 @@ static void set_got_args(struct llists *l, struct config *conf)
 	l->got_spre_args=conf->sprecount;
 	l->got_spost_args=conf->spostcount;
 	l->got_ss_args=conf->sscount;
+	l->got_rc_args=conf->rccount;
 }
 
 int load_config(const char *config_path, struct config *conf, bool loadall)
@@ -1299,7 +1427,7 @@ int load_config(const char *config_path, struct config *conf, bool loadall)
 		if(conf->configfile) free(conf->configfile);
 		if(!(conf->configfile=strdup(config_path)))
 		{
-			logp("out of memory\n");
+			log_out_of_memory(__FUNCTION__);
 			return -1;
 		}
 	}
@@ -1322,7 +1450,7 @@ int parse_incexcs_buf(struct config *conf, const char *incexc)
 	set_got_args(&l, conf);
 	if(!(copy=strdup(incexc)))
 	{
-		logp("out of memory\n");
+		log_out_of_memory(__FUNCTION__);
 		return -1;
 	}
 	free_incexcs(conf);
@@ -1347,7 +1475,32 @@ int parse_incexcs_buf(struct config *conf, const char *incexc)
 	return finalise_config("server override", conf, &l, FALSE);
 }
 
-/* The server runs this when parsing a restore file on the server. */
+int log_incexcs_buf(const char *incexc)
+{
+	char *tok=NULL;
+	char *copy=NULL;
+	if(!incexc || !*incexc) return 0;
+	if(!(copy=strdup(incexc)))
+	{
+		log_out_of_memory(__FUNCTION__);
+		return -1;
+	}
+	if(!(tok=strtok(copy, "\n")))
+	{
+		logp("unable to parse server incexc\n");
+		free(copy);
+		return -1;
+	}
+	do
+	{
+		logp("%s\n", tok);
+	} while((tok=strtok(NULL, "\n")));
+	free(copy);
+	return 0;
+}
+
+/* The server runs this when parsing a restore file on the server, and when
+   checking whether split_vss changed since the last backup. */
 int parse_incexcs_path(struct config *conf, const char *path)
 {
 	free_incexcs(conf);
@@ -1358,7 +1511,7 @@ static int set_global_str(char **dst, const char *src)
 {
 	if(src && !(*dst=strdup(src)))
 	{
-		logp("out of memory when setting global string\n");
+		log_out_of_memory(__FUNCTION__);
 		return -1;
 	}
 	return 0;
@@ -1383,7 +1536,11 @@ static int set_global_arglist(struct strlist ***dst, struct strlist **src, int *
 /* Remember to update the list in the man page when you change these.*/
 int set_client_global_config(struct config *conf, struct config *cconf, const char *client)
 {
-	cconf->syslog=conf->syslog;
+	cconf->log_to_syslog=conf->log_to_syslog;
+	cconf->log_to_stdout=conf->log_to_stdout;
+	cconf->progress_counter=conf->progress_counter;
+	cconf->password_check=conf->password_check;
+	cconf->client_can_delete=conf->client_can_delete;
 	cconf->client_can_force_backup=conf->client_can_force_backup;
 	cconf->client_can_list=conf->client_can_list;
 	cconf->client_can_restore=conf->client_can_restore;
@@ -1391,9 +1548,14 @@ int set_client_global_config(struct config *conf, struct config *cconf, const ch
 	cconf->hardlinked_archive=conf->hardlinked_archive;
 	cconf->librsync=conf->librsync;
 	cconf->compression=conf->compression;
+	cconf->version_warn=conf->version_warn;
+	cconf->resume_partial=conf->resume_partial;
 	cconf->notify_success_warnings_only=conf->notify_success_warnings_only;
 	cconf->notify_success_changes_only=conf->notify_success_changes_only;
 	cconf->server_script_post_run_on_fail=conf->server_script_post_run_on_fail;
+	cconf->server_script_pre_notify=conf->server_script_pre_notify;
+	cconf->server_script_post_notify=conf->server_script_post_notify;
+	cconf->server_script_notify=conf->server_script_notify;
 	cconf->directory_tree=conf->directory_tree;
 	if(set_global_str(&(cconf->directory), conf->directory))
 		return -1;
@@ -1439,6 +1601,9 @@ int set_client_global_config(struct config *conf, struct config *cconf, const ch
 	if(set_global_arglist(&(cconf->server_script_arg),
 		conf->server_script_arg,
 		&(cconf->sscount), conf->sscount)) return -1;
+	if(set_global_arglist(&(cconf->rclients),
+		conf->rclients,
+		&(cconf->rccount), conf->rccount)) return -1;
 
 	// If ssl_peer_cn is not set, default it to the client name.
 	if(!conf->ssl_peer_cn
@@ -1447,3 +1612,27 @@ int set_client_global_config(struct config *conf, struct config *cconf, const ch
 
 	return 0;
 }
+
+int load_client_config(struct config *conf, struct config *cconf, const char *client)
+{
+	char *cpath=NULL;
+	init_config(cconf);
+	if(!(cpath=prepend_s(conf->clientconfdir, client, strlen(client))))
+		return -1;
+	if(looks_like_tmp_or_hidden_file(client))
+	{
+		logp("client name '%s' is invalid\n", client);
+		free(cpath);
+		return -1;
+	}
+	// Some client settings can be globally set in the server config and
+	// overridden in the client specific config.
+	if(set_client_global_config(conf, cconf, client)
+	  || load_config(cpath, cconf, FALSE))
+	{
+		free(cpath);
+		return -1;
+	}
+	free(cpath);
+	return 0;
+}
