@@ -13,7 +13,6 @@
 
 #include <netdb.h>
 #include <librsync.h>
-#include <regex.h>
 #include <math.h>
 
 int recursive_hardlink(const char *src, const char *dst, const char *client, struct cntr *p1cntr, struct cntr *cntr, struct config *conf)
@@ -58,6 +57,22 @@ int recursive_hardlink(const char *src, const char *dst, const char *client, str
 			break;
 		}
 
+#ifdef _DIRENT_HAVE_D_TYPE
+// Faster evaluation on most systems.
+		if(dir[n]->d_type==DT_DIR)
+		{
+			if(recursive_hardlink(fullpatha, fullpathb, client,
+				p1cntr, cntr, conf))
+			{
+				free(fullpatha);
+				free(fullpathb);
+				break;
+			}
+		}
+		else
+#endif
+		// Otherwise, we have to do an lstat() anyway, because we
+		// will need to check the number of hardlinks in do_link().
 		if(lstat(fullpatha, &statp))
 		{
 			logp("could not lstat %s\n", fullpatha);
@@ -77,7 +92,8 @@ int recursive_hardlink(const char *src, const char *dst, const char *client, str
 			//logp("hardlinking %s to %s\n", fullpathb, fullpatha);
 			write_status(client, STATUS_SHUFFLING, fullpathb,
 				p1cntr, cntr);
-			if(do_link(fullpatha, fullpathb, &statp, conf))
+			if(do_link(fullpatha, fullpathb, &statp, conf,
+				FALSE /* do not overwrite target */))
 			{
 				free(fullpatha);
 				free(fullpathb);
@@ -116,7 +132,7 @@ int recursive_delete(const char *d, const char *file, bool delfiles)
 	}
 	else if(!(directory=prepend_s(d, file, strlen(file))))
 	{
-		logp("out of memory\n");
+		log_out_of_memory(__FUNCTION__);
 		return RECDEL_ERROR;
 	}
 
@@ -144,7 +160,7 @@ int recursive_delete(const char *d, const char *file, bool delfiles)
 			dir[n]->d_name, strlen(dir[n]->d_name))))
 				break;
 
-		if(is_dir(fullpath))
+		if(is_dir(fullpath, dir[n]))
 		{
 			int r;
 			if((r=recursive_delete(directory,
@@ -224,15 +240,14 @@ void free_current_backups(struct bu **arr, int a)
 	{
 		if((*arr)[b].path)
 			{ free((*arr)[b].path); (*arr)[b].path=NULL; }
+		if((*arr)[b].basename)
+			{ free((*arr)[b].basename); (*arr)[b].basename=NULL; }
 		if((*arr)[b].data)
 			{ free((*arr)[b].data); (*arr)[b].data=NULL; }
 		if((*arr)[b].delta)
 			{ free((*arr)[b].delta); (*arr)[b].delta=NULL; }
 		if((*arr)[b].timestamp)
 			{ free((*arr)[b].timestamp); (*arr)[b].timestamp=NULL; }
-		if((*arr)[b].forward_timestamp)
-			{ free((*arr)[b].forward_timestamp);
-				(*arr)[b].forward_timestamp=NULL; }
 	}
 	free(*arr);
 	*arr=NULL;
@@ -244,7 +259,7 @@ static int get_link(const char *basedir, const char *lnk, char real[], size_t r)
 	char *tmp=NULL;
 	if(!(tmp=prepend_s(basedir, lnk, strlen(lnk))))
 	{
-		logp("out of memory");
+		log_out_of_memory(__FUNCTION__);
 		return -1;
 	}
 	if((len=readlink(tmp, real, r-1))<0) len=0;
@@ -270,7 +285,6 @@ int get_current_backups(const char *basedir, struct bu **arr, int *a, int log)
 	if(get_link(basedir, "working", realwork, sizeof(realwork))
 	  || get_link(basedir, "finishing", realfinishing, sizeof(realfinishing)))
 			return -1;
-
 	if(!(d=opendir(basedir)))
 	{
 		if(log) log_and_send("could not open backup directory");
@@ -282,10 +296,9 @@ int get_current_backups(const char *basedir, struct bu **arr, int *a, int log)
 		struct stat statp;
 		char *fullpath=NULL;
 		char *timestamp=NULL;
-		char *forward=NULL;
 		char *timestampstr=NULL;
-		char *forward_timestampstr=NULL;
 		char *hlinkedpath=NULL;
+		char *basename=NULL;
 
 		if(dp->d_ino==0
 		  || !strcmp(dp->d_name, ".")
@@ -293,18 +306,19 @@ int get_current_backups(const char *basedir, struct bu **arr, int *a, int log)
 		  || !strcmp(dp->d_name, realwork)
 		  || !strcmp(dp->d_name, realfinishing))
 			continue;
-		if(!(fullpath=prepend_s(basedir,
-			dp->d_name, strlen(dp->d_name)))
+		if(!(basename=prepend("",
+			dp->d_name, strlen(dp->d_name), ""))
+		 || !(fullpath=prepend_s(basedir,
+			basename, strlen(basename)))
 		 || !(timestamp=prepend_s(fullpath,
 			"timestamp", strlen("timestamp")))
-		 || !(forward=prepend_s(fullpath,
-			"forward", strlen("forward")))
 		 || !(hlinkedpath=prepend_s(fullpath,
 			"hardlinked", strlen("hardlinked"))))
 		{
 			ret=-1;
-			if(timestamp) free(timestamp);
+			if(basename) free(basename);
 			if(fullpath) free(fullpath);
+			if(timestamp) free(timestamp);
 			break;
 		}
 		if((!lstat(fullpath, &statp) && !S_ISDIR(statp.st_mode))
@@ -312,8 +326,8 @@ int get_current_backups(const char *basedir, struct bu **arr, int *a, int log)
 		  || read_timestamp(timestamp, buf, sizeof(buf))
 		  || !(timestampstr=strdup(buf)))
 		{
+			free(basename);
 			free(fullpath);
-			free(forward);
 			free(timestamp);
 			free(hlinkedpath);
 			continue;
@@ -322,36 +336,23 @@ int get_current_backups(const char *basedir, struct bu **arr, int *a, int log)
 
 		if(!lstat(hlinkedpath, &statp)) hardlinked++;
 
-		if(!read_timestamp(forward, buf, sizeof(buf)))
-			forward_timestampstr=strdup(buf);
-		free(forward);
-
 		if(!(*arr=(struct bu *)realloc(*arr,(i+1)*sizeof(struct bu)))
 		  || !((*arr)[i].data=prepend_s(fullpath, "data", strlen("data")))
 		  || !((*arr)[i].delta=prepend_s(fullpath, "deltas.reverse", strlen("deltas.reverse"))))
 		{
-			if(log) log_and_send("out of memory");
+			if(log) log_and_send_oom(__FUNCTION__);
+			free(basename);
 			free(timestampstr);
 			free(fullpath);
 			free(hlinkedpath);
 			break;
 		}
 		(*arr)[i].path=fullpath;
+		(*arr)[i].basename=basename;
 		(*arr)[i].timestamp=timestampstr;
 		(*arr)[i].hardlinked=hardlinked;
 		(*arr)[i].deletable=0;
 		(*arr)[i].index=strtoul(timestampstr, NULL, 10);
-		if(forward_timestampstr)
-		{
-			(*arr)[i].forward_timestamp=forward_timestampstr;
-			(*arr)[i].forward_index=strtoul(forward_timestampstr,
-				NULL, 10);
-		}
-		else
-		{
-			(*arr)[i].forward_timestamp=NULL;
-			(*arr)[i].forward_index=0;
-		}
 		(*arr)[i].trindex=0;
 		i++;
 	}
@@ -359,7 +360,7 @@ int get_current_backups(const char *basedir, struct bu **arr, int *a, int log)
 
 	if(*arr) qsort(*arr, i, sizeof(struct bu), bu_cmp);
 
-	if(i>1)
+	if(i>=1)
 	{
 		tr=(*arr)[i-1].index;
 		// The oldest backup is deletable.
@@ -368,18 +369,6 @@ int get_current_backups(const char *basedir, struct bu **arr, int *a, int log)
 
 	for(j=0; j<i-1; j++)
 	{
-		if(!(*arr)[j].forward_timestamp
-		  || strcmp((*arr)[j].forward_timestamp, (*arr)[j+1].timestamp)
-		  || (*arr)[j].forward_index!=(*arr)[j+1].index)
-		{
-			char msg[256]="";
-			snprintf(msg, sizeof(msg), "%s does not match forward path from %s\n", (*arr)[j+1].path, (*arr)[j].path);
-			if(log) log_and_send(msg);
-			free_current_backups(arr, i);
-			ret=-1;
-			break;
-		}
-
 		// Backups that come after hardlinked backups are deletable.
 		if((*arr)[j].hardlinked) (*arr)[j+1].deletable=1;
 	}
@@ -467,8 +456,7 @@ static int compress(const char *src, const char *dst, struct config *cconf)
 		}
 	}
 	close_fp(&mp);
-	gzclose_fp(&zp);
-	return 0;
+	return gzclose_fp(&zp); // this can give an error when out of space
 }
 
 int compress_file(const char *src, const char *dst, struct config *cconf)
@@ -511,26 +499,62 @@ int compress_filename(const char *d, const char *file, const char *zfile, struct
 	return 0;
 }
 
-static int delete_backup(const char *basedir, struct bu *arr, int a, int b, const char *client)
+int delete_backup(const char *basedir, struct bu *arr, int a, int b, const char *client)
 {
 	char *deleteme=NULL;
 
 	logp("deleting %s backup %lu\n", client, arr[b].index);
 
-	// First, point the preceeding backup to the following backup.
-	if(b>0 && b+1<a)
+	if(b==a-1)
 	{
-		char *forward=NULL;
-		if(!(forward=prepend_s(arr[b-1].path,
-			"forward", strlen("forward")))) return -1;
-
-		if(write_timestamp(forward, arr[b+1].timestamp))
-		{
-			free(forward);
+		char *current=NULL;
+		// This is the current backup. Special measures are needed.
+		if(!(current=prepend_s(basedir, "current", strlen("current"))))
 			return -1;
+		if(!b)
+		{
+			// After the deletion, there will be no backups left.
+			// Just remove the symlink.
+			if(unlink(current))
+			{
+				logp("unlink %s: %s\n",
+					current, strerror(errno));
+				free(current);
+				return -1;
+			}
 		}
-
-		free(forward);
+		else
+		{
+			// Need to point the symlink at the previous backup.
+			char *tmp=NULL;
+			const char *target=NULL;
+			
+			if(!(tmp=prepend(current, ".tmp", strlen(".tmp"), "")))
+			{
+				free(current);
+				return -1;
+			}
+			target=arr[b-1].basename;
+			unlink(tmp);
+			if(symlink(target, tmp))
+			{
+				logp("could not symlink '%s' to '%s': %s\n",
+					tmp, target, strerror(errno));
+				logp("delete failed\n");
+				free(tmp);
+				free(current);
+				return -1;
+			}
+			if(do_rename(tmp, current))
+			{
+				logp("delete failed\n");
+				free(tmp);
+				free(current);
+				return -1;
+			}
+			free(tmp);
+		}
+		free(current);
 	}
 
 	if(!(deleteme=prepend_s(basedir, "deleteme", strlen("deleteme")))
@@ -672,31 +696,6 @@ int remove_old_backups(const char *basedir, struct config *cconf, const char *cl
 	return 0;
 }
 
-int compile_regex(regex_t **regex, const char *str)
-{
-	if(str && *str)
-	{
-		if(!(*regex=(regex_t *)malloc(sizeof(regex_t)))
-		  || regcomp(*regex, str, REG_EXTENDED))
-		{
-			log_and_send("unable to compile regex\n");
-			return -1;
-		}
-	}
-	return 0;
-}
-
-int check_regex(regex_t *regex, const char *buf)
-{
-	if(!regex) return 1;
-	switch(regexec(regex, buf, 0, NULL, 0))
-	{
-		case 0: return 1;
-		case REG_NOMATCH: return 0;
-		default: return 0;
-	}
-}
-
 /* Need to base librsync block length on the size of the old file, otherwise
    the risk of librsync collisions and silent corruption increases as the
    size of the new file gets bigger. */
@@ -744,7 +743,7 @@ finish:
 	return ret;
 }
 
-int do_link(const char *oldpath, const char *newpath, struct stat *statp, struct config *conf)
+int do_link(const char *oldpath, const char *newpath, struct stat *statp, struct config *conf, bool overwrite)
 {
 	/* Avoid creating too many hardlinks */
 	if(statp->st_nlink >= (unsigned int)conf->max_hardlinks)
@@ -753,8 +752,21 @@ int do_link(const char *oldpath, const char *newpath, struct stat *statp, struct
 	}
 	else if(link(oldpath, newpath))
 	{
-		logp("could not hard link '%s' to '%s': %s\n",
-			newpath, oldpath, strerror(errno));
+		if(overwrite && errno==EEXIST)
+		{
+			unlink(newpath);
+			if(link(oldpath, newpath))
+			{
+				logp("could not hard link '%s' to '%s': %s\n",
+					newpath, oldpath, strerror(errno));
+				return -1;
+			}
+			else
+			{
+				//logp("Successful hard link of '%s' to '%s' after unlinking the former\n", newpath, oldpath);
+				return 0;
+			}
+		}
 		return -1;
 	}
 	return 0;
