@@ -4,6 +4,7 @@
 #include "log.h"
 #include "asyncio.h"
 #include "handy.h"
+#include "regexp.h"
 #include "backup_phase1_client.h"
 #ifdef HAVE_DARWIN_OS
 #include <sys/param.h>
@@ -134,7 +135,7 @@ static FF_PKT *new_dir_ff_pkt(FF_PKT *ff_pkt)
    FF_PKT *dir_ff_pkt = NULL;
    if(!(dir_ff_pkt=(FF_PKT *)malloc(sizeof(FF_PKT))))
    {
-	logp("out of memory\n");
+	log_out_of_memory(__FUNCTION__);
 	return NULL;
    }
    memcpy(dir_ff_pkt, ff_pkt, sizeof(FF_PKT));
@@ -182,11 +183,20 @@ static int in_include_ext(struct strlist **incext, int incount, const char *fnam
 	const char *cp=NULL;
 	// If not doing include_ext, let the file get backed up. 
 	if(!incount) return 1;
-	// If file has no extension, it cannot be included.
-	if(!(cp=strrchr(fname, '.'))) return 0;
-	for(i=0; i<incount; i++)
-		if(!strcasecmp(incext[i]->path, cp+1))
-			return 1;
+
+	// The flag of the first item contains the maximum number of characters
+	// that need to be checked.
+	for(cp=fname+strlen(fname)-1; i<incext[0]->flag && cp>=fname; cp--, i++)
+	{
+		if(*cp=='.')
+		{
+			for(i=0; i<incount; i++)
+				if(!strcasecmp(incext[i]->path, cp+1))
+					return 1;
+			// If file has no extension, it cannot be included.
+			return 0;
+		}
+	}
 	return 0;
 }
 
@@ -196,16 +206,78 @@ static int in_exclude_ext(struct strlist **excext, int excount, const char *fnam
 	const char *cp=NULL;
 	// If not doing exclude_ext, let the file get backed up.
 	if(!excount) return 0;
-	// If file has no extension, it is included.
-	if(!(cp=strrchr(fname, '.'))) return 0;
-	for(i=0; i<excount; i++)
-		if(!strcasecmp(excext[i]->path, cp+1))
+
+	// The flag of the first item contains the maximum number of characters
+	// that need to be checked.
+	for(cp=fname+strlen(fname)-1; i<excext[0]->flag && cp>=fname; cp--, i++)
+	{
+		if(*cp=='.')
+		{
+			for(i=0; i<excount; i++)
+				if(!strcasecmp(excext[i]->path, cp+1))
+					return 1;
+			// If file has no extension, it is included.
+			return 0;
+		}
+	}
+
+	return 0;
+}
+
+// Returns the level of compression.
+int in_exclude_comp(struct strlist **excom, int excmcount, const char *fname, int compression)
+{
+	int i=0;
+	const char *cp=NULL;
+	// If not doing compression, or there are no excludes, return
+	// straight away.
+	if(!compression || !excmcount) return compression;
+
+	// The flag of the first item contains the maximum number of characters
+	// that need to be checked.
+	for(cp=fname+strlen(fname)-1; i<excom[0]->flag && cp>=fname; cp--, i++)
+	{
+		if(*cp=='.')
+		{
+			for(i=0; i<excmcount; i++)
+				if(!strcasecmp(excom[i]->path, cp+1))
+					return 0;
+			return compression;
+		}
+	}
+	return compression;
+}
+
+/* Return 1 to include the file, 0 to exclude it. */
+/* Currently not used - it needs more thinking about. */
+int in_include_regex(struct strlist **increg, int ircount, const char *fname)
+{
+	int i;
+	// If not doing include_regex, let the file get backed up.
+	if(!ircount) return 1;
+	for(i=0; i<ircount; i++)
+	{
+		if(check_regex(increg[i]->re, fname))
 			return 1;
+	}
+	return 0;
+}
+
+int in_exclude_regex(struct strlist **excreg, int ercount, const char *fname)
+{
+	int i;
+	// If not doing exclude_regex, let the file get backed up.
+	//if(!ercount) return 0; (will return 0 anyway)
+	for(i=0; i<ercount; i++)
+        {
+		if(check_regex(excreg[i]->re, fname))
+			return 1;
+	}
 	return 0;
 }
 
 // When recursing into directories, do not want to check the include_ext list.
-static int file_is_included_no_incext(struct strlist **ielist, int iecount, struct strlist **excext, int excount, const char *fname)
+static int file_is_included_no_incext(struct strlist **ielist, int iecount, struct strlist **excext, int excount, struct strlist **excreg, int ercount, const char *fname)
 {
 	int i=0;
 	int ret=0;
@@ -213,7 +285,9 @@ static int file_is_included_no_incext(struct strlist **ielist, int iecount, stru
 	int matching=0;
 	int best=-1;
 
-	if(in_exclude_ext(excext, excount, fname)) return 0;
+	if(in_exclude_ext(excext, excount, fname)
+	  || in_exclude_regex(excreg, ercount, fname))
+		return 0;
 
 	// Check include/exclude directories.
 	for(i=0; i<iecount; i++)
@@ -234,7 +308,12 @@ static int file_is_included_no_incext(struct strlist **ielist, int iecount, stru
 	return ret;
 }
 
-int file_is_included(struct strlist **ielist, int iecount, struct strlist **incexc, int incount, struct strlist **excext, int excount, const char *fname, bool top_level)
+int file_is_included(struct strlist **ielist, int iecount,
+	struct strlist **incexc, int incount,
+	struct strlist **excext, int excount,
+	struct strlist **increg, int ircount,
+	struct strlist **excreg, int ercount,
+	const char *fname, bool top_level)
 {
 	// Always save the top level directory.
 	// This will help in the simulation of browsing backups because it
@@ -248,7 +327,7 @@ int file_is_included(struct strlist **ielist, int iecount, struct strlist **ince
 	  && !in_include_ext(incexc, incount, fname)) return 0;
 
 	return file_is_included_no_incext(ielist, iecount,
-		excext, excount, fname);
+		excext, excount, excreg, ercount, fname);
 }
 
 static int fs_change_is_allowed(struct config *conf, const char *fname)
@@ -321,7 +400,6 @@ static int found_regular_file(FF_PKT *ff_pkt, struct config *conf,
 		ff_pkt->type=FT_REG;
 	rtn_stat=send_file(ff_pkt, top_level, conf, cntr);
 	if(ff_pkt->linked) ff_pkt->linked->FileIndex=ff_pkt->FileIndex;
-	if(ff_pkt->flags & FO_KEEPATIME) utime(fname, restore_times);
 	return rtn_stat;
 }
 
@@ -358,7 +436,6 @@ static int backup_dir_and_return(FF_PKT *ff_pkt, struct config *conf, struct cnt
 	free_dir_ff_pkt(dir_ff_pkt);
 	/* reset "link" */
 	ff_pkt->link=ff_pkt->fname;
-	if(ff_pkt->flags & FO_KEEPATIME) utime(fname, restore_times);
 	return rtn_stat;
 }
 
@@ -385,85 +462,13 @@ int fstype_excluded(struct config *conf, const char *fname, struct cntr *cntr)
 	return 0;
 }
 
-/* prototype, because found_directory() recurses using find_files() */
-static int
-find_files(FF_PKT *ff_pkt, struct config *conf, struct cntr *cntr,
-  char *fname, dev_t parent_device, bool top_level);
-
-static int found_directory(FF_PKT *ff_pkt, struct config *conf,
-	struct cntr *cntr, char *fname, dev_t parent_device, bool top_level,
-	struct utimbuf *restore_times)
+#if defined(HAVE_WIN32)
+static void windows_reparse_point_fiddling(FF_PKT *ff_pkt)
 {
-	int m;
-	int rtn_stat;
-	int count=0;
-	int allocated=0;
-	DIR *directory;
-	char *link=NULL;
-	size_t link_len;
-	size_t len;
-	int status;
-	int nbret=0;
-	bool recurse;
-	dev_t our_device;
-	bool volhas_attrlist;
-	struct dirent **nl=NULL;
-	struct dirent **ntmp=NULL;
-	struct dirent *entry=NULL;
-	struct dirent *result=NULL;
-
-	recurse=true;
-	our_device=ff_pkt->statp.st_dev;
-	/* Remember volhas_attrlist if we recurse */
-	volhas_attrlist=ff_pkt->volhas_attrlist;
-
-	if((ff_pkt->flags & FO_PORTABLE))
-	{
-		if(access(fname, R_OK)==-1 && geteuid())
-		{
-			/* Could not access() directory */
-			ff_pkt->type=FT_NOACCESS;
-			ff_pkt->ff_errno=errno;
-			rtn_stat=send_file(ff_pkt, top_level, conf, cntr);
-			if(ff_pkt->linked)
-				ff_pkt->linked->FileIndex=ff_pkt->FileIndex;
-			return rtn_stat;
-		}
-	}
-
-	/*
-	* Ignore this directory and everything below if one of the files defined
-	* by the 'nobackup' option exists.
-	*/
-	if((nbret=nobackup_directory(conf, ff_pkt->fname)))
-	{
-		if(nbret<0) return -1; // error
-		return 0; // do not back it up.
-	}
-
-	/* Build a canonical directory name with a trailing slash in link var */
-	len=strlen(fname);
-	link_len=len+200;
-	if(!(link=(char *)malloc(link_len+2)))
-	{
-		logp("out of memory\n");
-		return -1;
-	}
-	snprintf(link, link_len, "%s", fname);
-
-	/* Strip all trailing slashes */
-	while(len >= 1 && IsPathSeparator(link[len - 1])) len--;
-	/* add back one */
-	link[len++]='/';
-	link[len]=0;
-
-	ff_pkt->link=link;
-	ff_pkt->type=FT_DIRBEGIN;
 	/*
 	* We have set st_rdev to 1 if it is a reparse point, otherwise 0,
 	*  if st_rdev is 2, it is a mount point 
 	*/
-#if defined(HAVE_WIN32)
 	/*
 	 * A reparse point (WIN32_REPARSE_POINT)
 	 *  is something special like one of the following:
@@ -486,6 +491,193 @@ static int found_directory(FF_PKT *ff_pkt, struct config *conf,
 	} else if (ff_pkt->statp.st_rdev == WIN32_JUNCTION_POINT) {
 		ff_pkt->type = FT_JUNCTION;
 	}
+}
+#endif
+
+static int get_files_in_directory(DIR *directory, struct dirent ***nl, int *count)
+{
+	int status;
+	int allocated=0;
+	struct dirent **ntmp=NULL;
+	struct dirent *entry=NULL;
+	struct dirent *result=NULL;
+
+	/* Graham says: this here is doing a funky kind of scandir/alphasort
+	   that can also run on Windows.
+	   TODO: split into a scandir function
+	*/
+	while(1)
+	{
+		char *p;
+		if(!(entry=(struct dirent *)malloc(
+			sizeof(struct dirent)+name_max+100)))
+		{
+			log_out_of_memory(__FUNCTION__);
+			return -1;
+		}
+		status=readdir_r(directory, entry, &result);
+		if(status || !result)
+		{
+			free(entry);
+			break;
+		}
+
+		p=entry->d_name;
+		ASSERT(name_max+1 > (int)sizeof(struct dirent)+strlen(p));
+
+		/* Skip `.', `..', and excluded file names.  */
+		if(!p || !strcmp(p, ".") || !strcmp(p, ".."))
+		{
+			free(entry);
+			continue;
+		}
+
+		if(*count==allocated)
+		{
+			if(!allocated) allocated=10;
+			else allocated*=2;
+
+			if(!(ntmp=(struct dirent **)
+				realloc (*nl, allocated*sizeof(**nl))))
+			{
+				free(entry);
+				log_out_of_memory(__FUNCTION__);
+				return -1;
+			}
+			*nl=ntmp;
+		}
+		(*nl)[(*count)++]=entry;
+	}
+	if(*nl) qsort(*nl, *count, sizeof(**nl),
+		(int (*)(const void *, const void *))myalphasort);
+	return 0;
+}
+
+/* prototype, because process_files_in_directory() recurses using find_files()
+ */
+static int
+find_files(FF_PKT *ff_pkt, struct config *conf, struct cntr *cntr,
+  char *fname, dev_t parent_device, bool top_level);
+
+static int process_files_in_directory(struct dirent **nl, int count, int *rtn_stat, char **link, size_t len, size_t *link_len, struct config *conf, struct cntr *cntr, FF_PKT *ff_pkt, dev_t our_device)
+{
+	int m=0;
+	for(m=0; m<count; m++)
+	{
+		size_t i;
+		char *p=NULL;
+		char *q=NULL;
+
+		p=nl[m]->d_name;
+
+		if(strlen(p)+len>=*link_len)
+		{
+			*link_len=len+strlen(p)+1;
+			if(!(*link=(char *)realloc(*link, (*link_len)+1)))
+			{
+				log_out_of_memory(__FUNCTION__);
+				return -1;
+			}
+		}
+		q=(*link)+len;
+		for(i=0; i<strlen(nl[m]->d_name); i++) *q++=*p++;
+		*q=0;
+
+		if(file_is_included_no_incext(conf->incexcdir, conf->iecount,
+			conf->excext, conf->excount,
+			conf->excreg, conf->ercount,
+			*link))
+		{
+			*rtn_stat=find_files(ff_pkt,
+				conf, cntr, *link, our_device, false);
+			if(ff_pkt->linked)
+				ff_pkt->linked->FileIndex = ff_pkt->FileIndex;
+		}
+		else
+		{ 
+			// Excluded, but there might be a subdirectory that is
+			// included.
+			int ex=0;
+			for(ex=0; ex<conf->iecount; ex++)
+			{
+				if(conf->incexcdir[ex]->flag
+				  && is_subdir(*link, conf->incexcdir[ex]->path))
+				{
+					int ey;
+					if((*rtn_stat=find_files(ff_pkt, conf,
+						cntr, conf->incexcdir[ex]->path,
+						 our_device, false)))
+							break;
+					// Now need to skip subdirectories of
+					// the thing that we just stuck in
+					// find_one_file(), or we might get
+					// some things backed up twice.
+					for(ey=ex+1; ey<conf->iecount; ey++)
+					  if(is_subdir(
+						conf->incexcdir[ex]->path,
+						conf->incexcdir[ey]->path))
+							ex++;
+				}
+			}
+		}
+		free(nl[m]);
+		if(*rtn_stat) break;
+	}
+	return 0;
+}
+
+static int found_directory(FF_PKT *ff_pkt, struct config *conf,
+	struct cntr *cntr, char *fname, dev_t parent_device, bool top_level,
+	struct utimbuf *restore_times)
+{
+	int rtn_stat;
+	DIR *directory;
+	char *link=NULL;
+	size_t link_len;
+	size_t len;
+	int nbret=0;
+	int count=0;
+	bool recurse;
+	dev_t our_device;
+	bool volhas_attrlist;
+	struct dirent **nl=NULL;
+
+	recurse=true;
+	our_device=ff_pkt->statp.st_dev;
+	/* Remember volhas_attrlist if we recurse */
+	volhas_attrlist=ff_pkt->volhas_attrlist;
+
+	/*
+	* Ignore this directory and everything below if one of the files defined
+	* by the 'nobackup' option exists.
+	*/
+	if((nbret=nobackup_directory(conf, ff_pkt->fname)))
+	{
+		if(nbret<0) return -1; // error
+		return 0; // do not back it up.
+	}
+
+	/* Build a canonical directory name with a trailing slash in link var */
+	len=strlen(fname);
+	link_len=len+200;
+	if(!(link=(char *)malloc(link_len+2)))
+	{
+		log_out_of_memory(__FUNCTION__);
+		return -1;
+	}
+	snprintf(link, link_len, "%s", fname);
+
+	/* Strip all trailing slashes */
+	while(len >= 1 && IsPathSeparator(link[len - 1])) len--;
+	/* add back one */
+	link[len++]='/';
+	link[len]=0;
+
+	ff_pkt->link=link;
+	ff_pkt->type=FT_DIRBEGIN;
+
+#if defined(HAVE_WIN32)
+	windows_reparse_point_fiddling(ff_pkt);
 #endif
 
 	rtn_stat=send_file(ff_pkt, top_level, conf, cntr);
@@ -556,7 +748,13 @@ static int found_directory(FF_PKT *ff_pkt, struct config *conf,
 	*   all the files in it.
 	*/
 	errno = 0;
+#ifdef O_DIRECTORY
+	int dfd;
+	if((dfd=open(fname, O_RDONLY|O_DIRECTORY|O_NOATIME))<0
+	  || !(directory=fdopendir(dfd)))
+#else
 	if(!(directory=opendir(fname)))
+#endif
 	{
 		ff_pkt->type=FT_NOOPEN;
 		ff_pkt->ff_errno=errno;
@@ -573,115 +771,25 @@ static int found_directory(FF_PKT *ff_pkt, struct config *conf,
 	*    This would possibly run faster if we chdir to the directory
 	*    before traversing it.
 	*/
-	/* Graham says: this here is doing a funky kind of scandir/alphasort
-	   that can also run on Windows.
-	   TODO: split into a scandir function
-	*/
-	while(1)
+	if(get_files_in_directory(directory, &nl, &count))
 	{
-		char *p;
-		if(!(entry=(struct dirent *)malloc(
-			sizeof(struct dirent)+name_max+100)))
-		{
-			logp("out of memory\n");
-			return -1;
-		}
-		status=readdir_r(directory, entry, &result);
-		if(status || !result)
-		{
-			free(entry);
-			break;
-		}
-
-		p=entry->d_name;
-		ASSERT(name_max+1 > (int)sizeof(struct dirent)+strlen(p));
-
-		/* Skip `.', `..', and excluded file names.  */
-		if(!p || !strcmp(p, ".") || !strcmp(p, ".."))
-		{
-			free(entry);
-			continue;
-		}
-
-		if(count==allocated)
-		{
-			if(!allocated) allocated=10;
-			else allocated*=2;
-
-			if(!(ntmp=(struct dirent **)
-				realloc (nl, allocated*sizeof(*nl))))
-			{
-				free(entry);
-				logp("out of memory\n");
-				return -1;
-			}
-			nl=ntmp;
-		}
-		nl[count++]=entry;
+		closedir(directory);
+		free(link);
+		return -1;
 	}
-	if(nl) qsort(nl, count, sizeof(*nl),
-		(int (*)(const void *, const void *))myalphasort);
 	closedir(directory);
 
 	rtn_stat=0;
-	if(nl) for(m=0; m<count; m++)
+	if(nl)
 	{
-		size_t i;
-		char *p=NULL;
-		char *q=NULL;
-
-		p=nl[m]->d_name;
-
-		if(strlen(p)+len>=link_len)
+		if(process_files_in_directory(nl, count,
+			&rtn_stat, &link, len, &link_len, conf, cntr,
+			ff_pkt, our_device))
 		{
-			link_len=len+strlen(p)+1;
-			if(!(link=(char *)realloc(link, link_len+1)))
-			{
-				logp("out of memory\n");
-				return -1;
-			}
+			free(link);
+			if(nl) free(nl);
+			return -1;
 		}
-		q=link+len;
-		for(i=0; i<strlen(nl[m]->d_name); i++) *q++=*p++;
-		*q=0;
-
-		if(file_is_included_no_incext(conf->incexcdir, conf->iecount,
-			conf->excext, conf->excount, link))
-		{
-			rtn_stat=find_files(ff_pkt,
-				conf, cntr, link, our_device, false);
-			if(ff_pkt->linked)
-				ff_pkt->linked->FileIndex = ff_pkt->FileIndex;
-		}
-		else
-		{ 
-			// Excluded, but there might be a subdirectory that is
-			// included.
-			int ex=0;
-			for(ex=0; ex<conf->iecount; ex++)
-			{
-				if(conf->incexcdir[ex]->flag
-				  && is_subdir(link, conf->incexcdir[ex]->path))
-				{
-					int ey;
-					if((rtn_stat=find_files(ff_pkt, conf,
-						cntr, conf->incexcdir[ex]->path,
-						 our_device, false)))
-							break;
-					// Now need to skip subdirectories of
-					// the thing that we just stuck in
-					// find_one_file(), or we might get
-					// some things backed up twice.
-					for(ey=ex+1; ey<conf->iecount; ey++)
-					  if(is_subdir(
-						conf->incexcdir[ex]->path,
-						conf->incexcdir[ey]->path))
-							ex++;
-				}
-			}
-		}
-		free(nl[m]);
-		if(rtn_stat) break;
 	}
 	free(link);
 	if(nl) free(nl);
@@ -693,16 +801,13 @@ static int found_directory(FF_PKT *ff_pkt, struct config *conf,
 	*  the directory modes and dates.  Temp directory values
 	*  were used without this record.
 	*/
-	if(!rtn_stat)
-	{
 		/* handle directory entry */
+	if(!rtn_stat)
 		send_file(dir_ff_pkt, top_level, conf, cntr);
-	}
 	if(ff_pkt->linked)
 		ff_pkt->linked->FileIndex = dir_ff_pkt->FileIndex;
 	free_dir_ff_pkt(dir_ff_pkt);
 
-	if(ff_pkt->flags & FO_KEEPATIME) utime(fname, restore_times);
 	/* Restore value in case it changed. */
 	ff_pkt->volhas_attrlist=volhas_attrlist;
 	return rtn_stat;
@@ -825,7 +930,7 @@ find_files(FF_PKT *ff_pkt, struct config *conf, struct cntr *cntr,
 			if(!(ff_pkt->linkhash=(link_t **)malloc(
 				LINK_HASHTABLE_SIZE*sizeof(link_t *))))
 			{
-				logp("out of memory doing link hash\n");
+				log_out_of_memory(__FUNCTION__);
 				return -1;
 			}
 			memset(ff_pkt->linkhash, 0,
@@ -856,7 +961,7 @@ find_files(FF_PKT *ff_pkt, struct config *conf, struct cntr *cntr,
 		len=strlen(fname)+1;
 		if(!(lp=(struct f_link *)malloc(sizeof(struct f_link)+len)))
 		{
-			logp("out of memory\n");
+			log_out_of_memory(__FUNCTION__);
 			return -1;
 		}
 		lp->ino=ff_pkt->statp.st_ino;
@@ -876,15 +981,34 @@ find_files(FF_PKT *ff_pkt, struct config *conf, struct cntr *cntr,
 
 	/* This is not a link to a previously dumped file, so dump it.  */
 	if(S_ISREG(ff_pkt->statp.st_mode))
-		return found_regular_file(ff_pkt,
-	conf, cntr, fname, top_level, &restore_times);
-		else if(S_ISLNK(ff_pkt->statp.st_mode))
-	return found_soft_link(ff_pkt, conf, cntr, fname, top_level);
-		else if(S_ISDIR(ff_pkt->statp.st_mode))
-	return found_directory(ff_pkt,
-		conf, cntr, fname, parent_device, top_level, &restore_times);
-
-	return found_other(ff_pkt, conf, cntr, fname, top_level);
+		return found_regular_file(ff_pkt, conf, cntr, fname,
+			top_level, &restore_times);
+	else if(S_ISLNK(ff_pkt->statp.st_mode))
+	{
+#ifdef S_IFLNK
+		/* A symlink.
+		   If they have specified the symlink in a read_blockdev
+		   argument, treat it as a block device.
+		*/
+		int i=0;
+		for(i=0; i<conf->bdcount; i++)
+		{
+			if(!strcmp(conf->blockdevs[i]->path, fname))
+			{
+				ff_pkt->statp.st_mode ^= S_IFLNK;
+				ff_pkt->statp.st_mode |= S_IFBLK;
+				return found_other(ff_pkt, conf, cntr, fname,
+					top_level);
+			}
+		}
+#endif
+		return found_soft_link(ff_pkt, conf, cntr, fname, top_level);
+	}
+	else if(S_ISDIR(ff_pkt->statp.st_mode))
+		return found_directory(ff_pkt, conf, cntr, fname,
+			parent_device, top_level, &restore_times);
+	else
+		return found_other(ff_pkt, conf, cntr, fname, top_level);
 }
 
 int find_files_begin(FF_PKT *ff_pkt, struct config *conf, char *fname, struct cntr *cntr)
