@@ -44,9 +44,9 @@ int readdir_r(DIR *dirp, struct dirent *entry, struct dirent **result);
 #define FO_NOREPLACE    (1<<7)        /* never replace */
 //#define FO_READFIFO     (1<<8)        /* read data from fifo */
 //#define FO_SHA1         (1<<9)        /* Do SHA1 checksum */
-#define FO_PORTABLE     (1<<10)       /* Use portable data format -- no BackupWrite */
+//#define FO_PORTABLE     (1<<10)       /* Use portable data format -- no BackupWrite */
 #define FO_MTIMEONLY    (1<<11)       /* Use mtime rather than mtime & ctime */
-#define FO_KEEPATIME    (1<<12)       /* Reset access time */
+//#define FO_KEEPATIME    (1<<12)       /* Reset access time */
 #define FO_EXCLUDE      (1<<13)       /* Exclude file */
 #define FO_ACL          (1<<14)       /* Backup ACLs */
 #define FO_NO_HARDLINK  (1<<15)       /* don't handle hard links */
@@ -117,8 +117,14 @@ int find_files_begin(FF_PKT *ff_pkt, struct config *conf, char *fname, struct cn
 int pathcmp(const char *a, const char *b);
 int file_is_included(struct strlist **ielist, int iecount,
 	struct strlist **incext, int incount,
-	struct strlist **excext, int excount, const char *fname,
-	bool top_level);
+	struct strlist **excext, int excount,
+	struct strlist **increg, int ircount,
+	struct strlist **excreg, int ercount,
+	const char *fname, bool top_level);
+int in_include_regex(struct strlist **incre, int incount, const char *fname);
+int in_exclude_regex(struct strlist **excre, int excount, const char *fname);
+// Returns the level of compression.
+int in_exclude_comp(struct strlist **excom, int excmcount, const char *fname, int compression);
 
 /* from attribs.c */
 void encode_stat(char *buf, struct stat *statp, int64_t winattr, int compression);
