@@ -25,22 +25,37 @@ void close_fd(int *fd)
 	*fd=-1;
 }
 
-void close_fp(FILE **fp)
+int close_fp(FILE **fp)
 {
-	if(!*fp) return;
-	fclose(*fp);
+	if(!*fp) return 0;
+	if(fclose(*fp))
+	{
+		logp("fclose failed: %s\n", strerror(errno));
+		*fp=NULL;
+		return -1;
+	}
 	*fp=NULL;
+	return 0;
 }
 
-void gzclose_fp(gzFile *fp)
+int gzclose_fp(gzFile *fp)
 {
-	if(!*fp) return;
-	gzflush(*fp, Z_FINISH);
-	gzclose(*fp);
+	int e;
+	if(!*fp) return 0;
+	if((e=gzclose(*fp)))
+	{
+		const char *str=NULL;
+		if(e==Z_ERRNO) str=strerror(errno);
+		else str=gzerror(*fp, &e);
+		logp("gzclose failed: %d (%s)\n", e, str?:"");
+		*fp=NULL;
+		return -1;
+	}
 	*fp=NULL;
+	return 0;
 }
 
-int is_dir(const char *path)
+int is_dir_lstat(const char *path)
 {
         struct stat buf;
 
@@ -49,6 +64,23 @@ int is_dir(const char *path)
         return S_ISDIR(buf.st_mode);
 }
 
+int is_dir(const char *path, struct dirent *d)
+{
+#ifdef _DIRENT_HAVE_D_TYPE
+	// Faster evaluation on most systems.
+	switch(d->d_type)
+	{
+		case DT_DIR:
+			return 1;
+		case DT_UNKNOWN:
+			break;
+		default:
+			return 0;
+	}
+#endif
+	return is_dir_lstat(path);
+}
+
 int mkpath(char **rpath, const char *limit)
 {
 	char *cp=NULL;
@@ -183,19 +215,7 @@ int do_quick_read(const char *datapth, struct cntr *cntr)
 
 char *get_checksum_str(unsigned char *checksum)
 {
-	//int i=0;
-	//char tmp[3]="";
 	static char str[64]="";
-/*
-	str[0]='\0';
-	// Windows does not seem to like me writing it all at the same time.
-	// Fuck knows why.
-	for(i=0; i<MD5_DIGEST_LENGTH; i++)
-	{
-		snprintf(tmp, sizeof(tmp), "%02x", checksum[i]);
-		strcat(str, tmp);
-	}
-*/
 	snprintf(str, sizeof(str),
 	  "%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x",
 		checksum[0], checksum[1],
@@ -209,11 +229,9 @@ char *get_checksum_str(unsigned char *checksum)
 	return str;
 }
 
-int write_endfile(unsigned long long bytes, unsigned char *checksum)
+char *get_endfile_str(unsigned long long bytes, unsigned char *checksum)
 {
-	int ret=0;
-	char endmsg[128]="";
-
+	static char endmsg[128]="";
 	snprintf(endmsg, sizeof(endmsg),
 #ifdef HAVE_WIN32
 		"%I64u:%s",
@@ -221,8 +239,12 @@ int write_endfile(unsigned long long bytes, unsigned char *checksum)
 		"%llu:%s",
 #endif
 		bytes, get_checksum_str(checksum));
-	ret=async_write_str(CMD_END_FILE, endmsg);
-	return ret;
+	return endmsg;
+}
+
+int write_endfile(unsigned long long bytes, unsigned char *checksum)
+{
+	return async_write_str(CMD_END_FILE, get_endfile_str(bytes, checksum));
 }
 
 static int do_encryption(EVP_CIPHER_CTX *ctx, unsigned char *inbuf, size_t inlen, unsigned char *outbuf, size_t *outlen, MD5_CTX *md5)
@@ -256,7 +278,7 @@ EVP_CIPHER_CTX *enc_setup(int encrypt, const char *encryption_password)
 
 	if(!(ctx=(EVP_CIPHER_CTX *)malloc(sizeof(EVP_CIPHER_CTX))))
 	{
-		logp("out of memory\n");
+		log_out_of_memory(__FUNCTION__);
 		return NULL;
 	}
         memset(ctx, 0, sizeof(EVP_CIPHER_CTX));
@@ -282,11 +304,22 @@ EVP_CIPHER_CTX *enc_setup(int encrypt, const char *encryption_password)
 	return ctx;
 }
 
-int open_file_for_send(BFILE *bfd, FILE **fp, const char *fname, int64_t winattr, struct cntr *cntr)
+#ifdef HAVE_WIN32
+struct bsid {
+	int32_t dwStreamId;
+	int32_t dwStreamAttributes;
+	int64_t Size;
+	int32_t dwStreamNameSize;
+};
+#endif
+
+int open_file_for_send(BFILE *bfd, FILE **fp, const char *fname, struct stat *statp, int64_t winattr, size_t *datalen, struct cntr *cntr)
 {
 	if(fp)
 	{
-		if(!(*fp=fopen(fname, "rb")))
+		static int fd;
+		if((fd=open(fname, O_RDONLY|O_NOATIME))<0
+		  || !(*fp=fdopen(fd, "rb")))
 		{
 			logw(cntr,
 				"Could not open %s: %s\n", fname, strerror(errno));
@@ -296,7 +329,23 @@ int open_file_for_send(BFILE *bfd, FILE **fp, const char *fname, int64_t winattr
 #ifdef HAVE_WIN32
 	else
 	{
-		binit(bfd, winattr);
+		if(bfd->mode!=BF_CLOSED)
+		{
+			if(bfd->path && !strcmp(bfd->path, fname))
+			{
+				// Already open after reading the VSS data.
+				// Time now for the actual file data.
+				return 0;
+			}
+			else
+			{
+				// Close the open bfd so that it can be
+				// used again
+				close_file_for_send(bfd, NULL);
+			}
+		}
+		binit(bfd, statp, winattr);
+		*datalen=0;
 		if(bopen(bfd, fname, O_RDONLY | O_BINARY | O_NOATIME, 0,
 			(winattr & FILE_ATTRIBUTE_DIRECTORY))<=0)
 		{
@@ -310,12 +359,13 @@ int open_file_for_send(BFILE *bfd, FILE **fp, const char *fname, int64_t winattr
 	return 0;
 }
 
-void close_file_for_send(BFILE *bfd, FILE **fp)
+int close_file_for_send(BFILE *bfd, FILE **fp)
 {
-	if(fp) close_fp(fp);
+	if(fp) return close_fp(fp);
 #ifdef HAVE_WIN32
-	if(bfd) bclose(bfd);
+	if(bfd) return bclose(bfd);
 #endif
+	return -1;
 }
 
 /* OK, this function is getting a bit out of control.
@@ -327,7 +377,7 @@ void close_file_for_send(BFILE *bfd, FILE **fp)
    Encryption off and compression off uses send_whole_file().
    Perhaps a separate function is needed for encryption on compression off.
 */
-int send_whole_file_gz(const char *fname, const char *datapth, int quick_read, unsigned long long *bytes, const char *encpassword, struct cntr *cntr, int compression, BFILE *bfd, FILE *fp, const char *extrameta, size_t elen)
+int send_whole_file_gz(const char *fname, const char *datapth, int quick_read, unsigned long long *bytes, const char *encpassword, struct cntr *cntr, int compression, BFILE *bfd, FILE *fp, const char *extrameta, size_t elen, size_t datalen)
 {
 	int ret=0;
 	int zret=0;
@@ -345,6 +395,10 @@ int send_whole_file_gz(const char *fname, const char *datapth, int quick_read, u
 	unsigned char eoutbuf[ZCHUNK+EVP_MAX_BLOCK_LENGTH];
 
 	EVP_CIPHER_CTX *enc_ctx=NULL;
+#ifdef HAVE_WIN32
+	int do_known_byte_count=0;
+	if(datalen>0) do_known_byte_count=1;
+#endif
 
 	if(encpassword && !(enc_ctx=enc_setup(1, encpassword)))
 		return -1;
@@ -389,10 +443,28 @@ int send_whole_file_gz(const char *fname, const char *datapth, int quick_read, u
 		{
 			if(fp) strm.avail_in=fread(in, 1, ZCHUNK, fp);
 #ifdef HAVE_WIN32
-			else strm.avail_in=(uint32_t)bread(bfd, in, ZCHUNK);
+			else
+			{
+			  // Windows VSS headers give us how much data to
+			  // expect to read.
+			  if(do_known_byte_count)
+			  {
+				  if(datalen<=0) strm.avail_in=0;
+				  else strm.avail_in=
+					 (uint32_t)bread(bfd, in,
+						min((size_t)ZCHUNK, datalen));
+				  datalen-=strm.avail_in;
+			  }
+			  else
+			  {
+				  strm.avail_in=
+					 (uint32_t)bread(bfd, in, ZCHUNK);
+			  }
+			}
 #endif
 		}
 		if(!compression && !strm.avail_in) break;
+
 		if(strm.avail_in<0)
 		{
 			logp("Error in read: %d\n", strm.avail_in);
@@ -412,6 +484,10 @@ int send_whole_file_gz(const char *fname, const char *datapth, int quick_read, u
 			}
 		}
 
+#ifdef HAVE_WIN32
+		if(do_known_byte_count && datalen<=0) flush=Z_FINISH;
+		else
+#endif
 		if(strm.avail_in) flush=Z_NO_FLUSH;
 		else flush=Z_FINISH;
 
@@ -568,7 +644,7 @@ static DWORD WINAPI write_efs(PBYTE pbData, PVOID pvCallbackContext, ULONG ulLen
 }
 #endif
 
-int send_whole_file(char cmd, const char *fname, const char *datapth, int quick_read, unsigned long long *bytes, struct cntr *cntr, BFILE *bfd, FILE *fp, const char *extrameta, size_t elen)
+int send_whole_file(char cmd, const char *fname, const char *datapth, int quick_read, unsigned long long *bytes, struct cntr *cntr, BFILE *bfd, FILE *fp, const char *extrameta, size_t elen, size_t datalen)
 {
 	int ret=0;
 	size_t s=0;
@@ -636,8 +712,22 @@ int send_whole_file(char cmd, const char *fname, const char *datapth, int quick_
 
 		if(!ret && cmd!=CMD_EFS_FILE)
 		{
-		  while((s=(uint32_t)bread(bfd, buf, 4096))>0)
+		  int do_known_byte_count=0;
+		  if(datalen>0) do_known_byte_count=1;
+		  while(1)
 		  {
+			if(do_known_byte_count)
+			{
+				s=(uint32_t)bread(bfd,
+					buf, min((size_t)4096, datalen));
+				datalen-=s;
+			}
+			else
+			{
+				s=(uint32_t)bread(bfd, buf, 4096);
+			}
+			if(s<=0) break;
+
 			*bytes+=s;
 			if(!MD5_Update(&md5, buf, s))
 			{
@@ -664,6 +754,9 @@ int send_whole_file(char cmd, const char *fname, const char *datapth, int quick_
 					break;
 				}
 			}
+			// Windows VSS headers tell us how many bytes to
+			// expect.
+			if(do_known_byte_count && datalen<=0) break;
 		  }
 		}
 #else
@@ -789,6 +882,7 @@ int init_client_socket(const char *host, const char *port)
 		close_fd(&rfd);
 		return -1;
 	}
+	reuseaddr(rfd);
 
 #ifdef HAVE_WIN32
 	setmode(rfd, O_BINARY);
@@ -798,7 +892,7 @@ int init_client_socket(const char *host, const char *port)
 
 void reuseaddr(int fd)
 {
-	int tmpfd;
+	int tmpfd=0;
 	if(setsockopt(fd, SOL_SOCKET, SO_REUSEADDR,
 		(sockopt_val_t)&tmpfd, sizeof(tmpfd))<0)
 			logp("Error: setsockopt SO_REUSEADDR: %s",
@@ -806,87 +900,6 @@ void reuseaddr(int fd)
 }
 
 #ifndef HAVE_WIN32
-static void get_status_buf(char *buf, size_t len, const char *client, char phase, const char *path, struct cntr *p1cntr, struct cntr *cntr)
-{
-	int l=0;
-	snprintf(buf, len,
-		"%s\t%c\t%c\t"
-		"%llu/%llu/%llu/%llu\t"
-		"%llu/%llu/%llu/%llu\t"
-		"%llu/%llu/%llu/%llu\t"
-		"%llu/%llu/%llu/%llu\t"
-		"%llu/%llu/%llu/%llu\t"
-		"%llu/%llu/%llu/%llu\t"
-		"%llu/%llu/%llu/%llu\t"
-		"%llu/%llu/%llu/%llu\t"
-		"%llu/%llu/%llu/%llu\t"
-		"%llu/%llu/%llu/%llu\t"
-		"%llu\t%llu\t%llu\t%llu\t%llu\t%llu\t%li\t%s\n",
-			client, STATUS_RUNNING, phase,
-
-			cntr->total,
-			cntr->total_changed,
-			cntr->total_same,
-			p1cntr->total,
-
-			cntr->file,
-			cntr->file_changed,
-			cntr->file_same,
-			p1cntr->file,
-
-			cntr->enc,
-			cntr->enc_changed,
-			cntr->enc_same,
-			p1cntr->enc,
-
-			cntr->meta,
-			cntr->meta_changed,
-			cntr->meta_same,
-			p1cntr->meta,
-
-			cntr->encmeta,
-			cntr->encmeta_changed,
-			cntr->encmeta_same,
-			p1cntr->encmeta,
-
-			cntr->dir,
-			cntr->dir_changed,
-			cntr->dir_same,
-			p1cntr->dir,
-
-			cntr->slink,
-			cntr->slink_changed,
-			cntr->slink_same,
-			p1cntr->slink,
-
-			cntr->hlink,
-			cntr->hlink_changed,
-			cntr->hlink_same,
-			p1cntr->hlink,
-
-			cntr->special,
-			cntr->special_changed,
-			cntr->special_same,
-			p1cntr->special,
-
-			cntr->total,
-			cntr->total_changed,
-			cntr->total_same,
-			p1cntr->total,
-
-			cntr->gtotal,
-			cntr->warning,
-			p1cntr->byte,
-			cntr->byte,
-			cntr->recvbyte,
-			cntr->sentbyte,
-			p1cntr->start,
-			path?path:"");
-
-	// Make sure there is a new line at the end.
-	l=strlen(buf);
-	if(buf[l-1]!='\n') buf[l-1]='\n';
-}
 
 void write_status(const char *client, char phase, const char *path, struct cntr *p1cntr, struct cntr *cntr)
 {
@@ -910,7 +923,7 @@ void write_status(const char *client, char phase, const char *path, struct cntr
 		}
 		lasttime=now;
 
-		get_status_buf(wbuf, sizeof(wbuf),
+		counters_to_str(wbuf, sizeof(wbuf),
 			client, phase, path, p1cntr, cntr);
 
 		w=wbuf;
@@ -928,7 +941,25 @@ void write_status(const char *client, char phase, const char *path, struct cntr
 	}
 }
 
-static void log_script_output(FILE **fp, struct cntr *cntr, int logfunc)
+static int astrcat(char **buf, const char *append)
+{
+	int l=0;
+	char *copy=NULL;
+	if(append) l+=strlen(append);
+	if(*buf) l+=strlen(*buf);
+	l++;
+	if((*buf && !(copy=strdup(*buf)))
+	  || !(*buf=(char *)realloc(*buf, l)))
+	{
+		logp("out of memory in %s.\n", __func__);
+		return -1;
+	}
+	snprintf(*buf, l, "%s%s", copy?copy:"", append?append:"");
+	if(copy) free(copy);
+	return 0;
+}
+
+static int log_script_output(FILE **fp, struct cntr *cntr, int logfunc, char **logbuf)
 {
 	char buf[256]="";
 	if(fp && *fp)
@@ -938,6 +969,7 @@ static void log_script_output(FILE **fp, struct cntr *cntr, int logfunc)
 			// logc does not print a prefix
 			if(logfunc) logp("%s", buf);
 			else logc("%s", buf);
+			if(logbuf && astrcat(logbuf, buf)) return -1;
 			if(cntr) logw(cntr, "%s", buf);
 		}
 		if(feof(*fp))
@@ -946,6 +978,7 @@ static void log_script_output(FILE **fp, struct cntr *cntr, int logfunc)
 			*fp=NULL;
 		}
 	}
+	return 0;
 }
 
 static int got_sigchld=0;
@@ -967,7 +1000,7 @@ void setup_signal(int sig, void handler(int sig))
 	sigaction(sig, &sa, NULL);
 }
 
-static int run_script_select(FILE **sout, FILE **serr, struct cntr *cntr, int logfunc)
+static int run_script_select(FILE **sout, FILE **serr, struct cntr *cntr, int logfunc, char **logbuf)
 {
 	int mfd=-1;
 	fd_set fsr;
@@ -991,15 +1024,21 @@ static int run_script_select(FILE **sout, FILE **serr, struct cntr *cntr, int lo
 		{
 			if(errno!=EAGAIN && errno!=EINTR)
 			{
-				logp("run_script_select error: %s\n",
+				logp("%s error: %s\n", __func__,
 					strerror(errno));
 				return -1;
 			}
 		}
 		if(FD_ISSET(soutfd, &fsr))
-			log_script_output(sout, NULL, logfunc);
+		{
+			if(log_script_output(sout, NULL, logfunc, logbuf))
+				return -1;
+		}
 		if(FD_ISSET(serrfd, &fsr))
-			log_script_output(serr, cntr, logfunc);
+		{
+			if(log_script_output(serr, cntr, logfunc, logbuf))
+				return -1;
+		}
 
 		if(!*sout && !*serr && got_sigchld)
 		{
@@ -1016,8 +1055,7 @@ static int run_script_select(FILE **sout, FILE **serr, struct cntr *cntr, int lo
 
 #endif
 
-/* TODO: make arg1..n an array */
-int run_script(const char *script, struct strlist **userargs, int userargc, const char *arg1, const char *arg2, const char *arg3, const char *arg4, const char *arg5, const char *arg6, const char *arg7, const char *arg8, const char *arg9, const char *arg10, struct cntr *cntr, int do_wait, int logfunc)
+int run_script_to_buf(const char **args, struct strlist **userargs, int userargc, struct cntr *cntr, int do_wait, int logfunc, char **logbuf)
 {
 	int a=0;
 	int l=0;
@@ -1028,19 +1066,9 @@ int run_script(const char *script, struct strlist **userargs, int userargc, cons
 #ifndef HAVE_WIN32
 	int s=0;
 #endif
-	if(!script) return 0;
+	if(!args || !args[0]) return 0;
 
-	cmd[l++]=(char *)script;
-	if(arg1) cmd[l++]=(char *)arg1;
-	if(arg2) cmd[l++]=(char *)arg2;
-	if(arg3) cmd[l++]=(char *)arg3;
-	if(arg4) cmd[l++]=(char *)arg4;
-	if(arg5) cmd[l++]=(char *)arg5;
-	if(arg6) cmd[l++]=(char *)arg6;
-	if(arg7) cmd[l++]=(char *)arg7;
-	if(arg8) cmd[l++]=(char *)arg8;
-	if(arg9) cmd[l++]=(char *)arg9;
-	if(arg10) cmd[l++]=(char *)arg10;
+	for(a=0; args[a]; a++) cmd[l++]=(char *)args[a];
 	for(a=0; a<userargc && l<64-1; a++)
 		cmd[l++]=userargs[a]->path;
 	cmd[l++]=NULL;
@@ -1065,7 +1093,7 @@ int run_script(const char *script, struct strlist **userargs, int userargc, cons
 	// My windows forkchild currently just executes, then returns.
 	return 0;
 #else
-	s=run_script_select(&sout, &serr, cntr, logfunc);
+	s=run_script_select(&sout, &serr, cntr, logfunc, logbuf);
 
 	// Set SIGCHLD back to default.
 	setup_signal(SIGCHLD, SIG_DFL);
@@ -1075,28 +1103,34 @@ int run_script(const char *script, struct strlist **userargs, int userargc, cons
 	if(WIFEXITED(run_script_status))
 	{
 		int ret=WEXITSTATUS(run_script_status);
-		logp("%s returned: %d\n", script, ret);
+		logp("%s returned: %d\n", cmd[0], ret);
 		if(cntr && ret) logw(cntr, "%s returned: %d\n",
-			script, ret);
+			cmd[0], ret);
 		return ret;
 	}
 	else if(WIFSIGNALED(run_script_status))
 	{
 		logp("%s terminated on signal %d\n",
-			script, WTERMSIG(run_script_status));
+			cmd[0], WTERMSIG(run_script_status));
 		if(cntr) logw(cntr, "%s terminated on signal %d\n",
-			script, WTERMSIG(run_script_status));
+			cmd[0], WTERMSIG(run_script_status));
 	}
 	else
 	{
-		logp("Strange return when trying to run %s\n", script);
+		logp("Strange return when trying to run %s\n", cmd[0]);
 		if(cntr) logw(cntr, "Strange return when trying to run %s\n",
-			script);
+			cmd[0]);
 	}
 	return -1;
 #endif
 }
 
+int run_script(const char **args, struct strlist **userargs, int userargc, struct cntr *cntr, int do_wait, int logfunc)
+{
+	return run_script_to_buf(args, userargs, userargc, cntr, do_wait,
+		logfunc, NULL /* do not save output to buffer */);
+}
+
 char *comp_level(struct config *conf)
 {
 	static char comp[8]="";
@@ -1138,7 +1172,7 @@ int chuser_and_or_chgrp(const char *user, const char *group)
 	// Any OS uname pointer may get overwritten, so save name, uid, and gid
 	if(!(username=strdup(user)))
 	{
-		logp("out of memory\n");
+		log_out_of_memory(__FUNCTION__);
 		return -1;
 	}
 	uid=passw->pw_uid;
@@ -1277,48 +1311,22 @@ void cmd_to_text(char cmd, char *buf, size_t len)
 			snprintf(buf, len, "Warning"); break;
 		case CMD_END_FILE:
 			snprintf(buf, len, "End of file transmission"); break;
-		case CMD_FILE_CHANGED:
-			snprintf(buf, len, "Plain file changed"); break;
-		case CMD_FILE_SAME:
-			snprintf(buf, len, "Plain file unchanged"); break;
-		case CMD_METADATA_CHANGED:
-			snprintf(buf, len, "Meta data changed"); break;
-		case CMD_METADATA_SAME:
-			snprintf(buf, len, "Meta data unchanged"); break;
 		case CMD_ENC_METADATA:
 			snprintf(buf, len, "Encrypted meta data"); break;
-		case CMD_ENC_METADATA_CHANGED:
-			snprintf(buf, len, "Encrypted meta data changed"); break;
-		case CMD_ENC_METADATA_SAME:
-			snprintf(buf, len, "Encrypted meta data unchanged"); break;
-		case CMD_ENC_FILE_CHANGED:
-			snprintf(buf, len, "Encrypted file changed"); break;
-		case CMD_ENC_FILE_SAME:
-			snprintf(buf, len, "Encrypted file unchanged"); break;
 		case CMD_EFS_FILE:
 			snprintf(buf, len, "Windows EFS file"); break;
-		case CMD_EFS_FILE_SAME:
-			snprintf(buf, len, "Windows EFS file changed"); break;
-		case CMD_EFS_FILE_CHANGED:
-			snprintf(buf, len, "Windows EFS file unchanged"); break;
-		case CMD_DIRECTORY_CHANGED:
-			snprintf(buf, len, "Directory changed"); break;
-		case CMD_DIRECTORY_SAME:
-			snprintf(buf, len, "Directory unchanged"); break;
-		case CMD_HARD_LINK_CHANGED:
-			snprintf(buf, len, "Hard link changed"); break;
-		case CMD_HARD_LINK_SAME:
-			snprintf(buf, len, "Hard link unchanged"); break;
-		case CMD_SOFT_LINK_CHANGED:
-			snprintf(buf, len, "Soft link changed"); break;
-		case CMD_SOFT_LINK_SAME:
-			snprintf(buf, len, "Soft link unchanged"); break;
-		case CMD_SPECIAL_CHANGED:
-			snprintf(buf, len, "Special file changed"); break;
-		case CMD_SPECIAL_SAME:
-			snprintf(buf, len, "Special file unchanged"); break;
+		case CMD_FILE_CHANGED:
+			snprintf(buf, len, "Plain file changed"); break;
 		case CMD_TIMESTAMP:
 			snprintf(buf, len, "Backup timestamp"); break;
+		case CMD_VSS:
+			snprintf(buf, len, "Windows VSS header"); break;
+		case CMD_ENC_VSS:
+			snprintf(buf, len, "Encrypted windows VSS header"); break;
+		case CMD_VSS_T:
+			snprintf(buf, len, "Windows VSS footer"); break;
+		case CMD_ENC_VSS_T:
+			snprintf(buf, len, "Encrypted windows VSS footer"); break;
 		default:
 			snprintf(buf, len, "----------------"); break;
 	}
@@ -1343,6 +1351,8 @@ void log_restore_settings(struct config *cconf, int srestore)
 {
 	int i=0;
 	logp("Restore settings:\n");
+	if(cconf->orig_client)
+		logp("orig_client = %s\n", cconf->orig_client);
 	logp("backup = %s\n", cconf->backup);
 	if(srestore)
 	{
@@ -1370,7 +1380,7 @@ long version_to_long(const char *version)
 	if(!version || !*version) return 0;
 	if(!(copy=strdup(version)))
 	{
-		logp("out of memory\n");
+		log_out_of_memory(__FUNCTION__);
 		return -1;
 	}
 	if(!(tok1=strtok(copy, "."))
@@ -1391,6 +1401,7 @@ long version_to_long(const char *version)
    and the CA stuff, rather than backups/restores. */
 int receive_a_file(const char *path, struct cntr *p1cntr)
 {
+	int c=0;
 	int ret=0;
 #ifdef HAVE_WIN32
 	BFILE bfd;
@@ -1401,7 +1412,7 @@ int receive_a_file(const char *path, struct cntr *p1cntr)
 	unsigned long long sentbytes=0;
 
 #ifdef HAVE_WIN32
-	binit(&bfd, 0);
+	binit(&bfd, NULL, 0);
 	bfd.use_backup_api=0;
 	//set_win32_backup(&bfd);
 	if(bopen(&bfd, path,
@@ -1425,13 +1436,18 @@ int receive_a_file(const char *path, struct cntr *p1cntr)
 #ifdef HAVE_WIN32
 	ret=transfer_gzfile_in(NULL, path, &bfd, NULL,
 		&rcvdbytes, &sentbytes, NULL, 0, p1cntr, NULL);
-	bclose(&bfd);
+	c=bclose(&bfd);
 #else
 	ret=transfer_gzfile_in(NULL, path, NULL, fp,
 		&rcvdbytes, &sentbytes, NULL, 0, p1cntr, NULL);
-	close_fp(&fp);
+	c=close_fp(&fp);
 #endif
 end:
+	if(c)
+	{
+		logp("error closing %s in receive_a_file\n", path);
+		ret=-1;
+	}
 	if(!ret) logp("Received: %s\n", path);
 	return ret;
 }
@@ -1443,11 +1459,12 @@ int send_a_file(const char *path, struct cntr *p1cntr)
 {
 	int ret=0;
 	FILE *fp=NULL;
+	size_t datalen=0;
 	unsigned long long bytes=0;
-	if(open_file_for_send(NULL, &fp, path, 0, p1cntr)
+	if(open_file_for_send(NULL, &fp, path, NULL, 0, &datalen, p1cntr)
 	  || send_whole_file_gz(path, "datapth", 0, &bytes, NULL,
 		p1cntr, 9, // compression
-		NULL, fp, NULL, 0))
+		NULL, fp, NULL, 0, -1))
 	{
 		ret=-1;
 		goto end;
