@@ -9,18 +9,18 @@
 #include "counter.h"
 #include "dpth.h"
 #include "sbuf.h"
+#include "regexp.h"
 #include "list_server.h"
 #include "current_backups_server.h"
 
-int check_browsedir(const char *browsedir, char **path, size_t bdlen, char **lastpath)
+int check_browsedir(const char *browsedir, char **path, size_t bdlen, char **last_bd_match)
 {
 	char *cp=NULL;
 	char *copy=NULL;
-//	if(strncmp(browsedir, *path, bdlen)
-//	  || (bdlen && (*path)[bdlen]!='\0' && (*path)[bdlen]!='/'))
 	if(strncmp(browsedir, *path, bdlen))
 		return 0;
-	if((*path)[bdlen+1]=='\0') return 0;
+	if((*path)[bdlen+1]=='\0' || (bdlen>1 && (*path)[bdlen]!='/'))
+		return 0;
 
 	/* Lots of messing around related to whether browsedir was '', '/', or
    	   something else. */
@@ -42,30 +42,30 @@ int check_browsedir(const char *browsedir, char **path, size_t bdlen, char **las
 	else
 	{
 		if(!(copy=strdup((*path)+bdlen)))
-		{
-			logp("out of memory\n");
-			return -1;
-		}
+			goto err;
 		if(*copy=='/') *(copy+1)='\0';
 		// Messing around for Windows.
 		else if(strlen(copy)>2 && copy[1]==':' && copy[2]=='/')
 			copy[2]='\0';
 	}
-	if(*lastpath && !strcmp(*lastpath, copy))
+	if(*last_bd_match)
 	{
-		free(copy);
-		return 0;
+		if(!strcmp(*last_bd_match, copy))
+		{
+			// Got a duplicate match.
+			free(copy);
+			return 0;
+		}
+		free(*last_bd_match);
 	}
 	free(*path);
 	*path=copy;
-	if(*lastpath) free(*lastpath);
-	if(!(*lastpath=strdup(copy)))
+	if(!(*last_bd_match=strdup(copy)))
 		goto err;
 	return 1;
 err:
 	if(copy) free(copy);
-	if(*lastpath) free(*lastpath);
-	logp("out of memory\n");
+	log_out_of_memory(__FUNCTION__);
 	return -1;
 }
 
@@ -77,15 +77,15 @@ static int list_manifest(const char *fullpath, regex_t *regex, const char *brows
 	gzFile zp=NULL;
 	struct sbuf mb;
 	char *manifest=NULL;
+	char *last_bd_match=NULL;
 	size_t bdlen=0;
-	char *lastpath=NULL;
 
 	init_sbuf(&mb);
 
 	if(!(manifest=prepend_s(fullpath,
 		"manifest.gz", strlen("manifest.gz"))))
 	{
-		log_and_send("out of memory");
+		log_and_send_oom(__FUNCTION__);
 		return -1;
 	}
 	if(!(zp=gzopen_file(manifest, "rb")))
@@ -112,6 +112,14 @@ static int list_manifest(const char *fullpath, regex_t *regex, const char *brows
 			break;
 		}
 
+		if(mb.cmd!=CMD_DIRECTORY
+		 && mb.cmd!=CMD_FILE
+		 && mb.cmd!=CMD_ENC_FILE
+		 && mb.cmd!=CMD_EFS_FILE
+		 && mb.cmd!=CMD_SPECIAL
+		 && !cmd_is_link(mb.cmd))
+			continue;
+
 		//if(mb.path[mb.plen]=='\n') mb.path[mb.plen]='\0';
 		write_status(client, STATUS_LISTING, mb.path, p1cntr, cntr);
 
@@ -119,7 +127,7 @@ static int list_manifest(const char *fullpath, regex_t *regex, const char *brows
 		{
 			int r;
 			if((r=check_browsedir(browsedir,
-				&(mb.path), bdlen, &lastpath))<0)
+				&(mb.path), bdlen, &last_bd_match))<0)
 			{
 				quit++;
 				ret=-1;
@@ -144,10 +152,18 @@ static int list_manifest(const char *fullpath, regex_t *regex, const char *brows
 	}
 	gzclose_fp(&zp);
 	free_sbuf(&mb);
-	if(lastpath) free(lastpath);
+	if(last_bd_match) free(last_bd_match);
 	return ret;
 }
 
+static int send_backup_name_to_client(struct bu *arr)
+{
+	char msg[64]="";
+	snprintf(msg, sizeof(msg), "%s%s",
+		arr->timestamp, arr->deletable?" (deletable)":"");
+	async_write(CMD_TIMESTAMP, msg, strlen(msg));
+}
+
 int do_list_server(const char *basedir, const char *backup, const char *listregex, const char *browsedir, const char *client, struct cntr *p1cntr, struct cntr *cntr)
 {
 	int a=0;
@@ -191,8 +207,7 @@ int do_list_server(const char *basedir, const char *backup, const char *listrege
 				|| arr[i].index==index))
 			{
 				found=TRUE;
-				async_write(CMD_TIMESTAMP,
-				  arr[i].timestamp, strlen(arr[i].timestamp));
+				send_backup_name_to_client(&(arr[i]));
 				ret=list_manifest(arr[i].path, regex,
 					browsedir, client, p1cntr, cntr);
 			}
@@ -201,8 +216,7 @@ int do_list_server(const char *basedir, const char *backup, const char *listrege
 		else
 		{
 			found=TRUE;
-			async_write(CMD_TIMESTAMP,
-				arr[i].timestamp, strlen(arr[i].timestamp));
+			send_backup_name_to_client(&(arr[i]));
 		}
 	}
 	free_current_backups(&arr, a);
