@@ -3,9 +3,10 @@
 #include "log.h"
 #include "handy.h"
 
+// Return 0 for lock got, 1 for lock not got, -1 for error.
 int get_lock(const char *path)
 {
-#ifdef HAVE_WIN32
+#if defined(HAVE_WIN32) || !defined(HAVE_LOCKF)
 	// Would somebody please tell me how to get a lock on Windows?!
 	return 0;
 #else
@@ -30,11 +31,16 @@ int get_lock(const char *path)
 	if((fdlock=open(path, O_WRONLY|O_CREAT, 0666))==-1)
 		return -1;
 	if(lockf(fdlock, F_TLOCK, 0))
-		return -1;
-	snprintf(text, sizeof(text), "%s\n%d\n", progname(), (int)getpid());
+	{
+		if(errno==EACCES || errno==EAGAIN)
+			return 1; // Lock not got.
+		logp("Could not get lock %s: %s\n", path, strerror(errno));
+		return -1; // Some other error.
+	}
+	snprintf(text, sizeof(text), "%d\n%s\n", (int)getpid(), progname());
 	if(write(fdlock, text, strlen(text))!=(ssize_t)strlen(text))
 	{
-		logp("Could not write progname/pid to %s\n", path);
+		logp("Could not write pid/progname to %s\n", path);
 		return -1;
 	}
 	fsync(fdlock); // Make sure the pid gets onto the disk.
@@ -46,7 +52,7 @@ int get_lock(const char *path)
 
 int test_lock(const char *path)
 {
-#ifdef HAVE_WIN32
+#if defined(HAVE_WIN32) || !defined(HAVE_LOCKF)
 	// Would somebody please tell me how to test a lock on Windows?!
 	return 0;
 #else
@@ -67,3 +73,15 @@ int test_lock(const char *path)
 	return 0;
 #endif
 }
+
+
+// In this source file so that both bedup and status_server can see it.
+int looks_like_tmp_or_hidden_file(const char *filename)
+{
+	if(!filename) return 0;
+	if(filename[0]=='.' // Also avoids '.' and '..'.
+	// I am told that emacs tmp files end with '~'.
+	  || filename[strlen(filename)-1]=='~')
+		return 1;
+	return 0;
+}
