@@ -5,8 +5,8 @@ extern void init_log(char *progname);
 extern void logp(const char *fmt, ...);
 extern void logc(const char *fmt, ...);
 extern const char *progname(void);
-extern int set_logfp(FILE *fp, struct config *conf);
+extern int set_logfp(const char *path, struct config *conf);
 extern FILE *get_logfp(void);
-extern int open_logfile(const char *logfile, struct config *conf);
+extern void log_out_of_memory(const char *function);
 
 #endif
