@@ -9,7 +9,20 @@ static char *get_config_path(void)
 {
 	static char path[256]="";
 #ifdef HAVE_WIN32
-	snprintf(path, sizeof(path), "%s", "C:/Program Files/Burp/burp.conf");
+	char *pfenv=NULL;
+
+	// Burp used to always install to 'C:/Program Files/Burp/', but as
+	// of 1.3.11, it changed to %PROGRAMFILES%. Still want the old way
+	// to work though. So check %PROGRAMFILES% first, then fall back.
+	if((pfenv=getenv("PROGRAMFILES")))
+	{
+		struct stat statp;
+		snprintf(path, sizeof(path), "%s/Burp/burp.conf", pfenv);
+		if(!lstat(path, &statp)
+		  && !S_ISDIR(statp.st_mode))
+			return path;
+	}
+	snprintf(path, sizeof(path), "C:/Program Files/Burp/burp.conf");
 #else
 	snprintf(path, sizeof(path), "%s", SYSCONFDIR "/burp.conf");
 #endif
@@ -50,6 +63,7 @@ static void usage_client(void)
 	printf(" Options:\n");
 	printf("  -a <action>    The action can be one of the following.\n");
 	printf("                  b: backup\n");
+	printf("                  D: delete\n");
 	printf("                  e: estimate\n");
 	printf("                  l: list (this is the default when an action is not given)\n");
 	printf("                  L: long list\n");
@@ -65,7 +79,11 @@ static void usage_client(void)
 	printf("  -l <path>      Path to log file.\n");
 	printf("  -r <regex>     Specify a regular expression.\n");
 	printf("  -s <number>    Number of leading path components to strip during restore.\n");
+	printf("  -j             Format long list as JSON.\n");
 	printf("  -v             Print version and exit.\n");
+#ifndef HAVE_WIN32
+	printf("  -x             Do not use the Windows VSS API when restoring.\n");
+#endif
 	printf("\n");
 #ifndef HAVE_WIN32
 	printf(" See http://burp.grke.net/ or the man page ('man burp') for usage examples\n");
@@ -76,24 +94,20 @@ static void usage_client(void)
 #endif
 }
 
-int reload(struct config *conf, const char *configfile, char **logfile, bool firsttime, int oldmax_children, int oldmax_status_children)
+int reload(struct config *conf, const char *configfile, bool firsttime, int oldmax_children, int oldmax_status_children)
 {
 	if(!firsttime) logp("Reloading config\n");
 
 	init_config(conf);
 
-	// If logfile is defined, use it
-	// Have to do this twice, because init_config uses logp().
-	if(open_logfile(*logfile, conf)) return 1;
-
 	if(load_config(configfile, conf, TRUE)) return 1;
 
 	/* change umask */
 	umask(conf->umask);
 
-	// Open the second time. This will turn on syslogging which could not
-	// be turned on before load_config.
-	if(open_logfile(*logfile, conf)) return 1;
+	// This will turn on syslogging which could not be turned on before
+	// load_config.
+	set_logfp(NULL, conf);
 
 #ifndef HAVE_WIN32
 	if(conf->mode==MODE_SERVER)
@@ -105,13 +119,6 @@ int reload(struct config *conf, const char *configfile, char **logfile, bool fir
 	if(firsttime && chuser_and_or_chgrp(conf->user, conf->group))
 		return 1;
 
-	// If logfile is defined in config...
-	if(conf->logfile)
-	{
-		*logfile=conf->logfile;
-		if(open_logfile(*logfile, conf)) return 1;
-	}
-
 	return 0;
 }
 
@@ -122,7 +129,7 @@ static int replace_conf_str(const char *newval, char **dest)
 		if(*dest) free(*dest);
 		if(!(*dest=strdup(newval)))
 		{
-			logp("out of memory\n");
+			log_out_of_memory(__FUNCTION__);
 			return -1;
 		}
 	}
@@ -146,7 +153,6 @@ int main (int argc, char *argv[])
 	int forking=1;
 	int gotlock=0;
 	int strip=0;
-	char *logfile=NULL;
 	struct config conf;
 	int forceoverwrite=0;
 	enum action act=ACTION_LIST;
@@ -157,14 +163,19 @@ int main (int argc, char *argv[])
 	const char *browsedir=NULL;
 	FILE *fp=NULL;
 	const char *configfile=get_config_path();
+	const char *orig_client=NULL;
+	// The orig_client is the original client that the normal client
+	// would like to restore from.
 #ifndef HAVE_WIN32
-	const char *sclient=NULL;
+	const char *sclient=NULL; // Status monitor client to view.
 	int generate_ca_only=0;
 #endif
+	int vss_restore=1;
+	int json=0;
 
 	init_log(argv[0]);
 
-	while((option=getopt(argc, argv, "a:b:c:C:d:ghfFinr:s:l:vz:?"))!=-1)
+	while((option=getopt(argc, argv, "a:b:c:C:d:ghfFil:nr:s:vxjz:?"))!=-1)
 	{
 		switch(option)
 		{
@@ -187,6 +198,10 @@ int main (int argc, char *argv[])
 					act=ACTION_STATUS_SNAPSHOT;
 				else if(!strncmp(optarg, "estimate", 1))
 					act=ACTION_ESTIMATE;
+				// Start 'Delete' with a capital letter so that it is less
+				// likely to be used accidently.
+				else if(!strncmp(optarg, "Delete", 1))
+					act=ACTION_DELETE;
 				else
 				{
 					usage();
@@ -200,6 +215,7 @@ int main (int argc, char *argv[])
 				configfile=optarg;
 				break;
 			case 'C':
+				orig_client=optarg;
 #ifndef HAVE_WIN32
 				sclient=optarg;
 #endif
@@ -222,6 +238,9 @@ int main (int argc, char *argv[])
 			case 'i':
 				print_all_cmds();
 				return 0;
+			case 'l':
+				logp("-l <logfile> option obsoleted\n");
+				break;
 			case 'n':
 				forking=0;
 				break;
@@ -231,12 +250,15 @@ int main (int argc, char *argv[])
 			case 's':
 				strip=atoi(optarg);
 				break;
-			case 'l':
-				logfile=optarg;
-				break;
 			case 'v':
 				printf("%s-%s\n", progname(), VERSION);
 				return 0;
+			case 'x':
+				vss_restore=0;
+				break;
+			case 'j':
+				json=1;
+				break;
 			case 'z':
 				browsefile=optarg;
 				break;
@@ -254,7 +276,7 @@ int main (int argc, char *argv[])
 		return 1;
 	}
 
-	if(reload(&conf, configfile, &logfile,
+	if(reload(&conf, configfile,
 	  1 /* first time */,
 	  0 /* no oldmax_children setting */,
 	  0 /* no oldmax_status_children setting */)) return 1;
@@ -265,6 +287,24 @@ int main (int argc, char *argv[])
 		backup="0";
 	}
 
+	if(act==ACTION_DELETE && !backup)
+	{
+		logp("No backup specified for deletion.\n");
+		return 1;
+	}
+
+	if(conf.mode==MODE_CLIENT)
+	{
+		if(orig_client && *orig_client)
+		{
+			if(!(conf.orig_client=strdup(orig_client)))
+			{
+				log_out_of_memory(__FUNCTION__);
+				return 1;
+			}
+		}
+	}
+
 	if(conf.mode==MODE_SERVER
 	  && (act==ACTION_STATUS || act==ACTION_STATUS_SNAPSHOT))
 	{
@@ -306,13 +346,13 @@ int main (int argc, char *argv[])
 		}
 		else
 			ret=server(&conf, configfile,
-				&logfile, generate_ca_only);
+				generate_ca_only);
 #endif
 	}
 	else
 	{
 		logp("before client\n");
-		ret=client(&conf, act);
+		ret=client(&conf, act, vss_restore, json);
 		logp("after client\n");
 	}
 
