@@ -16,16 +16,17 @@ enum action
 	ACTION_STATUS,
 	ACTION_STATUS_SNAPSHOT,
 	ACTION_ESTIMATE,
+	ACTION_DELETE,
 };
 
 #include "find.h"
 #include "log.h"
 
 extern int setup_signals(int oldmax_children, int max_children, int oldmax_status_children, int max_status_children);
-extern int reload(struct config *conf, const char *configfile, char **logfile, bool firsttime, int oldmax_children, int oldmax_status_children);
+extern int reload(struct config *conf, const char *configfile, bool firsttime, int oldmax_children, int oldmax_status_children);
 
-extern int server(struct config *conf, const char *configfile, char **logfile,
+extern int server(struct config *conf, const char *configfile,
 	int generate_ca_only);
-extern int client(struct config *conf, enum action act);
+extern int client(struct config *conf, enum action act, int vss_restore, int json);
 
 #endif // _PROG_H
