@@ -30,7 +30,11 @@ static int restore_interrupt(struct sbuf *sb, const char *msg, struct cntr *cntr
 	// to interrupt the flow and move on.
 	if((sb->cmd!=CMD_FILE
 	   && sb->cmd!=CMD_ENC_FILE
-	   && sb->cmd!=CMD_EFS_FILE)
+	   && sb->cmd!=CMD_EFS_FILE
+	   && sb->cmd!=CMD_VSS
+	   && sb->cmd!=CMD_ENC_VSS
+	   && sb->cmd!=CMD_VSS_T
+	   && sb->cmd!=CMD_ENC_VSS_T)
 	 || !(sb->datapth))
 		return 0;
 
@@ -83,7 +87,7 @@ static int make_link(const char *fname, const char *lnk, char cmd, const char *r
 		char *flnk=NULL;
 		if(!(flnk=prepend_s(restoreprefix, lnk, strlen(lnk))))
 		{
-			logp("out of memory\n");
+			log_out_of_memory(__FUNCTION__);
 			return -1;
 		}
 		//printf("%s -> %s\n", fname, flnk);
@@ -109,17 +113,93 @@ static int make_link(const char *fname, const char *lnk, char cmd, const char *r
 	return ret;
 }
 
-static int restore_file_or_get_meta(struct sbuf *sb, const char *fname, enum action act, const char *encpassword, struct cntr *cntr, char **metadata, size_t *metalen)
+#ifdef HAVE_WIN32
+static int bclose_and_set_attributes(BFILE *bfd, struct cntr *cntr)
+{
+	char *path=NULL;
+	if(bfd->path && !(path=strdup(bfd->path)))
+	{
+		log_out_of_memory(__FUNCTION__);
+		return -1;
+	}
+	if(bclose(bfd))
+	{
+		if(path) free(path);
+		logp("error closing %s in %s()\n", path, __FUNCTION__);
+			return -1;
+	}
+	if(path)
+	{
+		set_attributes(path, '\0', &bfd->statp, bfd->winattr, cntr);
+		free(path);
+	}
+	return 0;
+}
+#endif
+
+static int open_for_restore(BFILE *bfd, FILE **fp, const char *path, struct sbuf *sb, int vss_restore, struct config *conf, struct cntr *cntr)
+{
+#ifdef HAVE_WIN32
+	int bopenret;
+	if(bfd->mode!=BF_CLOSED)
+	{
+		if(bfd->path && !strcmp(bfd->path, path))
+		{
+			// Already open after restoring the VSS data.
+			// Time now for the actual file data.
+			return 0;
+		}
+		else
+		{
+			if(bclose_and_set_attributes(bfd, cntr))
+				return -1;
+		}
+	}
+	binit(bfd, &sb->statp, sb->winattr);
+	if(vss_restore)
+		set_win32_backup(bfd);
+	else
+		bfd->use_backup_api=0;
+	if(S_ISDIR(sb->statp.st_mode))
+	{
+		mkdir(path, 0777);
+		bopenret=bopen(bfd, path, O_WRONLY | O_BINARY, 0, 1);
+	}
+	else
+		bopenret=bopen(bfd, path,
+		  O_WRONLY | O_CREAT | O_TRUNC | O_BINARY,
+		  S_IRUSR | S_IWUSR, 0);
+	if(bopenret<=0)
+	{
+		berrno be;
+		char msg[256]="";
+		snprintf(msg, sizeof(msg),
+			"Could not open for writing %s: %s",
+				path, be.bstrerror(errno));
+		if(restore_interrupt(sb, msg, cntr))
+			return -1;
+	}
+#else
+	if(!(*fp=open_file(path, "wb")))
+	{
+		char msg[256]="";
+		snprintf(msg, sizeof(msg),
+			"Could not open for writing %s: %s",
+				path, strerror(errno));
+		if(restore_interrupt(sb, msg, cntr))
+			return -1;
+	}
+#endif
+	return 0;
+}
+
+static int restore_file_or_get_meta(BFILE *bfd, struct sbuf *sb, const char *fname, enum action act, const char *encpassword, struct cntr *cntr, char **metadata, size_t *metalen, int vss_restore, struct config *conf)
 {
 	size_t len=0;
 	int ret=0;
 	char *rpath=NULL;
-#ifdef HAVE_WIN32
-	int bopenret;
-	BFILE bfd;
-#else
 	FILE *fp=NULL;
-#endif
+
 	if(act==ACTION_VERIFY)
 	{
 		do_filecounter(cntr, sb->cmd, 1);
@@ -136,44 +216,21 @@ static int restore_file_or_get_meta(struct sbuf *sb, const char *fname, enum act
 		goto end;
 	}
 
+#ifndef HAVE_WIN32
+	// We always want to open the file if it is on Windows. Otherwise,
+	// only open it if we are not doing metadata.
 	if(!metadata)
 	{
-#ifdef HAVE_WIN32
-		binit(&bfd, sb->winattr);
-		set_win32_backup(&bfd);
-		if(S_ISDIR(sb->statp.st_mode))
-		{
-			mkdir(rpath, 0777);
-			bopenret=bopen(&bfd, rpath, O_WRONLY | O_BINARY, 0, 1);
-		}
-		else
-			bopenret=bopen(&bfd, rpath,
-			  O_WRONLY | O_CREAT | O_TRUNC | O_BINARY,
-			  S_IRUSR | S_IWUSR, 0);
-		if(bopenret<=0)
-		{
-			berrno be;
-			char msg[256]="";
-			snprintf(msg, sizeof(msg),
-				"Could not open for writing %s: %s",
-					rpath, be.bstrerror(errno));
-			if(restore_interrupt(sb, msg, cntr))
-				ret=-1;
-			goto end;
-		}
-#else
-		if(!(fp=open_file(rpath, "wb")))
-		{
-			char msg[256]="";
-			snprintf(msg, sizeof(msg),
-				"Could not open for writing %s: %s",
-					rpath, strerror(errno));
-			if(restore_interrupt(sb, msg, cntr))
-				ret=-1;
-			goto end;
-		}
 #endif
+		if(open_for_restore(bfd, &fp,
+			rpath, sb, vss_restore, conf, cntr))
+		{
+			ret=-1;
+			goto end;
+		}
+#ifndef HAVE_WIN32
 	}
+#endif
 
 	if(!ret)
 	{
@@ -206,19 +263,28 @@ static int restore_file_or_get_meta(struct sbuf *sb, const char *fname, enum act
 		}
 		else
 		{
+			int c=0;
 #ifdef HAVE_WIN32
-			ret=transfer_gzfile_in(sb, fname, &bfd, NULL,
+			ret=transfer_gzfile_in(sb, fname, bfd, NULL,
 				&rcvdbytes, &sentbytes,
 				encpassword, enccompressed, cntr, NULL);
-			bclose(&bfd);
+			//c=bclose(bfd);
 #else
 			ret=transfer_gzfile_in(sb, fname, NULL, fp,
 				&rcvdbytes, &sentbytes,
 				encpassword, enccompressed, cntr, NULL);
-			close_fp(&fp);
+			c=close_fp(&fp);
 #endif
+			if(c)
+			{
+				logp("error closing %s in restore_file_or_get_meta\n", fname);
+				ret=-1;
+			}
+#ifndef HAVE_WIN32
+// Windows will set attributes when closing the file.
 			if(!ret) set_attributes(rpath, sb->cmd,
 				&(sb->statp), sb->winattr, cntr);
+#endif
 		}
 		if(ret)
 		{
@@ -232,7 +298,6 @@ static int restore_file_or_get_meta(struct sbuf *sb, const char *fname, enum act
 		}
 	}
 	if(!ret) do_filecounter(cntr, sb->cmd, 1);
-
 end:
 	if(rpath) free(rpath);
 	return ret;
@@ -277,12 +342,14 @@ static int restore_special(struct sbuf *sb, const char *fname, enum action act,
 			set_attributes(rpath, CMD_SPECIAL, &statp, sb->winattr, cntr);
 			do_filecounter(cntr, CMD_SPECIAL, 1);
 		}
+/*
 	}
 	else if(S_ISSOCK(statp.st_mode)) {
 		char msg[256]="";
 		snprintf(msg, sizeof(msg),
 			"Skipping restore of socket: %s\n", fname);
 		logw(cntr, "%s", msg);
+*/
 #ifdef S_IFDOOR     // Solaris high speed RPC mechanism
 	} else if (S_ISDOOR(statp.st_mode)) {
 		char msg[256]="";
@@ -333,7 +400,7 @@ static int restore_dir(struct sbuf *sb, const char *dname, enum action act, stru
 				ret=-1;
 			goto end;
 		}
-		else if(!is_dir(rpath))
+		else if(!is_dir_lstat(rpath))
 		{
 			if(mkdir(rpath, 0777))
 			{
@@ -397,7 +464,7 @@ end:
 	return ret;
 }
 
-static int restore_metadata(struct sbuf *sb, const char *fname, enum action act, const char *encpassword, struct cntr *cntr)
+static int restore_metadata(BFILE *bfd, struct sbuf *sb, const char *fname, enum action act, const char *encpassword, int vss_restore, struct config *conf, struct cntr *cntr)
 {
 	// If it is directory metadata, try to make sure the directory
 	// exists. Pass in NULL as the cntr, so no counting is done.
@@ -414,11 +481,13 @@ static int restore_metadata(struct sbuf *sb, const char *fname, enum action act,
 			return -1;
 
 		// Read in the metadata...
-		if(restore_file_or_get_meta(sb, fname, act, encpassword,
-			cntr, &metadata, &metalen)) return -1;
+		if(restore_file_or_get_meta(bfd, sb, fname, act, encpassword,
+			cntr, &metadata, &metalen, vss_restore, conf))
+				return -1;
 		if(metadata)
 		{
-			if(set_extrameta(fname, sb->cmd,
+			
+			if(set_extrameta(bfd, fname, sb->cmd,
 				&(sb->statp), metadata, metalen, cntr))
 			{
 				free(metadata);
@@ -426,12 +495,12 @@ static int restore_metadata(struct sbuf *sb, const char *fname, enum action act,
 				return 0;
 			}
 			free(metadata);
-
+#ifndef HAVE_WIN32
 			// set attributes again, since we just diddled with
 			// the file
 			set_attributes(fname, sb->cmd,
 				&(sb->statp), sb->winattr, cntr);
-
+#endif
 			do_filecounter(cntr, sb->cmd, 1);
 		}
 	}
@@ -501,7 +570,7 @@ static int strip_path_components(struct sbuf *sb, char **path, int strip, struct
 	}
 	if(!(tmp=strdup(cp)))
 	{
-		log_and_send("out of memory");
+		log_and_send_oom(__FUNCTION__);
 		return -1;
 	}
 	free(*path);
@@ -509,15 +578,67 @@ static int strip_path_components(struct sbuf *sb, char **path, int strip, struct
 	return 1;
 }
 
-int do_restore_client(struct config *conf, enum action act, struct cntr *p1cntr, struct cntr *cntr)
+static int overwrite_ok(struct sbuf *sb, struct config *conf, BFILE *bfd, const char *fullpath)
+{
+	struct stat checkstat;
+
+	// User specified overwrite is OK.
+#ifdef HAVE_WIN32
+	if(conf->overwrite) return 1;
+#else
+	// User specified overwrite is OK,
+	// UNLESS we're trying to overwrite the file with the trailing VSS data
+	if(conf->overwrite)
+		return (sb->cmd!=CMD_VSS_T
+			&& sb->cmd!=CMD_ENC_VSS_T);
+#endif
+
+	if(!S_ISDIR(sb->statp.st_mode)
+	  && sb->cmd!=CMD_METADATA
+	  && sb->cmd!=CMD_ENC_METADATA
+	  && sb->cmd!=CMD_VSS
+	  && sb->cmd!=CMD_ENC_VSS)
+	{
+#ifdef HAVE_WIN32
+		// If Windows previously got some VSS data, it needs to append
+		// the file data to the already open bfd.
+		// And trailing VSS data.
+		if(bfd->mode!=BF_CLOSED
+		  && (sb->cmd==CMD_FILE || sb->cmd==CMD_ENC_FILE
+		      || sb->cmd==CMD_VSS_T || sb->cmd==CMD_ENC_VSS_T)
+		  && bfd->path && !strcmp(bfd->path, fullpath))
+		{
+			return 1;
+		}
+#endif
+		// If we have file data and the destination is
+		// a fifo, it is OK to write to the fifo.
+		if((sb->cmd==CMD_FILE || sb->cmd==CMD_ENC_FILE)
+	  	  && S_ISFIFO(sb->statp.st_mode))
+			return 1;
+
+		// File path exists. Do not overwrite.
+		if(!lstat(fullpath, &checkstat)) return 0;
+	}
+
+	return 1;
+}
+
+int do_restore_client(struct config *conf, enum action act, int vss_restore, struct cntr *p1cntr, struct cntr *cntr)
 {
 	int ars=0;
 	int ret=0;
 	int quit=0;
-	char msg[64]="";
-	struct stat checkstat;
+	char msg[512]="";
 	struct sbuf sb;
 	int wroteendcounter=0;
+// Windows needs to have the VSS data written first, and the actual data
+// written immediately afterwards. The server is transferring them in two
+// chunks. So, leave bfd open after a Windows metadata transfer.
+	BFILE bfd;
+#ifdef HAVE_WIN32
+	binit(&bfd, NULL, 0);
+#endif
 
 	logp("doing %s\n", act_str(act));
 
@@ -528,9 +649,14 @@ int do_restore_client(struct config *conf, enum action act, struct cntr *p1cntr,
 		return -1;
 	logp("doing %s confirmed\n", act_str(act));
 
+	if(conf->send_client_counters)
+	{
+		if(recv_counters(p1cntr, cntr))
+			return -1;
+	}
+
 #if defined(HAVE_WIN32)
-	if(act==ACTION_RESTORE)
-		win32_enable_backup_privileges(1 /* ignore_errors */);
+	if(act==ACTION_RESTORE) win32_enable_backup_privileges();
 #endif
 
 	init_sbuf(&sb);
@@ -546,7 +672,7 @@ int do_restore_client(struct config *conf, enum action act, struct cntr *p1cntr,
 			{
 				// ars==1 means it ended ok.
 				print_endcounter(cntr);
-				print_filecounters(p1cntr, cntr, act, 1);
+				print_filecounters(p1cntr, cntr, act);
 				wroteendcounter++;
 				logp("got %s end\n", act_str(act));
 				if(async_write_str(CMD_GEN, "restoreend ok"))
@@ -565,6 +691,10 @@ int do_restore_client(struct config *conf, enum action act, struct cntr *p1cntr,
 			case CMD_SPECIAL:
 			case CMD_METADATA:
 			case CMD_ENC_METADATA:
+			case CMD_VSS:
+			case CMD_ENC_VSS:
+			case CMD_VSS_T:
+			case CMD_ENC_VSS_T:
 			case CMD_EFS_FILE:
 				if(conf->strip)
 				{
@@ -587,23 +717,14 @@ int do_restore_client(struct config *conf, enum action act, struct cntr *p1cntr,
 				if(!(fullpath=prepend_s(conf->restoreprefix,
 					sb.path, strlen(sb.path))))
 				{
-					log_and_send("out of memory");
+					log_and_send_oom(__FUNCTION__);
 					ret=-1;
 					quit++;
 				}
 				if(act==ACTION_RESTORE)
 				{
 				  strip_invalid_characters(&fullpath);
-				  if(!conf->overwrite
-				   && !S_ISDIR(sb.statp.st_mode)
-				   && sb.cmd!=CMD_METADATA
-				   && sb.cmd!=CMD_ENC_METADATA
-				   && !lstat(fullpath, &checkstat)
-				// If we have file data and the destination is
-				// a fifo, it is OK to write to the fifo.
-				   && !((sb.cmd==CMD_FILE
-					  || sb.cmd==CMD_ENC_FILE)
-					&& S_ISFIFO(checkstat.st_mode)))
+				  if(!overwrite_ok(&sb, conf, &bfd, fullpath))
 				  {
 					char msg[512]="";
 					// Something exists at that path.
@@ -641,12 +762,15 @@ int do_restore_client(struct config *conf, enum action act, struct cntr *p1cntr,
 				}
 				break;
 			case CMD_FILE:
+			case CMD_VSS_T:
 				// Have it a separate statement to the
 				// encrypted version so that encrypted and not
 				// encrypted files can be restored at the
 				// same time.
-				if(restore_file_or_get_meta(&sb, fullpath, act,
-					NULL, cntr, NULL, NULL))
+				if(restore_file_or_get_meta(&bfd, &sb,
+					fullpath, act,
+					NULL, cntr, NULL, NULL,
+					vss_restore, conf))
 				{
 					logp("restore_file error\n");
 					ret=-1;
@@ -654,9 +778,11 @@ int do_restore_client(struct config *conf, enum action act, struct cntr *p1cntr,
 				}
 				break;
 			case CMD_ENC_FILE:
-				if(restore_file_or_get_meta(&sb, fullpath, act,
+			case CMD_ENC_VSS_T:
+				if(restore_file_or_get_meta(&bfd, &sb,
+					fullpath, act,
 					conf->encryption_password, cntr,
-					NULL, NULL))
+					NULL, NULL, vss_restore, conf))
 				{
 					logp("restore_file error\n");
 					ret=-1;
@@ -680,31 +806,35 @@ int do_restore_client(struct config *conf, enum action act, struct cntr *p1cntr,
 				}
 				break;
 			case CMD_METADATA:
-				if(restore_metadata(&sb, fullpath, act,
-					NULL, cntr))
+			case CMD_VSS:
+				if(restore_metadata(&bfd, &sb, fullpath, act,
+					NULL, vss_restore, conf, cntr))
 				{
 					ret=-1;
 					quit++;
 				}
 				break;
 			case CMD_ENC_METADATA:
-				if(restore_metadata(&sb, fullpath, act,
-					conf->encryption_password, cntr))
+			case CMD_ENC_VSS:
+				if(restore_metadata(&bfd, &sb, fullpath, act,
+					conf->encryption_password,
+					vss_restore, conf, cntr))
 				{
 					ret=-1;
 					quit++;
 				}
 				break;
 			case CMD_EFS_FILE:
-				if(restore_file_or_get_meta(&sb, fullpath, act,
-					NULL, cntr, NULL, NULL))
+				if(restore_file_or_get_meta(&bfd, &sb,
+					fullpath, act,
+					NULL, cntr,
+					NULL, NULL, vss_restore, conf))
 				{
 					logp("restore_file error\n");
 					ret=-1;
 					quit++;
 				}
 				break;
-				break;
 			default:
 				logp("unknown cmd: %c\n", sb.cmd);
 				quit++; ret=-1;
@@ -715,10 +845,15 @@ int do_restore_client(struct config *conf, enum action act, struct cntr *p1cntr,
 	}
 	free_sbuf(&sb);
 
+#ifdef HAVE_WIN32
+	// It is possible for a bfd to still be open.
+	bclose_and_set_attributes(&bfd, cntr);
+#endif
+
 	if(!wroteendcounter)
 	{
 		print_endcounter(cntr);
-		print_filecounters(p1cntr, cntr, act, 1);
+		print_filecounters(p1cntr, cntr, act);
 	}
 
 	if(!ret) logp("%s finished\n", act_str(act));
