@@ -9,6 +9,7 @@
 #include "counter.h"
 #include "dpth.h"
 #include "sbuf.h"
+#include "regexp.h"
 #include "current_backups_server.h"
 #include "restore_server.h"
 
@@ -63,13 +64,21 @@ int do_patch(const char *dst, const char *del, const char *upd, bool gzupd, int
 	fclose(dstp);
 	gzclose_fp(&delzp);
 	close_fp(&delfp);
-	if(updp) gzclose_fp(&updp);
-	if(updfp) fclose(updfp);
+	if(close_fp(&updfp))
+	{
+		logp("error closing %s after rs_patch_gzfile\n", upd);
+		result=RS_IO_ERROR;
+	}
+	if(gzclose_fp(&updp))
+	{
+		logp("error gzclosing %s after rs_patch_gzfile\n", upd);
+		result=RS_IO_ERROR;
+	}
 
 	return result;
 }
 
-static int inflate_or_link_oldfile(const char *oldpath, const char *infpath, int compression)
+static int inflate_or_link_oldfile(const char *oldpath, const char *infpath, struct config *cconf, int compression)
 {
 	int ret=0;
 	struct stat statp;
@@ -113,17 +122,19 @@ static int inflate_or_link_oldfile(const char *oldpath, const char *infpath, int
 			logp("zlib_inflate returned: %d\n", ret);
 
 		close_fp(&source);
-		close_fp(&dest);
+		if(close_fp(&dest))
+		{
+			logp("error closing %s in inflate_or_link_oldfile\n",
+				dest);
+			return -1;
+		}
 	}
 	else
 	{
 		// Not compressed - just hard link it.
-		if(link(oldpath, infpath))
-		{
-			logp("hardlink %s to %s failed: %s\n",
-				infpath, oldpath, strerror(errno));
-			ret=-1;
-		}
+		if(do_link(oldpath, infpath, &statp, cconf,
+			TRUE /* allow overwrite of infpath */))
+				return -1;
 	}
 	return ret;
 }
@@ -131,8 +142,9 @@ static int inflate_or_link_oldfile(const char *oldpath, const char *infpath, int
 static int send_file(const char *fname, int patches, const char *best, const char *datapth, unsigned long long *bytes, char cmd, int64_t winattr, int compression, struct cntr *cntr, struct config *cconf)
 {
 	int ret=0;
+	size_t datalen=0;
 	FILE *fp=NULL;
-	if(open_file_for_send(NULL, &fp, best, winattr, cntr))
+	if(open_file_for_send(NULL, &fp, best, NULL, winattr, &datalen, cntr))
 		return -1;
 	//logp("sending: %s\n", best);
 	if(async_write(cmd, fname, strlen(fname)))
@@ -142,7 +154,7 @@ static int send_file(const char *fname, int patches, const char *best, const cha
 		// If we did some patches, the resulting file
 		// is not gzipped. Gzip it during the send. 
 		ret=send_whole_file_gz(best, datapth, 1, bytes, NULL, cntr,
-			9, NULL, fp, NULL, 0);
+			9, NULL, fp, NULL, 0, -1);
 	}
 	else
 	{
@@ -151,10 +163,12 @@ static int send_file(const char *fname, int patches, const char *best, const cha
 		// sort it out.
 		if(cmd==CMD_ENC_FILE
 		  || cmd==CMD_ENC_METADATA
+		  || cmd==CMD_ENC_VSS
+		  || cmd==CMD_ENC_VSS_T
 		  || cmd==CMD_EFS_FILE)
 		{
-			ret=send_whole_file(cmd, best,
-				datapth, 1, bytes, cntr, NULL, fp, NULL, 0);
+			ret=send_whole_file(cmd, best, datapth, 1, bytes,
+				cntr, NULL, fp, NULL, 0, -1);
 		}
 		// It might have been stored uncompressed. Gzip it during
 		// the send. If the client knew what kind of file it would be
@@ -162,14 +176,14 @@ static int send_file(const char *fname, int patches, const char *best, const cha
 		else if(!dpth_is_compressed(compression, datapth))
 		{
 			ret=send_whole_file_gz(best, datapth, 1, bytes,
-				NULL, cntr, 9, NULL, fp, NULL, 0);
+				NULL, cntr, 9, NULL, fp, NULL, 0, -1);
 		}
 		else
 		{
 			// If we did not do some patches, the resulting
 			// file might already be gzipped. Send it as it is.
-			ret=send_whole_file(cmd, best,
-				datapth, 1, bytes, cntr, NULL, fp, NULL, 0);
+			ret=send_whole_file(cmd, best, datapth, 1, bytes,
+				cntr, NULL, fp, NULL, 0, -1);
 		}
 	}
 	close_file_for_send(NULL, &fp);
@@ -198,6 +212,7 @@ static int verify_file(const char *fname, int patches, const char *best, const c
 	}
 	if(patches
 	  || cmd==CMD_ENC_FILE || cmd==CMD_ENC_METADATA || cmd==CMD_EFS_FILE
+	  || cmd==CMD_ENC_VSS
 	  || (!patches && !dpth_is_compressed(compression, best)))
 	{
 		// If we did some patches or encryption, or the compression
@@ -286,7 +301,7 @@ static int restore_file(struct bu *arr, int a, int i, const char *datapth, const
 		struct stat statp;
 		if(!(path=prepend_s(arr[x].data, datapth, strlen(datapth))))
 		{
-			log_and_send("out of memory");
+			log_and_send_oom(__FUNCTION__);
 			return -1;
 		}
 
@@ -315,7 +330,7 @@ static int restore_file(struct bu *arr, int a, int i, const char *datapth, const
 				if(!(dpath=prepend_s(arr[x].delta,
 						datapth, strlen(datapth))))
 				{
-					log_and_send("out of memory");
+					log_and_send_oom(__FUNCTION__);
 					free(path);
 					return -1;
 				}
@@ -331,7 +346,7 @@ static int restore_file(struct bu *arr, int a, int i, const char *datapth, const
 				{
 					// Need to gunzip the first one.
 					if(inflate_or_link_oldfile(best, tmp,
-						compression))
+						cconf, compression))
 					{
 						logp("error when inflating %s\n", best);
 						free(path);
@@ -375,7 +390,12 @@ static int restore_file(struct bu *arr, int a, int i, const char *datapth, const
 					free(path);
 					return -1;
 				}
-				else do_filecounter(cntr, cmd, 0);
+				else
+				{
+					do_filecounter(cntr, cmd, 0);
+					do_filecounter_bytes(cntr,
+                 			    strtoull(endfile, NULL, 10));
+				}
 			}
 			else if(act==ACTION_VERIFY)
 			{
@@ -386,16 +406,22 @@ static int restore_file(struct bu *arr, int a, int i, const char *datapth, const
 					free(path);
 					return -1;
 				}
-				else do_filecounter(cntr, cmd, 0);
+				else
+				{
+					do_filecounter(cntr, cmd, 0);
+					do_filecounter_bytes(cntr,
+                 			    strtoull(endfile, NULL, 10));
+				}
 			}
+			do_filecounter_sentbytes(cntr, bytes);
 			free(path);
 			return 0;
 		}
 	}
 
-	snprintf(msg, sizeof(msg), "restore could not find %s (%s)\n", fname, datapth);
-	log_and_send(msg);
-	return -1;
+	logw(cntr, "restore could not find %s (%s)\n", fname, datapth);
+	//return -1;
+	return 0;
 }
 
 static int restore_sbuf(struct sbuf *sb, struct bu *arr, int a, int i, const char *tmppath1, const char *tmppath2, enum action act, const char *client, char status, struct cntr *p1cntr, struct cntr *cntr, struct config *cconf)
@@ -411,6 +437,10 @@ static int restore_sbuf(struct sbuf *sb, struct bu *arr, int a, int i, const cha
 	  || sb->cmd==CMD_ENC_FILE
 	  || sb->cmd==CMD_METADATA
 	  || sb->cmd==CMD_ENC_METADATA
+	  || sb->cmd==CMD_VSS
+	  || sb->cmd==CMD_ENC_VSS
+	  || sb->cmd==CMD_VSS_T
+	  || sb->cmd==CMD_ENC_VSS_T
 	  || sb->cmd==CMD_EFS_FILE)
 	{
 		return restore_file(arr, a, i, sb->datapth,
@@ -433,8 +463,6 @@ static int restore_sbuf(struct sbuf *sb, struct bu *arr, int a, int i, const cha
 	}
 	return 0;
 }
-//	logp("wrote restoreend\n");
-//	if(ret) logp("did not write restoreend (%d)\n", ret);
 
 static int do_restore_end(enum action act, struct cntr *cntr)
 {
@@ -558,7 +586,7 @@ static int check_srestore(struct config *cconf, const char *path)
 
 // a = length of struct bu array
 // i = position to restore from
-static int restore_manifest(struct bu *arr, int a, int i, const char *tmppath1, const char *tmppath2, regex_t *regex, int srestore, enum action act, const char *client, struct cntr *p1cntr, struct cntr *cntr, struct config *cconf, bool all)
+static int restore_manifest(struct bu *arr, int a, int i, const char *tmppath1, const char *tmppath2, regex_t *regex, int srestore, enum action act, const char *client, char **dir_for_notify, struct cntr *p1cntr, struct cntr *cntr, struct config *cconf)
 {
 	int ret=0;
 	gzFile zp=NULL;
@@ -580,10 +608,10 @@ static int restore_manifest(struct bu *arr, int a, int i, const char *tmppath1,
 	 || (act==ACTION_VERIFY && !(logpathz=prepend_s(arr[i].path, "verifylog.gz", strlen("verifylog.gz"))))
 	 || !(manifest=prepend_s(arr[i].path, "manifest.gz", strlen("manifest.gz"))))
 	{
-		log_and_send("out of memory");
+		log_and_send_oom(__FUNCTION__);
 		ret=-1;
 	}
-	else if(!(logfp=open_file(logpath, "ab")) || set_logfp(logfp, cconf))
+	else if(set_logfp(logpath, cconf))
 	{
 		char msg[256]="";
 		snprintf(msg, sizeof(msg),
@@ -592,8 +620,72 @@ static int restore_manifest(struct bu *arr, int a, int i, const char *tmppath1,
 		ret=-1;
 	}
 
+	*dir_for_notify=strdup(arr[i].path);
+
 	log_restore_settings(cconf, srestore);
 
+	// First, do a pass through the manifest to set up the counters.
+	// This is the equivalent of a phase1 scan during backup.
+	if(!ret && !(zp=gzopen_file(manifest, "rb")))
+	{
+		log_and_send("could not open manifest");
+		ret=-1;
+	}
+	else
+	{
+		int ars=0;
+		int quit=0;
+		struct sbuf sb;
+		init_sbuf(&sb);
+		while(!quit)
+		{
+			if((ars=sbuf_fill(NULL, zp, &sb, cntr)))
+			{
+				if(ars<0) ret=-1;
+				// ars==1 means end ok
+				quit++;
+			}
+			else
+			{
+				if((!srestore
+				    || check_srestore(cconf, sb.path))
+				  && check_regex(regex, sb.path))
+				{
+					do_filecounter(p1cntr, sb.cmd, 0);
+					if(sb.endfile)
+					  do_filecounter_bytes(p1cntr,
+                 			    strtoull(sb.endfile, NULL, 10));
+/*
+					if(sb.cmd==CMD_FILE
+					  || sb.cmd==CMD_ENC_FILE
+					  || sb.cmd==CMD_METADATA
+					  || sb.cmd==CMD_ENC_METADATA
+					  || sb.cmd==CMD_VSS
+					  || sb.cmd==CMD_ENC_VSS
+					  || sb.cmd==CMD_VSS_T
+					  || sb.cmd==CMD_ENC_VSS_T
+					  || sb.cmd==CMD_EFS_FILE)
+						do_filecounter_bytes(p1cntr,
+							(unsigned long long)
+							sb.statp.st_size);
+*/
+				}
+			}
+			free_sbuf(&sb);
+		}
+		free_sbuf(&sb);
+		gzclose_fp(&zp);
+	}
+
+	if(cconf->send_client_counters)
+	{
+		if(send_counters(client, p1cntr, cntr))
+		{
+			ret=-1;
+		}
+	}
+
+	// Now, do the actual restore.
 	if(!ret && !(zp=gzopen_file(manifest, "rb")))
 	{
 		log_and_send("could not open manifest");
@@ -686,13 +778,13 @@ static int restore_manifest(struct bu *arr, int a, int i, const char *tmppath1,
 		}
 		free_sbufs(sblist, scount);
 
-		if(!ret && !all) ret=do_restore_end(act, cntr);
+		if(!ret) ret=do_restore_end(act, cntr);
 
-		print_endcounter(cntr);
-		print_filecounters(p1cntr, cntr, act, 0);
+		//print_endcounter(cntr);
+		print_filecounters(p1cntr, cntr, act);
 
-		reset_filecounter(p1cntr);
-		reset_filecounter(cntr);
+		reset_filecounter(p1cntr, time(NULL));
+		reset_filecounter(cntr, time(NULL));
 	}
 	set_logfp(NULL, cconf);
 	compress_file(logpath, logpathz, cconf);
@@ -703,7 +795,7 @@ static int restore_manifest(struct bu *arr, int a, int i, const char *tmppath1,
 	return ret;
 }
 
-int do_restore_server(const char *basedir, enum action act, const char *client, int srestore, struct cntr *p1cntr, struct cntr *cntr, struct config *cconf)
+int do_restore_server(const char *basedir, enum action act, const char *client, int srestore, char **dir_for_notify, struct cntr *p1cntr, struct cntr *cntr, struct config *cconf)
 {
 	int a=0;
 	int i=0;
@@ -714,7 +806,6 @@ int do_restore_server(const char *basedir, enum action act, const char *client,
 	char *tmppath1=NULL;
 	char *tmppath2=NULL;
 	regex_t *regex=NULL;
-	bool all=FALSE;
 
 	logp("in do_restore\n");
 
@@ -736,38 +827,31 @@ int do_restore_server(const char *basedir, enum action act, const char *client,
 		return -1;
 	}
 
-	if(cconf->backup && *(cconf->backup)=='a')
-	{
-		all=TRUE;
-	}
-	else if(!(index=strtoul(cconf->backup, NULL, 10)) && a>0)
+	if(!(index=strtoul(cconf->backup, NULL, 10)) && a>0)
 	{
 		// No backup specified, do the most recent.
 		ret=restore_manifest(arr, a, a-1,
 			tmppath1, tmppath2, regex, srestore, act, client,
-			p1cntr, cntr, cconf, all);
+			dir_for_notify,
+			p1cntr, cntr, cconf);
 		found=TRUE;
 	}
 
 	if(!found) for(i=0; i<a; i++)
 	{
-		if(all || !strcmp(arr[i].timestamp, cconf->backup)
+		if(!strcmp(arr[i].timestamp, cconf->backup)
 			|| arr[i].index==index)
 		{
 			found=TRUE;
 			//logp("got: %s\n", arr[i].path);
 			ret|=restore_manifest(arr, a, i,
 				tmppath1, tmppath2, regex,
-				srestore, act, client,
-				p1cntr, cntr, cconf, all);
-			if(!all) break;
+				srestore, act, client, dir_for_notify,
+				p1cntr, cntr, cconf);
+			break;
 		}
 	}
 
-	// If doing all backups, send restore end.
-	if(!ret && all && found)
-		ret=do_restore_end(act, cntr);
-
 	free_current_backups(&arr, a);
 
 	if(!found)
