@@ -84,7 +84,7 @@ static int do_sbuf_fill_from_net(struct sbuf *sb, struct cntr *cntr)
 	if((ars=async_read(&(sb->cmd), &(sb->path), &(sb->plen)))) return ars;
 	if(sbuf_is_link(sb))
 	{
-		char cmd;
+		char cmd=0;
 		if((ars=async_read(&cmd, &(sb->linkto), &(sb->llen))))
 				return ars;
 		if(!cmd_is_link(cmd))
@@ -123,6 +123,10 @@ static int do_sbuf_fill_from_file(FILE *fp, gzFile zp, struct sbuf *sb, int phas
 			|| sb->cmd==CMD_ENC_FILE
 			|| sb->cmd==CMD_METADATA
 			|| sb->cmd==CMD_ENC_METADATA
+			|| sb->cmd==CMD_VSS
+			|| sb->cmd==CMD_ENC_VSS
+			|| sb->cmd==CMD_VSS_T
+			|| sb->cmd==CMD_ENC_VSS_T
 			|| sb->cmd==CMD_EFS_FILE))
 	{
 		char cmd;
@@ -167,6 +171,10 @@ static int sbuf_to_fp(struct sbuf *sb, FILE *mp, int write_endfile)
 		  || sb->cmd==CMD_ENC_FILE
 		  || sb->cmd==CMD_METADATA
 		  || sb->cmd==CMD_ENC_METADATA
+		  || sb->cmd==CMD_VSS
+		  || sb->cmd==CMD_ENC_VSS
+		  || sb->cmd==CMD_VSS_T
+		  || sb->cmd==CMD_ENC_VSS_T
 		  || sb->cmd==CMD_EFS_FILE))
 		{
 			if(send_msg_fp(mp, CMD_END_FILE,
@@ -193,6 +201,10 @@ static int sbuf_to_zp(struct sbuf *sb, gzFile zp, int write_endfile)
 		  || sb->cmd==CMD_ENC_FILE
 		  || sb->cmd==CMD_METADATA
 		  || sb->cmd==CMD_ENC_METADATA
+		  || sb->cmd==CMD_VSS
+		  || sb->cmd==CMD_ENC_VSS
+		  || sb->cmd==CMD_VSS_T
+		  || sb->cmd==CMD_ENC_VSS_T
 		  || sb->cmd==CMD_EFS_FILE))
 		{
 			if(send_msg_zp(zp, CMD_END_FILE,
@@ -233,13 +245,13 @@ int add_to_sbuf_arr(struct sbuf ***sblist, struct sbuf *sb, int *count)
         if(!(sbtmp=(struct sbuf **)realloc(*sblist,
                 ((*count)+1)*sizeof(struct sbuf *))))
         {
-                logp("out of memory in add_to_sbuf_arr()\n");
+                log_out_of_memory(__FUNCTION__);
                 return -1;
         }
         *sblist=sbtmp;
 	if(!(sbnew=(struct sbuf *)malloc(sizeof(struct sbuf))))
 	{
-		logp("out of memory in add_to_sbuf_arr()\n");
+                log_out_of_memory(__FUNCTION__);
 		return -1;
 	}
 	memcpy(sbnew, sb, sizeof(struct sbuf));
@@ -272,7 +284,7 @@ int del_from_sbuf_arr(struct sbuf ***sblist, int *count)
         if(*count && !(sbtmp=(struct sbuf **)realloc(*sblist,
                 (*count)*sizeof(struct sbuf *))))
         {
-                logp("out of memory in del_from_sbuf_arr()\n");
+                log_out_of_memory(__FUNCTION__);
                 return -1;
         }
         *sblist=sbtmp;
@@ -284,7 +296,7 @@ int del_from_sbuf_arr(struct sbuf ***sblist, int *count)
 }
 
 // Like pathcmp, but sort entries that have the same paths so that metadata
-// comes later.
+// comes later, and vss comes earlier, and trailing vss comes later.
 int sbuf_pathcmp(struct sbuf *a, struct sbuf *b)
 {
 	int r;
@@ -294,9 +306,21 @@ int sbuf_pathcmp(struct sbuf *a, struct sbuf *b)
 		if(b->cmd==CMD_METADATA || b->cmd==CMD_ENC_METADATA) return 0;
 		else return 1;
 	}
+	else if(a->cmd==CMD_VSS || a->cmd==CMD_ENC_VSS)
+	{
+		if(b->cmd==CMD_VSS || b->cmd==CMD_ENC_VSS) return 0;
+		else return -1;
+	}
+	else if(a->cmd==CMD_VSS_T || a->cmd==CMD_ENC_VSS_T)
+	{
+		if(b->cmd==CMD_VSS_T || b->cmd==CMD_ENC_VSS_T) return 0;
+		else return 1;
+	}
 	else
 	{
 		if(b->cmd==CMD_METADATA || b->cmd==CMD_ENC_METADATA) return -1;
+		else if(b->cmd==CMD_VSS || b->cmd==CMD_ENC_VSS) return 1;
+		else if(b->cmd==CMD_VSS_T || b->cmd==CMD_ENC_VSS_T) return -1;
 		else return 0;
 	}
 }
