@@ -110,6 +110,10 @@ SSL_CTX *ssl_initialise_ctx(struct config *conf)
 
 	if(ssl_load_keys_and_certs(ctx, conf)) return NULL;
 
+	if(conf->ssl_ciphers) {
+		SSL_CTX_set_cipher_list(ctx, conf->ssl_ciphers);
+	}
+
 	return ctx;
 }
 
@@ -162,7 +166,7 @@ static int setenv_x509(X509_NAME *x509, const char *type)
 		name_expand_size = 64 + strlen (objbuf);
 		if(!(name_expand=(char *)malloc(name_expand_size)))
 		{
-			logp("out of memory\n");
+			log_out_of_memory(__FUNCTION__);
 			return -1;
 		}
 		snprintf(name_expand, name_expand_size,
@@ -183,7 +187,7 @@ static int setenv_x509_date(ASN1_TIME *tm, const char *env)
 	char tmpbuf[256]="";
 	if(!(bio_out=BIO_new(BIO_s_mem())))
 	{
-		logp("out of memory\n");
+		log_out_of_memory(__FUNCTION__);
 		return -1;
 	}
 	ASN1_TIME_print(bio_out, tm);
@@ -202,7 +206,7 @@ static int setenv_x509_serialnumber(ASN1_INTEGER *i, const char *env)
 	char tmpbuf[256]="";
 	if(!(bio_out=BIO_new(BIO_s_mem())))
 	{
-		logp("out of memory\n");
+		log_out_of_memory(__FUNCTION__);
 		return -1;
 	}
 	i2a_ASN1_INTEGER(bio_out, i);
@@ -225,7 +229,9 @@ int ssl_check_cert(SSL *ssl, struct config *conf)
 		logp("ssl_peer_cn not set.\n");
 		return -1;
 	}
-
+	logp("Client uses %s %s\n",
+		SSL_CIPHER_get_version(SSL_get_current_cipher(ssl)),
+		SSL_get_cipher_name(ssl));
 	if(!(peer=SSL_get_peer_certificate(ssl)))
 	{
 		logp("Could not get peer certificate.\n");
