@@ -48,7 +48,7 @@ static int cstat_add_initial_details(struct cstat *c, const char *name, const ch
 	if(!(c->conffile=prepend_s(clientconfdir, name, strlen(name)))
 	  || !(c->name=strdup(name)))
 	{
-		logp("out of memory in cstat_add_initial_details()\n");
+		log_out_of_memory(__FUNCTION__);
 		return -1;
 	}
 	c->conf_mtime=0;
@@ -93,13 +93,13 @@ static int cstat_add(struct cstat ***clist, int *clen,
 	if(!(ctmp=(struct cstat **)realloc(*clist,
 		((*clen)+1)*sizeof(struct cstat *))))
 	{
-		logp("out of memory in cstat_add()\n");
+		log_out_of_memory(__FUNCTION__);
 		return -1;
 	}
 	*clist=ctmp;
 	if(!(cnew=(struct cstat *)malloc(sizeof(struct cstat))))
 	{
-		logp("out of memory in cstat_add()\n");
+		log_out_of_memory(__FUNCTION__);
 		return -1;
 	}
 	if(cstat_add_initial_details(cnew, name, clientconfdir))
@@ -130,6 +130,11 @@ static void cstat_blank(struct cstat *c)
 static int set_cstat_from_conf(struct cstat *c, struct config *conf, struct config *cconf)
 {
 	char *lockbasedir=NULL;
+	char *client_lockdir=NULL;
+
+	if(!(client_lockdir=conf->client_lockdir))
+		client_lockdir=cconf->directory;
+
 	if(c->basedir) { free(c->basedir); c->basedir=NULL; }
 	if(c->working) { free(c->working); c->working=NULL; }
 	if(c->current) { free(c->current); c->current=NULL; }
@@ -139,11 +144,11 @@ static int set_cstat_from_conf(struct cstat *c, struct config *conf, struct conf
 	  || !(c->working=prepend_s(c->basedir, "working", strlen("working")))
 	  || !(c->current=prepend_s(c->basedir, "current", strlen("current")))
 	  || !(c->timestamp=prepend_s(c->current, "timestamp", strlen("timestamp")))
-	  || !(lockbasedir=prepend_s(conf->client_lockdir, c->name, strlen(c->name)))
+	  || !(lockbasedir=prepend_s(client_lockdir, c->name, strlen(c->name)))
 	  || !(c->lockfile=prepend_s(lockbasedir, "lockfile", strlen("lockfile"))))
 	{
 		if(lockbasedir) free(lockbasedir);
-		logp("out of memory\n");
+		log_out_of_memory(__FUNCTION__);
 		return -1;
 	}
 	c->basedir_mtime=0;
@@ -172,7 +177,7 @@ static char *get_last_backup_time(const char *timestamp)
 	snprintf(ret, sizeof(ret), "0");
 	if(read_timestamp(timestamp, wbuf, sizeof(wbuf))) return ret;
 
-	snprintf(ret, sizeof(ret), "%lu %li", atol(wbuf),
+	snprintf(ret, sizeof(ret), "%lu 0 %li", atol(wbuf),
 		(long)timestamp_to_long(wbuf));
 	  
 	return ret;
@@ -189,17 +194,16 @@ static int set_summary(struct cstat *c)
 		if(lstat(c->working, &statp))
 		{
 			c->status=STATUS_IDLE;
-			snprintf(wbuf, sizeof(wbuf), "%s\t%c\t%s\n",
-				c->name,
-				c->status,
+			snprintf(wbuf, sizeof(wbuf), "%s\t%c\t%c\t%s\n",
+				c->name, COUNTER_VERSION_2, c->status,
 				get_last_backup_time(c->timestamp));
 		}
 		else
 		{
 			// client process crashed
 			c->status=STATUS_CLIENT_CRASHED;
-			snprintf(wbuf, sizeof(wbuf), "%s\t%c\t%s\n",
-				c->name, c->status,
+			snprintf(wbuf, sizeof(wbuf), "%s\t%c\t%c\t%s\n",
+				c->name, COUNTER_VERSION_2, c->status,
 				get_last_backup_time(c->timestamp));
 			//	statp.st_ctime);
 		}
@@ -219,8 +223,8 @@ static int set_summary(struct cstat *c)
 			//if(!lstat(c->working, &statp)) t=statp.st_ctime;
 			// server process crashed
 			c->status=STATUS_SERVER_CRASHED;
-			snprintf(wbuf, sizeof(wbuf), "%s\t%c\t%s\n",
-				c->name, c->status,
+			snprintf(wbuf, sizeof(wbuf), "%s\t%c\t%c\t%s\n",
+				c->name, COUNTER_VERSION_2, c->status,
 				get_last_backup_time(c->timestamp));
 			// It is not running, so free the running_detail.
 			if(c->running_detail)
@@ -233,9 +237,8 @@ static int set_summary(struct cstat *c)
 		{
 			// running normally
 			c->status=STATUS_RUNNING;
-			snprintf(wbuf, sizeof(wbuf), "%s\t%c\t%s\n",
-				c->name,
-				c->status,
+			snprintf(wbuf, sizeof(wbuf), "%s\t%c\t%c\t%s\n",
+				c->name, COUNTER_VERSION_2, c->status,
 				get_last_backup_time(c->timestamp));
 		}
 		if(prog) free(prog);
@@ -247,17 +250,6 @@ static int set_summary(struct cstat *c)
 	return 0;
 }
 
-static int looks_like_vim_tmpfile(const char *filename)
-{
-	const char *cp=NULL;
-	// vim tmpfiles look like ".filename.swp".
-	if(filename[0]=='.'
-	  && (cp=strrchr(filename, '.'))
-	  && !strcmp(cp, ".swp"))
-		return 1;
-	return 0;
-}
-
 static int load_data_from_disk(struct config *conf, struct cstat ***clist, int *clen)
 {
 	int q=0;
@@ -278,9 +270,8 @@ static int load_data_from_disk(struct config *conf, struct cstat ***clist, int *
         for(m=0; m<n; m++)
 	{
 		if(dir[m]->d_ino==0
-		  || !strcmp(dir[m]->d_name, ".")
-		  || !strcmp(dir[m]->d_name, "..")
-		  || looks_like_vim_tmpfile(dir[m]->d_name))
+		// looks_like...() also avoids '.' and '..'.
+		  || looks_like_tmp_or_hidden_file(dir[m]->d_name))
 			continue;
 		for(q=0; q<*clen; q++)
 		{
@@ -427,7 +418,8 @@ static int send_data_to_client(int cfd, const char *data, size_t len)
 		{
 			if(errno!=EAGAIN && errno!=EINTR)
 			{
-				logp("select error: %s\n", strerror(errno));
+				logp("select error in %s: %s\n", __func__,
+					strerror(errno));
 				ret=-1;
 				break;
 			}
@@ -517,20 +509,21 @@ static int send_summaries_to_client(int cfd, struct cstat **clist, int clen, con
 				// make more than enough room for the message
 				len+=strlen(clist[q]->name)+1;
 				len+=(a*2)+1;
-				len+=(a*16)+1;
+				len+=(a*32)+1;
 				if(!(curback=(char *)malloc(len)))
 				{
-					logp("out of memory");
+					log_out_of_memory(__FUNCTION__);
 					return -1;
 				}
-				snprintf(curback, len, "%s\t%c",
-					clist[q]->name, clist[q]->status);
+				snprintf(curback, len, "%s\t%c\t%c",
+					clist[q]->name, COUNTER_VERSION_2,
+					clist[q]->status);
 				for(i=a-1; i>=0; i--)
 				{
-					char tmp[16]="";
+					char tmp[32]="";
 					t=timestamp_to_long(arr[i].timestamp);
-					snprintf(tmp, sizeof(tmp), "\t%lu %li",
-						arr[i].index, (long)t);
+					snprintf(tmp, sizeof(tmp), "\t%lu %d %li",
+						arr[i].index, arr[i].deletable, (long)t);
 					strcat(curback, tmp);
 				}
 				if(!a) strcat(curback, "\t0");
@@ -543,6 +536,7 @@ static int send_summaries_to_client(int cfd, struct cstat **clist, int clen, con
 				clist[q]->summary=curback;
 				tosend=clist[q]->summary;
 			}
+			else tosend=clist[q]->summary;
 		}
 		else tosend=clist[q]->summary;
 
@@ -604,7 +598,7 @@ static int parse_parent_data_entry(char *tok, struct cstat **clist, int clen)
 			// Need to add the newline back on the end.
 			if(!(clist[q]->running_detail=(char *)malloc(x+2)))
 			{
-				logp("out of memory\n");
+				log_out_of_memory(__FUNCTION__);
 				return -1;
 			}
 			snprintf(clist[q]->running_detail, x+2, "%s\n",
@@ -622,7 +616,7 @@ static int parse_parent_data(const char *data, struct cstat **clist, int clen)
 
 	if(!(copyall=strdup(data)))
 	{
-		logp("out of memory\n");
+		log_out_of_memory(__FUNCTION__);
 		return -1;
 	}
 
@@ -685,8 +679,8 @@ static int browse_manifest(int cfd, gzFile zp, const char *browse)
 	struct sbuf sb;
 	struct cntr cntr;
 	size_t blen=0;
-	char *lastpath=NULL;
-	reset_filecounter(&cntr);
+	char *last_bd_match=NULL;
+	reset_filecounter(&cntr, time(NULL));
 	init_sbuf(&sb);
 	if(browse) blen=strlen(browse);
 	while(1)
@@ -699,7 +693,17 @@ static int browse_manifest(int cfd, gzFile zp, const char *browse)
 			// ars==1 means it ended ok.
 			break;
 		}
-		if((r=check_browsedir(browse, &sb.path, blen, &lastpath))<0)
+
+		if(sb.cmd!=CMD_DIRECTORY
+		  && sb.cmd!=CMD_FILE
+		  && sb.cmd!=CMD_ENC_FILE
+		  && sb.cmd!=CMD_EFS_FILE
+		  && sb.cmd!=CMD_SPECIAL
+		  && !cmd_is_link(sb.cmd))
+			continue;
+
+		if((r=check_browsedir(browse,
+			&sb.path, blen, &last_bd_match))<0)
 		{
 			ret=-1;
 			break;
@@ -716,7 +720,7 @@ static int browse_manifest(int cfd, gzFile zp, const char *browse)
 		}
 	}
 	free_sbuf(&sb);
-	if(lastpath) free(lastpath);
+	if(last_bd_match) free(last_bd_match);
 	return ret;
 }
 
@@ -867,7 +871,7 @@ static int parse_rbuf(const char *rbuf, int cfd, struct cstat **clist, int clen)
 		if(file) free(file);
 		if(!(file=strdup("manifest.gz")))
 		{
-			logp("out of memory\n");
+			log_out_of_memory(__FUNCTION__);
 			ret=-1;
 			goto end;
 		}
@@ -1000,7 +1004,8 @@ int status_server(int *cfd, struct config *conf)
 		{
 			if(errno!=EAGAIN && errno!=EINTR)
 			{
-				logp("select error: %s\n", strerror(errno));
+				logp("select error in %s: %s\n", __func__,
+					strerror(errno));
 				ret=-1;
 				break;
 			}
