@@ -1,12 +1,19 @@
 #ifndef _STRLIST_H
 #define _STRLIST_H
 
+#if defined(HAVE_PCREPOSIX)
+#include <pcreposix.h>
+#else
+#include <regex.h>
+#endif
+
 typedef struct strlist strlist_t;
 
 struct strlist
 {
 	long flag;
 	char *path;
+	regex_t *re;
 };
 
 extern void strlists_free(struct strlist **bd, int count);
